      subroutine prd_2d_pr_hs 
     +
     +  (NSG
     +  ,RL
     +  ,ICH1,THMAX
     +  ,ICH2,SPMAX
     +  ,ICH3,SPMIN
     +  ,ICH4,thrs1,thrs2
     +  ,ARCT,ARET
     +  ,XCNT,YCNT
     +  ,P1,P2,P3
     +  ,P4,P5
     +  ,Istop
     +  )

c===========================================
c FDLIB, BEMLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c===========================================

c------------------------------------------------
c  point redistibution on a periodic line with
c  period RL
c
c  point NSG+1 is the periodic repetition of point 1
c
c  the line is symmetric with respect to
c  x at the point J=1 and J=NSGM=NSG/2+1
c
c  CHECK FOR MAXIMUM ANGLE, 
c                    MAXIMUM POINT SEPARATION,
c                    MINIMUM POINT SEPARATION,
c
c  RESET THE PERIODIC LINE SO THAT THE FIRST POINT
c  IS WITHIN THE WINDON (-thrs1, thrs2)
c
c  RETURN AN IMPROVED POINT DISTRIBUTION
c
c  INTERPOLATES FOR PROPERTY VARIABLES P1-P5
c
c            P1:  x-vel
c            P2:  s-vel
c            P3:  surface tension
c            P4:  arc length
c            P5:  unused
c----------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension  X(0:200), Y(0:200)
      Dimension P1(0:200),P2(0:200)
      Dimension P3(0:200)
      Dimension P4(0:200),P5(0:200)

      Dimension    XC(200),   YC(200),  R(200),   S(200)
      Dimension   TH1(200),  TH2(200),TH3(200),ORNT(200)
      Dimension AREA1(200),AREA2(200),ARE(200)
      Dimension  XCN1(200), XCN2(200),XCN(200)
      Dimension  YCN1(200), YCN2(200),YCN(200)

      Parameter (Nmax=200)

      common/XXYY/X,Y
      common/ARCC/XC,YC,R,S,TH1,TH2,TH3,ORNT

c---
c  constants
c---

      pi  = 3.14159 265358 97932384D0
      pih = 0.5D0*pi

c---
c prepare
c---

      Isafe = 100*NSG

      arclt = P4(NSG+1)-P4(1)

      Ipass = 0

c------------

  98  Continue

      Ipass = Ipass+1

      if(NSG.GE.Nmax) then
        write (6,*)
        write (6,*) " more than ",Nmax," points "
        write (6,*) " I will stop "
        Istop = 1
        return
      end if

      if(Ipass.gt.Isafe) then

        write (6,*)
        write (6,*) " Caught in a loop redistributing points "
        write (6,*) " I will stop "
        Istop = 1
        return

      end if

c---
c prepare more
c---

      NSG1  = NSG+1
      NSG2  = NSG+2

      NSGM  = NSG/2+1
      NSGMa = NSGM-1
      NSGMb = NSGM-2
      NSGMc = NSGM-3

c---
c reflect across the plane of symmetry
c---

      Do I=1,NSGM

       IP = NSGM+I
       IM = NSGM-I
        X(IP) = - X(IM)+2.0*X (NSGM)
        Y(IP) =   Y(IM)
       P1(IP) =  P1(IM)
       P2(IP) =  P2(IM)
       P3(IP) =  P3(IM)
       P4(IP) = -P4(IM)+2.0*P4(NSGM)
       P5(IP) =  P5(IM)

      End Do

c---
c wrap around
c---

       X(0) = X (NSG)-RL
       Y(0) = Y (NSG)
      P1(0) = P1(NSG)
      P2(0) = P2(NSG)
      P3(0) = P3(NSG)
      P4(0) = P4(NSG)-arclt
      P5(0) = P5(NSG)

       X(NSG1) =  X(1)  +RL
       Y(NSG1) =  Y(1)
      P1(NSG1) = P1(1)
      P2(NSG1) = P2(1)
      P3(NSG1) = P3(1)
      P4(NSG1) = P4(1)+arclt
      P5(NSG1) = P5(1)

       X(NSG2) =  X(2)  +RL
       Y(NSG2) =  Y(2)
      P1(NSG2) = P1(2)
      P2(NSG2) = P2(2)
      P3(NSG2) = P3(2)
      P4(NSG2) = P4(2)+arclt
      P5(NSG2) = P5(2)

      Istop = 0
      Iloop = 1

c------------------
c  DEFINE THE ARCS
c------------------

      Do 3 I = Iloop,NSG1

        M   = I-2
        L   = I-1
        K   = I+1
        XX1 = X(L)
        YY1 = Y(L)
        XX2 = X(I)
        YY2 = Y(I)
        XX3 = X(K)
        YY3 = Y(K)

        call arc_2d 
     +
     +    (I
     +    ,XX1,XX2,XX3
     +    ,YY1,YY2,YY3
     +    ,XC(I),YC(I),R(I)
     +    ,TH1(I),TH2(I),TH3(I),ORNT(I)
     +    ,AREA1(I),AREA2(I),ARE(I)
     +    ,XCN1(I),XCN2(I),XCN(I)
     +    ,YCN1(I),YCN2(I),YCN(I)
     +    ,Istop
     +    )

      if(I.gt.NSGM) Go to 3

c--------------------------------------
c CHECK FOR ANGLES SUBTENDED BY THE ARCS
c--------------------------------------
 
      if(Ich1.eq.1) then

        THETOT  = TH3(I)-TH1(I)
        Dth     = Dabs(THETOT)
        iF(Dth.ge.pih  ) Go to 96
        iF(Dth.ge.THMAX) Go to 97

      end if

c----------------------------------
c CHECK FOR MAXIMUM POINT SEPARATION
c----------------------------------

      if(Ich2.eq.1.and.I.gt.1) then

        SP1 = R(I)*Dabs(TH2(I)-TH1(I))
        if(SP1.gt.SPMAX) Go to 91

c       SP2 = R(I)*ABS(TH3(I)-TH2(I))       ! optional
c       IF(SP2.GT.SPMAX.and.I.ne.NSGM) GO TO 92

      end if

c-----------------------------------
c CHECK FOR MINIMUM POINT SEPARATION
c-----------------------------------

      if(Ich3.EQ.0) Go to 3
      if(I.EQ.NSGM) Go to 3

      SP2 = R(I)*ABS(TH3(I)-TH2(I))
      if(SP2.GT.SPMIN) Go to 3
 
c---------------------------------------------------------
C  A POINT WILL BE REMOVED ONLY IF THE RESULTING ANGLES
C  AND POINT SEPARATIONS DO NOT EXCEED THE PRE-ESTABLISHED
C  MAXIMA
c---------------------------------------------------------

c---
      if(I.eq.1) then
c---
c  write (6,*) " Checking arc 1"
c  Point 1 will stay, 
c  Point 2 may be removed
c---

         XTMP = 2.0D0*X(1)-X(3)
         YTMP =            Y(3)

         call arc_2d 
     +
     +       (I
     +       ,XTMP,X(1),X(3)
     +       ,YTMP,Y(1),Y(3)
     +       ,XCT,YCT,RT
     +       ,TH1T,TH2T,TH3T,ORNTT
     +       ,AREA1T,AREA2T,ARET
     +       ,XCN1T,XCN2T,XCNT
     +       ,YCN1T,YCN2T,YCNT
     +       ,Istop)

         TOTANG0 = ABS(TH3T-TH1T)

         call arc_2d 
     +
     +        (I
     +        ,X(1),X(3),X(4)
     +        ,Y(1),Y(3),Y(4)
     +        ,XCT,YCT,RT
     +        ,TH1T,TH2T,TH3T,ORNTT
     +        ,AREA1T,AREA2T,ARET
     +        ,XCN1T,XCN2T,XCNT
     +        ,YCN1T,YCN2T,YCNT
     +        ,Istop)

         TOTANG1 = ABS(TH3T-TH1T)

         SEP1    = RT * ABS(TH2T-TH1T)

         if( TOTANG0.LT.THMAX.and.
     +       TOTANG1.LT.THMAX.and.
     +          SEP1.LT.SPMAX)
     +   Go to 93

c---
      else if(I.eq.NSGMa) then
c---
c  Point NSGM will stay
c  Point NSGM-1 may be removed
c---

         call arc_2d 
     +
     +     (I
     +     ,X(NSGMc),X(NSGMb),X(NSGM)
     +     ,Y(NSGMc),Y(NSGMb),Y(NSGM)
     +     ,XCT,YCT,RT
     +     ,TH1T,TH2T,TH3T,ORNTT
     +     ,AREA1T,AREA2T,ARET
     +     ,XCN1T,XCN2T,XCNT
     +     ,YCN1T,YCN2T,YCNT
     +     ,Istop
     +     )

         TOTANG0 = ABS(TH3T-TH1T)

         XTMP = 2.0*X(NSGM)-X(NSGMb)
         YTMP =             Y(NSGMb)

         call arc_2d 
     +
     +    (I
     +    ,X(NSGMb),X(NSGM),XTMP
     +    ,Y(NSGMb),Y(NSGM),YTMP
     +    ,XCT,YCT,RT
     +    ,TH1T,TH2T,TH3T,ORNTT
     +    ,AREA1T,AREA2T,ARET
     +    ,XCN1T,XCN2T,XCNT
     +    ,YCN1T,YCN2T,YCNT
     +    ,Istop
     +    )

         TOTANG1 = ABS(TH3T-TH1T)

         SEP1    = RT * ABS(TH2T-TH1T)

         if( TOTANG0.lt.THMAX.and.
     +       TOTANG1.lt.THMAX.and.
     +          SEP1.lt.SPMAX)
     +   Go to 93
c---
      else
c---
         ANGL = 0.5*(TH2(I)+TH3(I))
         XTMP = XC(I) + R(I) * DCOS(ANGL)
         YTMP = YC(I) + R(I) * DSIN(ANGL)

         if(I.eq.2) then
           Xim = 2.0*X(1)-XTMP
           Yim =          YTMP
         else
           Xim = X(M)
           Yim = Y(M)
         end if

         call arc_2d 
     +
     +       (I
     +       ,Xim,X(L),XTMP
     +       ,Yim,Y(L),YTMP
     +       ,XCT,YCT,RT
     +       ,TH1T,TH2T,TH3T,ORNTT
     +       ,AREA1T,AREA2T,ARET
     +       ,XCN1T,XCN2T,XCNT
     +       ,YCN1T,YCN2T,YCNT
     +       ,Istop
     +       )

         TOTANG0 = ABS(TH3T-TH1T)

         call arc_2d 
     +
     +       (I
     +       ,X(L),XTMP,X(I+2)
     +       ,Y(L),YTMP,Y(I+2)
     +       ,XCT,YCT,RT
     +       ,TH1T,TH2T,TH3T,ORNTT
     +       ,AREA1T,AREA2T,ARET
     +       ,XCN1T,XCN2T,XCNT
     +       ,YCN1T,YCN2T,YCNT
     +       ,Istop
     +       )

         TOTANG1= ABS(TH3T-TH1T)

         SEP1   = RT * ABS(TH2T-TH1T)
         SEP2   = RT * ABS(TH3T-TH2T)

         if(I.eq.NSGMb) then
           Xim = 2.0*X(NSGM)-XTMP
           Yim =             YTMP
         else
           Xim = X(I+3)
           Yim = Y(I+3)
         end if

         call arc_2d 
     +
     +    (I
     +    ,XTMP,X(I+2),Xim
     +    ,YTMP,Y(I+2),Yim
     +    ,XCT,YCT,RT
     +    ,TH1T,TH2T,TH3T,ORNTT
     +    ,AREA1T,AREA2T,ARET
     +    ,XCN1T,XCN2T,XCNT
     +    ,YCN1T,YCN2T,YCNT
     +    ,Istop
     +    )

         TOTANG2 = ABS(TH3T-TH1T)

         if(TOTANG0.lt.THMAX.and.
     +      TOTANG1.lt.THMAX.and.
     +      TOTANG2.lt.THMAX.and.
     +         SEP1.lt.SPMAX.and.
     +         SEP2.lt.SPMAX)
     +   Go to 93
c---
      end if
c---

  3   Continue

c----------------------------------
c  ALL CHECKS DONE
c  WE NOW HAVE 
c  AN ACCEPTABLE POINT DISTRIBUTION
c----------------------------------

      GO TO 29

c--------------------
c  THERE IS A PROBLEM
c  execution will stop
c--------------------

  96  Continue

      write (6,*)
      write (6,200) I,THETOT,THMAX
      write (6,*)
      write (6,201)

      Istop = 1
      return

c---

  97  CONTINUE

c-----------------------------------
c  IF AN ARC IS TOO LARGE, 
c  REMOVE MIDDLE POINT,
c  AND ADD TWO POINTS ALONG THE ARC
c  Except for points 1 and NSGM
c-----------------------------------

      write (6,200) I,Dth,THMAX

c---
      if(I.eq.1) then
c---
        TH23  = 0.5*(TH2(1)+TH3(1))

        call prd_2d_pr_hs_interp
     +
     +      (TH1(I),TH2(I),TH3(I),TH23
     +      ,P1(L),P1(I),P1(K),P1_int
     +      ,P2(L),P2(I),P2(K),P2_int
     +      ,P3(L),P3(I),P3(K),P3_int
     +      ,P4(L),P4(I),P4(K),P4_int
     +      ,P5(L),P5(I),P5(K),P5_int
     +      )

        Do J=NSGM,2,-1
         J1     = J+1
         X (J1) = X (J)
         Y (J1) = Y (J)
         P1(J1) = P1(J)
         P2(J1) = P2(J)
         P3(J1) = P3(J)
         P4(J1) = P4(J)
         P5(J1) = P5(J)
        End Do

        X (2) = XC(1)+R(1)*COS(TH23)
        Y (2) = YC(1)+R(1)*SIN(TH23)
        P1(2) = P1_INT
        P2(2) = P2_INT
        P3(2) = P3_INT
        P4(2) = P4_INT
        P5(2) = P5_INT

c---
      else if(I.eq.NSGM) then
c---
        TH12 = 0.5*(TH1(I)+TH2(I))

        call prd_2d_pr_hs_interp
     +
     +       (TH1(I),TH2(I),TH3(I),TH12
     +       ,P1(L),P1(I),P1(K),P1_int
     +       ,P2(L),P2(I),P2(K),P2_int
     +       ,P3(L),P3(I),P3(K),P3_int
     +       ,P4(L),P4(I),P4(K),P4_int
     +       ,P5(L),P5(I),P5(K),P5_int
     +       )

        X (K) = X (I)
        Y (K) = Y (I)
        P1(K) = P1(I)
        P2(K) = P2(I)
        P3(K) = P3(I)
        P4(K) = P4(I)
        P5(K) = P5(I)

        X (I) = XC(I)+R(I)*COS(TH12)
        Y (I) = YC(I)+R(I)*SIN(TH12)
        P1(I) = P1_INT
        P2(I) = P2_INT
        P3(I) = P3_INT
        P4(I) = P4_INT
        P5(I) = P5_INT

c---
      else
c---
        Dth  = THETOT/3.0
        TH13 = TH1(I)+    Dth
        TH23 = TH1(I)+2.0*Dth

        call prd_2d_pr_hs_interp
     +
     +     (TH1(I),TH2(I),TH3(I),TH13
     +     ,P1(L),P1(I),P1(K),P1_int1
     +     ,P2(L),P2(I),P2(K),P2_int1
     +     ,P3(L),P3(I),P3(K),P3_int1
     +     ,P4(L),P4(I),P4(K),P4_int1
     +     ,P5(L),P5(I),P5(K),P5_int1
     +     )

        call prd_2d_pr_hs_interp
     +
     +     (TH1(I),TH2(I),TH3(I),TH23
     +     ,P1(L),P1(I),P1(K),P1_int2
     +     ,P2(L),P2(I),P2(K),P2_int2
     +     ,P3(L),P3(I),P3(K),P3_int2
     +     ,P4(L),P4(I),P4(K),P4_int2
     +     ,P5(L),P5(I),P5(K),P5_int2
     +     )

        Do J=NSGM,K,-1
          J1     = J+1
          X (J1) = X (J)
          Y (J1) = Y (J)
          P1(J1) = P1(J)
          P2(J1) = P2(J)
          P3(J1) = P3(J)
          P4(J1) = P4(J)
          P5(J1) = P5(J)
        End Do

         X(K) = XC(I)+R(I)*COS(TH23)
         Y(K) = YC(I)+R(I)*SIN(TH23)
        P1(K) = P1_int2
        P2(K) = P2_int2
        P3(K) = P3_int2
        P4(K) = P4_int2
        P5(K) = P5_int2

         X(I) = XC(I)+R(I)*COS(TH13)
         Y(I) = YC(I)+R(I)*SIN(TH13)
        P1(I) = P1_int1
        P2(I) = P2_int1
        P3(I) = P3_int1
        P4(I) = P4_int1
        P5(I) = P5_int1
c---
      end if
c---

      NSG = NSG+2

      write (6,206) NSG

      Go to 98

c---------------------------
c  IF A SEGMENT IS TOO LONG,
c  ADD A POINT IN THE MIDDLE
c---------------------------
 
  91  Continue

      write (6,203) I

      ANGL1  = 0.5*(TH1(I)+TH2(I))

      X_INT1 = XC(I)+R(I)*COS(ANGL1)
      Y_INT1 = YC(I)+R(I)*SIN(ANGL1)

      La = L-1

        call prd_2d_pr_hs_interp
     +
     +      (TH1(L),TH2(L),TH3(L),ANGL1
     +      ,P1(La),P1(L),P1(I),P1_int1
     +      ,P2(La),P2(L),P2(I),P2_int1
     +      ,P3(La),P3(L),P3(I),P3_int1
     +      ,P4(La),P4(L),P4(I),P4_int1
     +      ,P5(La),P5(L),P5(I),P5_int1
     +      )

      ANGL2 = 0.5*(TH2(L)+TH3(L))

      X_INT2 = XC(L)+R(L)*COS(ANGL2)
      Y_INT2 = YC(L)+R(L)*SIN(ANGL2)

        call prd_2d_pr_hs_interp
     +
     +     (TH1(I),TH2(I),TH3(I),ANGL2
     +     ,P1(L),P1(I),P1(K),P1_int2
     +     ,P2(L),P2(I),P2(K),P2_int2
     +     ,P3(L),P3(I),P3(K),P3_int2
     +     ,P4(L),P4(I),P4(K),P4_int2
     +     ,P5(L),P5(I),P5(K),P5_int2
     +     )

       X_INT = 0.5*( X_int1 +  X_int2)
       Y_INT = 0.5*( Y_int1 +  Y_int2)
      P1_int = 0.5*(P1_int1 + P1_int2)
      P2_int = 0.5*(P2_int1 + P2_int2)
      P3_int = 0.5*(P3_int1 + P3_int2)
      P4_int = 0.5*(P4_int1 + P4_int2)
      P5_int = 0.5*(P5_int1 + P5_int2)

      Do J=NSGM,I,-1
       J1 = J+1
        X(J1) =  X(J)
        Y(J1) =  Y(J)
       P1(J1) = P1(J)
       P2(J1) = P2(J)
       P3(J1) = P3(J)
       P4(J1) = P4(J)
       P5(J1) = P5(J)
      End Do

       X(I) =  X_INT
       Y(I) =  Y_INT
      P1(I) = P1_INT
      P2(I) = P2_INT
      P3(I) = P3_INT
      P4(I) = P4_INT
      P5(I) = P5_INT

      NSG = NSG+2

      write (6,206) NSG

      Go to 98
c---

  92  Continue

      write (6,204) I

      K1 = K+1

      call arc_2d 
     +
     +   (I
     +   ,X(I),X(K),X(K1)
     +   ,Y(I),Y(K),Y(K1)
     +   ,XC(K),YC(K),R(K)
     +   ,TH1(K),TH2(K),TH3(K),ORNT(K)
     +   ,AREA1(K),AREA2(K),ARE(K)
     +   ,XCN1(K),XCN2(K),XCN(K)
     +   ,YCN1(K),YCN2(K),YCN(K)
     +   ,Istop
     +   )

      ANGL1  = 0.5*(TH2(I)+TH3(I))
      X_INT1 = XC(I)+R(I)*COS(ANGL1)
      Y_INT1 = YC(I)+R(I)*SIN(ANGL1)

        call prd_2d_pr_hs_interp
     +
     +     (TH1(I),TH2(I),TH3(I),ANGL1
     +     ,P1(L),P1(I),P1(K),P1_int1
     +     ,P2(L),P2(I),P2(K),P2_int1
     +     ,P3(L),P3(I),P3(K),P3_int1
     +     ,P4(L),P4(I),P4(K),P4_int1
     +     ,P5(L),P5(I),P5(K),P5_int1
     +     )

      ANGL2 = 0.5*(TH1(K)+TH2(K))
      XINT2 = XC(K)+R(K)*COS(ANGL2)
      YINT2 = YC(K)+R(K)*SIN(ANGL2)

        call prd_2d_pr_hs_interp
     +
     +     (TH1(K),TH2(K),TH3(K),ANGL2
     +     ,P1(I),P1(K),P1(K1),P1_int2
     +     ,P2(I),P2(K),P2(K1),P2_int2
     +     ,P3(I),P3(K),P3(K1),P3_int2
     +     ,P4(I),P4(K),P4(K1),P4_int2
     +     ,P5(I),P5(K),P5(K1),P5_int2
     +     )

       X_INT = 0.5*( X_INT1 +  X_INT2)
       Y_INT = 0.5*( Y_INT1 +  Y_INT2)
      P1_INT = 0.5*(P1_INT1 + P1_INT2)
      P2_INT = 0.5*(P2_INT1 + P2_INT2)
      P3_INT = 0.5*(P3_INT1 + P3_INT2)
      P4_INT = 0.5*(P4_INT1 + P4_INT2)
      P5_INT = 0.5*(P5_INT1 + P5_INT2)

      Do J=NSG1,K,-1
        J1     = J+1
        X (J1) = X(J)
        Y (J1) = Y(J)
        P1(J1) = P1(J)
        P2(J1) = P2(J)
        P3(J1) = P3(J)
        P4(J1) = P4(J)
        P5(J1) = P5(J)
      End Do

      X (K) =  X_INT
      Y (K) =  Y_INT
      P1(K) = P1_INT
      P2(K) = P2_INT
      P3(K) = P3_INT
      P4(K) = P4_INT
      P5(K) = P5_INT

      NSG = NSG+2

      write (6,*)
      write (6,206) NSG
      write (6,*)

      Go to 98
 
c-------------------------------
c  IF A SEGMENT IS TOO SHORT,
c  REMOVE END-POINTS
c  AND ADD A POINT IN THE MIDDLE
c
c  FIRST SEGMENT:
c
c  Point 1 will stay, 
c  Point 2 may be removed
c
c---
c  SEGMENT NSGM
c  Point NSGM will stay, 
c  Point NSGM-1 may be removed
c-------------------------------
 
  93  Continue

      write (6,209) I

      if(I.ne.NSGMa) then

        Do J=K,NSGM
          J1    = J+1
          X (J) = X(J1)
          Y (J) = Y(J1)
          P1(J) = P1(J1)
          P2(J) = P2(J1)
          P3(J) = P3(J1)
          P4(J) = P4(J1)
          P5(J) = P5(J1)
        End Do

        if(I.ne.1) then

         ANGL = 0.5*(TH2(I)+TH3(I))

        call prd_2d_pr_hs_interp
     +
     +      (TH1(I),TH2(I),TH3(I),ANGL
     +      ,P1(L),P1(I),P1(K),P1_int
     +      ,P2(L),P2(I),P2(K),P2_int
     +      ,P3(L),P3(I),P3(K),P3_int
     +      ,P4(L),P4(I),P4(K),P4_int
     +      ,P5(L),P5(I),P5(K),P5_int
     +      )

          X(I)  = XTMP
          Y(I)  = YTMP
          P1(I) = P1_int
          P2(I) = P2_int
          P3(I) = P3_int
          P4(I) = P4_int
          P5(I) = P5_int

        end if

      else

          X(NSGMa) =  X(NSGM)
          Y(NSGMa) =  Y(NSGM)
         P1(NSGMa) = P1(NSGM)
         P2(NSGMa) = P2(NSGM)
         P3(NSGMa) = P3(NSGM)
         P4(NSGMa) = P4(NSGM)
         P5(NSGMa) = P5(NSGM)

      end if

      NSG = NSG-2

      write (6,*) 
      write (6,208) NSG
      write (6,*) 

      GO TO 98

C-=-=--=-=-=-=-=-=-=-=-=-=-=-=-==-=-

  29  Continue

      If(ICH4.eq.1) then

c----------------
c REDEFINE PERIOD
c----------------

      Iloop = 1

  89  Continue

      Iloop = Iloop + 1

      if(Iloop.eq.10) Go to 88  ! to prevent infinite looping

c---
      if(X(1).lt.thrs1) then
c---
      write (6,*) 
      write (6,*) " Period redefined, 1"
      write (6,*) 

        Do I=0,NSG1
          I1 = I+1
          X (I) = X (I1) 
          Y (I) = Y (I1) 
          P1(I) = P1(I1) 
          P2(I) = P2(I1) 
          P3(I) = P3(I1) 
          P4(I) = P4(I1) 
          P5(I) = P5(I1) 
        End Do

         X(NSG2) =  X(2)+RL 
         Y(NSG2) =  Y(2) 
        P1(NSG2) = P1(2) 
        P2(NSG2) = P2(2) 
        P3(NSG2) = P3(2) 
        P4(NSG2) = P4(2)+arclt
        P5(NSG2) = P5(2) 

        Go to 89
c---
      else if(X(1).gt.thrs2) then
c---
      write (6,*) 
      write (6,*) " Period redefined, 2"
      write (6,*)

        Do I=1,NSG1
         I2 = NSG2-I
         I1 = NSG1-I
         X (I1) = X(I1) 
         Y (I1) = Y(I1) 
         P1(I1) = P1(I1) 
         P2(I1) = P2(I1) 
         P3(I1) = P3(I1) 
         P4(I1) = P4(I1) 
         P5(I1) = P5(I1) 
        End Do

         X(NSG2) =  X(2) + RL 
         Y(NSG2) =  Y(2) 
        P1(NSG2) = P1(2) 
        P2(NSG2) = P2(2) 
        P3(NSG2) = P3(2) 
        P4(NSG2) = P4(2)+arclt 
        P5(NSG2) = P5(2) 

         X(0) =  X(NSG) - RL
         Y(0) =  Y(NSG) 
        P1(0) = P1(NSG) 
        P2(0) = P2(NSG) 
        P3(0) = P3(NSG) 
        P4(0) = P4(NSG)-arclt 
        P5(0) = P5(NSG) 

        Go to 89

      end if

  88  Continue

      end if

c-----------------------------
c THE SUCCESSFUL DISTRIBUTION
c-----------------------------

c-----------------------
c Compute the arc length
c-----------------------

      S(1) = 0.0D0

      Do i=2,NSG1
       ia = i-1
       S(i) = S(ia)+0.50*(R(i) *ABS(TH2(i )-TH1(i ))
     +                   +R(ia)*ABS(TH3(ia)-TH2(ia)))
      End Do

      S(NSG2) = S(NSG1)+S(2)
      ARCT = S(NSG1)

c------------------
c area and centroid
c------------------

      ARET = 0.0D0
      XCNT = 0.0D0
      YCNT = 0.0D0

      Do I=2,NSG1
        ia = i-1
        ARET = ARET + 0.5*(AREA1(i)+AREA2(ia))
        XCNT = XCNT + 0.5*(XCN1 (i)+XCN2 (ia))
        YCNT = YCNT + 0.5*(YCN1 (i)+YCN2 (ia))
      End Do

      ARET = - ARET

c---
c normalization
c---

      XCNT = XCNT/ARET
      YCNT = YCNT/ARET

c---
c done
c---

 100  Format (1X,F10.5,2X,F10.5,10X,F10.5,2X,F10.5)
 201  Format (' EXECUTION WILL BE INTERRUPTED')

 200  Format (/,' ARC',I3,' IS TOO BIG',2(F10.5))
 203  Format (/,' SEGMENT 1 OF ARC',I3,' IS TOO LARGE')
 204  Format (/,' SEGMENT 2 OF ARC',I3,' IS TOO LARGE')
 209  Format (/,' SEGMENT 2 OF ARC',I3,' IS TOO SMALL')

 206  Format (' ONE POINT INSERTED, TOTAL Points ',I3,/)
 208  Format (' ONE POINT REMOVED , TOTAL NSG ',I3,/)

      return
      end
