      subroutine rec_2d_vgt
     +
     +   (ax,bx
     +   ,ay,by
     +   ,Nx,Ny
     +   ,Axx,Axy
     +   ,Ayx,Ayy
     +   )

c------------------------------
c Compute the velocity gradient
c tensor, A_ij, at the grid points
c by numerical differentiation
c-------------------------------

      Dimension glx(129),gly(129)

      Dimension gx(129,129),gy (129,129)
      Dimension gux(129,129),guy(129,129)

      Dimension Axx(129,129),Axy(129,129)
      Dimension Ayx(129,129),Ayy(129,129)

c---
c common blocks
c---

      common/grid_2d_1/glx,gly
      common/grid_2d_2/gx,gy
      common/grid_2d_3/gux,guy

c----------------
c interior points
c compute derivatives by central differences
c----------------

      Do i=2,Nx
        Do j=2,Ny
         Axx(i,j) = (gux(i+1,j)-gux(i-1,j))/(glx(i+1)-glx(i-1))
         Axy(i,j) = (guy(i+1,j)-guy(i-1,j))/(glx(i+1)-glx(i-1))
         Ayx(i,j) = (gux(i,j+1)-gux(i,j-1))/(gly(j+1)-gly(j-1))
         Ayy(i,j) = (guy(i,j+1)-guy(i,j-1))/(gly(j+1)-gly(j-1))
        End Do
      End Do

c----------
c left wall
c compute derivatives by central or forward differences
c----------

      i=1
      Do j=2,Ny
         Axx(i,j) = (gux(i+1,j)-gux(i,j))/(glx(i+1)-glx(i))
         Axy(i,j) = (guy(i+1,j)-guy(i,j))/(glx(i+1)-glx(i))
         Ayx(i,j) = (gux(i,j+1)-gux(i,j-1))/(gly(j+1)-gly(j-1))
         Ayy(i,j) = (guy(i,j+1)-guy(i,j-1))/(gly(j+1)-gly(j-1))
      End Do

c----------
c bottom wall
c compute derivatives by central or forward differences
c----------

      j=1
      Do i=2,Nx
         Axx(i,j) = (gux(i+1,j)-gux(i-1,j))/(glx(i+1)-glx(i-1))
         Axy(i,j) = (guy(i+1,j)-guy(i-1,j))/(glx(i+1)-glx(i-1))
         Ayx(i,j) = (gux(i,j+1)-gux(i,j))/(gly(j+1)-gly(j))
         Ayy(i,j) = (guy(i,j+1)-guy(i,j))/(gly(j+1)-gly(j))
      End Do

c----------
c right wall
c compute derivatives by central or backward differences
c----------

      i=Nx+1
      Do j=2,Ny
         Axx(i,j) = (gux(i,j)-gux(i-1,j))/(glx(i)-glx(i-1))
         Axy(i,j) = (guy(i,j)-guy(i-1,j))/(glx(i)-glx(i-1))
         Ayx(i,j) = (gux(i,j+1)-gux(i,j-1))/(gly(j+1)-gly(j-1))
         Ayy(i,j) = (guy(i,j+1)-guy(i,j-1))/(gly(j+1)-gly(j-1))
      End Do

c----------
c top wall
c compute derivatives by central or backward differences
c----------

      j=Ny+1
      Do i=2,Nx
         Axx(i,j) = (gux(i+1,j)-gux(i-1,j))/(glx(i+1)-glx(i-1))
         Axy(i,j) = (guy(i+1,j)-guy(i-1,j))/(glx(i+1)-glx(i-1))
         Ayx(i,j) = (gux(i,j)  -gux(i,j-1))/(gly(j)  -gly(j-1))
         Ayy(i,j) = (guy(i,j)  -guy(i,j-1))/(gly(j)  -gly(j-1))
      End Do

c-------------------
c four corner points
c-------------------

      i=1
      j=1
      Axx(i,j) = (gux(i+1,j)-gux(i,j))/(glx(i+1)-glx(i))
      Axy(i,j) = (guy(i+1,j)-guy(i,j))/(glx(i+1)-glx(i))
      Ayx(i,j) = (gux(i,j+1)-gux(i,j))/(gly(j+1)-gly(j))
      Ayy(i,j) = (guy(i,j+1)-guy(i,j))/(gly(j+1)-gly(j))

      i=Nx+1
      j=1
      Axx(i,j) = (gux(i,j)-gux(i-1,j))/(glx(i)-glx(i-1))
      Axy(i,j) = (guy(i,j)-guy(i-1,j))/(glx(i)-glx(i-1))
      Ayx(i,j) = (gux(i,j+1)-gux(i,j))/(gly(j+1)-gly(j))
      Ayy(i,j) = (guy(i,j+1)-guy(i,j))/(gly(j+1)-gly(j))

      i=Nx+1
      j=Ny+1
      Axx(i,j) = (gux(i,j)-gux(i-1,j))/(glx(i)-glx(i-1))
      Axy(i,j) = (guy(i,j)-guy(i-1,j))/(glx(i)-glx(i-1))
      Ayx(i,j) = (gux(i,j)-gux(i,j-1))/(gly(j)-gly(j-1))
      Ayy(i,j) = (guy(i,j)-guy(i,j-1))/(gly(j)-gly(j-1))

      i=1
      j=Ny+1
      Axx(i,j) = (gux(i+1,j)-gux(i,j))/(glx(i+1)-glx(i))
      Axy(i,j) = (guy(i+1,j)-guy(i,j))/(glx(i+1)-glx(i))
      Ayx(i,j) = (gux(i,j)-gux(i,j-1))/(gly(j)-gly(j-1))
      Ayy(i,j) = (guy(i,j)-guy(i,j-1))/(gly(j)-gly(j-1))

c-----
c done
c-----

      Return
      End
