       subroutine sm_3d_df_interp
     +
     +    (x1,y1,z1
     +    ,x2,y2,z2
     +    ,x3,y3,z3
     +    ,x4,y4,z4
     +    ,x5,y5,z5
     +    ,x6,y6,z6
     +    ,al,be,ga
     +    ,f1,f2,f3,f4,f5,f6
     +    ,xi,eta
     +    ,rlap
     +    )

c=========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c=========================================

c-------------------------------------------------
c interpolates over an element
c to compute the pseudo-laplacian of the function f
c--------------------------------------------------

      Implicit Double Precision (a-h,o-z)

c---
c prepare
c---

      alc = 1.0D0-al
      bec = 1.0D0-be
      gac = 1.0D0-ga

      alalc = al*alc
      bebec = be*bec
      gagac = ga*gac

c---
c  local interpolation functions phi
c---

      ph2 = xi *(xi -al+eta*(al-ga)/gac)   /alc
      ph3 = eta*(eta-be+xi*(be+ga-1.0)/ga)/bec
      ph4 = xi *(1.0-xi-eta)/alalc
      ph5 = xi*eta/gagac
      ph6 = eta*(1.0-xi-eta)/bebec
      ph1 = 1.0-ph2-ph3-ph4-ph5-ph6

c---
c  xi derivatives of phi
c---

      dph2 =  (2.0*xi-al +eta*(al-ga)/gac)/alc
      dph3 =  eta*(be+ga-1.0)/(ga*bec)
      dph4 =  (1.0-2.0*xi-eta)/alalc
      dph5 =  eta/gagac
      dph6 = -eta/bebec
      dph1 = -dph2-dph3-dph4-dph5-dph6

c---
c  eta derivatives of phi
c---

      pph2 =  xi*(al-ga)/(alc*gac)
      pph3 =  (2.0*eta-be+xi*(be+ga-1.0)/ga)/bec
      pph4 =  -xi/alalc
      pph5 =   xi/gagac
      pph6 =  (1.0-xi-2.0*eta)/bebec
      pph1 = - pph2 - pph3 - pph4 - pph5 - pph6

c---
c  second xi derivatives of phi
c---

      ddph2 =  2.0/alc
      ddph3 =  0.0
      ddph4 = -2.0/alalc
      ddph5 =  0.0
      ddph6 =  0.0
      ddph1 = -ddph2-ddph3-ddph4-ddph5-ddph6

c---
c  second eta derivatives of phi
c---

      ppph2 =  0.0
      ppph3 =  2.0/bec
      ppph4 =  0.0
      ppph5 =  0.0
      ppph6 = -2.0/bebec
      ppph1 = - ppph2 - ppph3 - ppph4 - ppph5 - ppph6

c---
c  second xi-eta derivatives of phi
c---

      dpph2 =  (al-ga)/(alc*gac)
      dpph3 =  (be+ga-1.0)/(ga*bec)
      dpph4 =  -1.0/alalc
      dpph5 =   1.0/gagac
      dpph6 =  -1.0/bebec
      dpph1 = - dpph2 - dpph3 - dpph4 - dpph5 - dpph6

c---
c  compute dx/dxi from xi derivatives of phi
c  compute xi metric
c---

      DxDxi = x1*dph1 + x2*dph2 + x3*dph3 + x4*dph4
     +      + x5*dph5 + x6*dph6
      DyDxi = y1*dph1 + y2*dph2 + y3*dph3 + y4*dph4
     +      + y5*dph5 + y6*dph6
      DzDxi = z1*dph1 + z2*dph2 + z3*dph3 + z4*dph4
     +      + z5*dph5 + z6*dph6

      hxi = sqrt(DxDxi**2 +  DyDxi**2 +  DzDxi**2)

c---
c  compute dx/deta from eta derivatives of phi
c  compute eta metric
c---

      DxDet = x1*pph1 + x2*pph2 + x3*pph3 + x4*pph4
     +      + x5*pph5 + x6*pph6
      DyDet = y1*pph1 + y2*pph2 + y3*pph3 + y4*pph4
     +      + y5*pph5 + y6*pph6
      DzDet = z1*pph1 + z2*pph2 + z3*pph3 + z4*pph4
     +      + z5*pph5 + z6*pph6

      het = sqrt(DxDet**2 +  DyDet**2 +  DzDet**2)

c---
c  compute the raw normal vector and the
c  surface metric hs
c---

      vnx = DyDxi * DzDet - DyDet * DzDxi
      vny = DzDxi * DxDet - DzDet * DxDxi
      vnz = DxDxi * DyDet - DxDet * DyDxi

      hs = sqrt(vnx**2+vny**2+vnz**2)

c---
c compute second derivatives of f
c---

      ddf = f1*ddph1 + f2*ddph2 + f3*ddph3 + f4*ddph4
     +    + f5*ddph5 + f6*ddph6

      dpf = f1*dpph1 + f2*dpph2 + f3*dpph3 + f4*dpph4
     +    + f5*dpph5 + f6*dpph6

      ppf = f1*ppph1 + f2*ppph2 + f3*ppph3 + f4*ppph4
     +    + f5*ppph5 + f6*ppph6

c---
c finally
c---

      hxis = hxi*hxi
      hets = het*het

      beta = DxDxi*DxDet + DyDxi*DyDet + DzDxi*DzDet

      par  = hxis*hets - beta*beta

      fc1  =       hets/par
      fc2  = - 2.0*beta/par
      fc3  =       hxis/par

c     fc1 = het**2/hs**2
c     fc2 = (DxDxi*DxDet + DyDxi*DyDet + DzDxi*DzDet )/hs**2
c     fc3 = hxi**2/hs**2

      rlap =  fc1*ddf + fc2*dpf + fc3*ppf

c---
c done
c---

      return
      end
