      subroutine fleg_cf
     +
     +  (nelm,npts
     +  ,mint
     +  ,f
     +  ,j_max
     +  )

c========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c========================================

c----------------------------------------
c Computate the Fourier-Legendre coefficients
c
c SYMBOLS
c -------
c
c f(i):    surface function
c          evaluated at the ith node
c
c rmpp(i,j,m):  
c
c   Lower triangular matrix
c   with dimensions (j_max+1)x(j_max+1)
c   holding the modified Legendre functions
c
c   Generated by subroutine: fleg_prep
c
c afl,bfl:  Fourier coefficients
c----------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension    p(1026,3)
      Dimension   ne(1026,7)
      Dimension    f(1026)
      Dimension  phi(1026)
      Dimension rmpp(1026,0:32,0:32)

      Dimension fncs(1026)
      Dimension fnc1(1026,0:32,0:32)
      Dimension fnc2(1026,0:32,0:32)

      Dimension n(512,6),nbe(512,3)

      Dimension afl(0:32,0:32),bfl(0:32,0:32)

c--------------
c common blocks
c--------------

      common/points/p,ne
      common/elmnts/n,nbe

      common/fleg/phi,rmpp
      common/flegc/afl,bfl

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi
      pi4 = 4.0D0*pi

c-----------------------------------------
c Evaluate the integrands 
c of the projection integrals at the nodes
c-----------------------------------------

      Do i=1,npts
        Do j=0,j_max
         Do m=0,j

          arg = m*phi(i)

          fnc1(i,j,m) = f(i)*rmpp(i,j,m)*dcos(arg)
          fnc2(i,j,m) = f(i)*rmpp(i,j,m)*dsin(arg)

c       if(j.eq.0.and.m.eq.0) then
c        write (6,100) i,f(i),rmpp(i,j,m),phi(i)
c       end if

         End Do
        End Do
      End Do

c-------------------------------------
c perform the surface integration
c to recover the Fourier coefficients
c-------------------------------------

c     area = pi4
c     factor = pi4/area  ! normalization factor for surface area
      factor = 1.0D0

      Do j=0,j_max
       Do m=0,j

       Do i=1,npts
        fncs(i) = fnc1(i,j,m)
c       If(j.eq.0.and.m.eq.0) then
c        write (6,100)  i,fncs(i),rmpp(i,j,m)
c       End If
       End Do

       call srf_int_3d
     +
     +    (nelm,npts,mint
     +    ,fncs
     +    ,olok
     +    )

       afl(j,m) = 2.0D0*olok*factor

       Do i=1,npts
        fncs(i) = fnc2(i,j,m)
       End Do

       call srf_int_3d
     +
     +    (nelm,npts,mint
     +    ,fncs
     +    ,olok
     +    )

       bfl(j,m) = 2.0D0*olok*factor

       End Do
      End Do

c-----
c done
c-----

 100  Format (1x,i4,10(1x,f10.5))

      return
      end
