      subroutine fleg_prep (npts,j_max)

c===========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c===========================================

c----------------------------------------
c Compute phi and the matrix rmpp at the nodes
c
c SYMBOLS
c -------
c
c rmpp(i,j,m):  Lower triangular matrix
c               with dimensions (j_max+1)x(j_max+1)
c               holding the Legendre functions
c               evaluated at the ith point
c
c phi(i): azimuthal angle
c----------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension    p(1026,3)
      Dimension   ne(1026,7)

      Dimension  phi(1026)
      Dimension rmpp(1026,0:32,0:32)

      Dimension t(0:32,0:32)

      Parameter (tol=0.999999D0)

c--------------
c common procks
c--------------

      common/points/p,ne
      common/fleg/phi,rmpp

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

c----------------------------------
c Compute phi and rmpp at the nodes
c----------------------------------

      Do i=1,npts

c----------------------------------
c compute the azimuthal angle: phi
c----------------------------------

        sigma = dsqrt( p(i,2)**2+p(i,3)**2 )

        if(sigma.lt.0.000001D0) then
         phi(i) = 0.0D0
        else
         cosphi = p(i,2)/sigma
         if(cosphi.gt. tol) cosphi = tol
         if(cosphi.lt.-tol) cosphi =-tol
         phi(i) = dacos(cosphi)
         if(p(i,3).lt.0) phi(i) = pi2-phi(i)
        end if

c-----------------------------------
c compute the triangular matrix rmpp
c-----------------------------------

        rdist = dsqrt( p(i,1)**2+p(i,2)**2+p(i,3)**2 )
        costh = p(i,1)/rdist

        call leg_fnc_ortho 
     +
     +     (costh
     +     ,j_max
     +     ,t
     +     )

        Do j=0,j_max
         Do m=0,j
          rmpp(i,j,m) = t(j,m)
         End Do
        End Do

c       Do j=0,j_max
c        write (6,101) (t(j,m),m=0,j)
c       End Do

      End Do

c-----
c done
c-----

  101 Format(10(1x,f12.8))

      return
      end
