      program sm_3d_tr_tbl

c===========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c===========================================

c-----------------------------------------------
c  Generate a matrix (table--tbl)
c  for smoothing of a scalar function
c  over a closed surface
c  described by triangulation
c
c  The smoothing is done by mapping the surface onto
c  the unit sphere, and then applying spectrum truncation
c
c  SYMBOLS:
c  -------
c
c  p(i,j): coordinates of node i (j=1,2,3)
c
c  n(k,i): connectivity table: 
c          global labels of points for element k
c          i = 1, ..., 6
c
c  nbe(k,j): the three neighboring elements of element k (j=1,2,3)
c
c  ne(k,j):  ne(k,1) is the number of elements adjacent to point k
c            ne(k,2), ... are the elements numbers, j = 2, ..., 7
c            for this triangulation, up to six
c
c  npts:  total number of points
c  nelm:  total number of elements
c
c  alpha, beta, gamma: triangle mapping coefficients
c
c  smm:  smoothing matrix
c------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension    p(1026,3)
      Dimension   ne(1026,7)
      Dimension  vna(1026,3)

      Dimension    n(512,6),nbe(512,3)
      Dimension alpha(512),beta(512),gamma(512)

      Dimension xiq(20),etq(20),wq(20)

      Dimension     f(1026),f_sm(1026)
      Dimension   smm(1026,1026)

c--------------
c common blocks
c--------------

      common/points/p,ne
      common/elmnts/n,nbe

      common/albega/alpha,beta,gamma

      common/trq/xiq,etq,wq

      common/pii/pi,piq,pih,pi2,pi4,pi6,pi8

c----------
c constants
c----------

      pi = 3.14159 265358 D0

      piq = 0.25D0*pi
      pih = 0.50D0*pi
      pi2 = 2.00D0*pi
      pi4 = 4.00D0*pi
      pi6 = 6.00D0*pi
      pi8 = 8.00D0*pi

c-----------
c input data
c-----------

  91  Continue

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to discretize an octahedron"
      write (6,*) " 2 to discretize an icosahedron"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) Ioctaicos

      if(Ioctaicos.eq.0) Go to 99

      write (6,*)
      write (6,*) " Select the level of triangulation"
      write (6,*)
      write (6,*) " Enter 0, 1, 2, 3; 99 to quit"
      write (6,*) " ----------------------------"
      read  (5,*) ndiv 

      if(ndiv.eq.99) Go to 99

      if(ndiv.gt.3)  then
        write (6,*)
        write (6,*) 'out of range; please try again.'
        write (6,*)
        Go to 91
      end if

      write (6,*)
      write (6,*) " Will use the m-point gauss-triangle rule"
      write (6,*) " for integration over each triangle"
      write (6,*)
      write (6,*) " Please enter m"
      write (6,*)
      write (6,*) " choose from 1, 3, 4, 6, 7, 9, 12, 13"
      write (6,*) " Enter 0 to quit"
      write (6,*) "----------------"
      read  (5,*) mint

c     mint = 7

c---------------------------------
c weights for triangle integration
c---------------------------------

      call gauss_trgl
     +
     +   (mint
     +   ,xiq,etq,wq
     +   )

c----------------------------
c triangulate the unit sphere
c----------------------------

      if(Ioctaicos.eq.1) then

       call trgl6_octa
     +
     +  (ndiv
     +  ,npts,nelm
     +  )

      else

      call trgl6_icos
     +
     +  (ndiv
     +  ,npts,nelm
     +  )

      end if

      write (6,*)
      write (6,*) "Number of points:   ",npts
      write (6,*)
      write (6,*) "Number of elements: ",nelm
      write (6,*)

c----------------------------
c Compute alpha, beta, gamma,
c for the xi-eta mapping
c over each triangle
c---------------------------

      Do k=1,nelm

       i1 = n(k,1)
       i2 = n(k,2)
       i3 = n(k,3)
       i4 = n(k,4)
       i5 = n(k,5)
       i6 = n(k,6)

       call abc 
     +
     +   (p(i1,1),p(i1,2),p(i1,3)
     +   ,p(i2,1),p(i2,2),p(i2,3)
     +   ,p(i3,1),p(i3,2),p(i3,3)
     +   ,p(i4,1),p(i4,2),p(i4,3)
     +   ,p(i5,1),p(i5,2),p(i5,3)
     +   ,p(i6,1),p(i6,2),p(i6,3)
     +   ,alpha(k),beta(k),gamma(k)
     +   )

      End Do

c-----------------------------
c select the truncation level
c-----------------------------

      advice   = sqrt(npts-1.0D0)
      J_advice = Nint(advice)

      write (6,*)
      write (6,*) " Enter the j truncation level"
      write (6,*) " for the Legendre expansion"
      write (6,*) " in the range: 0, 1, ...",J_advice
      write (6,*)
      write (6,*) " 99 to quit"
      write (6,*) "-----------"
      read  (5,*) j_max

      if(j_max.eq.99) Go to 99

      write (6,*) " sm_3d_tbl: preparing"

      call fleg_prep (npts,j_max)

      write (6,*) "            done"

c------------------------------
c Generate the smoothing matrix
c by the method of impulses
c
c fimp: impulse vector; all components 0
c       except for one component that is 1
c
c smm: smoothing matrix
c
c Smoothing is effected by the projection:
c
c f_smoothed(i) = smm(i,j) f(j)
c--------------------------------

      open (2,file="sm_3d_tr.tbl")

      j_trunc = j_max

      write (2,120) j_trunc

      write (6,*) " will count up to ",npts

      Do i=1,npts

       write (6,*) i

       Do j=1,npts          ! initialize to zero
         f(j) = 0.0D0
       End Do

       f(i) = 1.0D0            ! impulse
      
       call fleg_cf
     +
     +   (nelm,npts
     +   ,mint
     +   ,f
     +   ,j_max
     +   )

       call sm_3d_tr
     +
     +   (npts
     +   ,f
     +   ,j_trunc
     +   ,f_sm
     +   )

       Do k=1,npts
         smm(k,i) = f_sm(k)
         write (2,120) k,i,f_sm(k)
       End Do

      End Do

      close (2)

c-----
c done
c-----

  99  Continue

  100 Format(1x,i4,10(1x,f8.4))
  101 Format(10(1x,f12.5))
  102 Format(1x,i4,1x,i4,10(f15.10))

  110 Format(" Surface Area :",F15.10)
  111 Format(" Volume       :",F15.10)

  120 Format(1x,i4,1x,i4,1x,f16.11)

      stop
      end
