function [Bfinal,x,y] = drop_2d ...
 ...
  (Jsp,gac,gamma,rhod,rhoa,area...
  ,alpha,npts,epsilon,maxiter,tol ...
  ,Binit ...
  )
 
%--------------------------------------------
% Hydrostatic shape of a two-dimensional
% sessile drop resting on a horizontal plate,
% or pendant drop hanging underneath
% a horizontal plate,
% for a specified area and contact angle
%--------------------------------------------

%--------
% prepare
%--------

 Iflag=0;

 drho  = rhod-rhoa;            % density difference

 Isp = 1;                        % Isp is an orientation index
 if(drho<0) 
   Isp = -Isp;
 end
 if(Jsp==-1)
   Isp = -Isp;
 end

 capls = gamma/(gac*abs(drho));  % square of the capillary number

%----------
% initial B
%----------

  B(1) = Binit;

%---
% compute initial solution of the odes
% to start-up the secant method
%---

  dpsi = alpha/npts;

  Ic=1;   % counter

   [x,y,area1] = drop_2d_ode ...
   ...
   (npts ...
   ,capls ...
   ,Isp ...
   ,dpsi ...
   ,B(Ic) ...
   );

   error(Ic) = area1 - area;
   err = abs(error(Ic));

%-------------------------
% second start-up solution 
%-------------------------

   Ic=2;
   B(2) = B(1)+epsilon;

   [x,y,area1] = drop_2d_ode ...
   ...
   (npts ...
   ,capls ...
   ,Isp ...
   ,dpsi ...
   ,B(Ic) ...
   );

    error(Ic) = area1 - area;
    err = abs(error(Ic));

%---------------------------------------
% iterate on B using the secant method
% until convergence
%---------------------------------------

   for iter=1:maxiter

    Ic = Ic+1;

%---
% secant updating
%---
     
   Icb = Ic-2;
   Ica = Ic-1;

   dedc = (error(Ica)-error(Icb))/(B(Ica)-B(Icb));

   B(Ic) = B(Ica)-error(Ica)/dedc;

   [x,y,area1] = drop_2d_ode ...
   ...
   (npts ...
   ,capls ...
   ,Isp ...
   ,dpsi ...
   ,B(Ic) ...
   );

   error(Ic) = area1 - area;
   err       = abs(error(Ic));

   if(err<tol)
    break
   end

%---
   end
%---

   if(iter==maxiter)
    disp('drop_2d: ODE solver failed')
    Iflag=1;
    return
   end

   Bfinal = B(Ic);

%---
% done
%---
 
 return
