function [x,y,area] = drop_2d_ode ...
   ...
   (npts ...
   ,capls ...
   ,Isp ...
   ,dpsi ...
   ,B ...
   )

%-----------------------------------------
% FDLIB
%
% Copyright by C. Pozrikidis, 1999
% All rights reserved.
%
% This program is to be used only under the
% stipulations of the licensing agreement.
%----------------------------------------

%--------------------------------------------
% Two-dimensional drop on a horizontal plane.
% Integrate ODEs by RK4 with uniform
% step size for the slope angle psi
%
% SYMBOLS:
% -------
%
% dpsi: increments in psi
% B:  shooting parameter
%--------------------------------------------

%--------
% prepare
%--------

 dpsih = 0.5*dpsi;

%----------------
% top of the drop
%----------------

 psi  = 0.0;
 x(1) = 0.0;
 y(1) = 0.0;

 for i=1:npts

   if(i==1) 
     xp = 0.0;
     yp = 1.0/B;
   else
     Q  = Isp*x(i)/capls-B;
     xp = sin(psi)/Q;
     yp =-cos(psi)/Q;
   end
   xp1 = xp;
   yp1 = yp;

   psi    = psi    +dpsih;
   x(i+1) = x(i)+xp*dpsih;
   y(i+1) = y(i)+yp*dpsih;

   Q  = Isp*x(i+1)/capls-B;
   xp = sin(psi)/Q;
   yp =-cos(psi)/Q;
   xp2 = xp;
   yp2 = yp;

   x(i+1) = x(i)+xp*dpsih;
   y(i+1) = y(i)+yp*dpsih;

   Q  = Isp*x(i+1)/capls-B;
   xp = sin(psi)/Q;
   yp =-cos(psi)/Q;
   xp3 = xp;
   yp3 = yp;

   psi    = psi    +dpsih;
   x(i+1) = x(i)+xp*dpsi;
   y(i+1) = y(i)+yp*dpsi;

   Q  = Isp*x(i+1)/capls-B;
   xp = sin(psi)/Q;
   yp =-cos(psi)/Q;
   xp4 = xp;
   yp4 = yp;

   x(i+1) = x(i) + (xp1+2*xp2+2*xp3+xp4)*dpsi/6.0;
   y(i+1) = y(i) + (yp1+2*yp2+2*yp3+yp4)*dpsi/6.0;

   end

%----------------------------------------
% compute the are of the integrated shape
% by the trapezoidal rule
%----------------------------------------

 area = 0.0;

 for i=1:npts
   area = area+(y(i+1)+y(i))*abs(x(i+1)-x(i));
 end

 area = 0.5*area; % to account for trapezoidal weights

%---
% double the area to get the full shape
%---

 area = 2.0*area;

%-----
% done
%-----

 return
