 close all
 clear all

%------------------------------------------------------
% Hydrostatic shape of a two-dimensional
% sessile drop resting on an inclined plane
% or a pendant drop hanging underneath a inclined plane,
% for a specified area and first contact angle
%------------------------------------------------------

  global area capls Isp npts beta alpha1 alpha2 x y

  gac  = 1.0;  % acceleration of gravity
  rhod = 1.0;  % density of the drop
  rhoa = 0.0;  % density of the ambient fluid

  area   = pi;  % drop area

  beta = 0.00*pi;  % inclination angle
  beta = 0.50*pi;  % inclination angle
  beta = 0.75*pi;  % inclination angle
  beta = 1.25*pi;  % inclination angle
  beta = 0.125*pi;  % inclination angle
  beta = 0.25*pi;  % inclination angle

  alpha1 = 0.75*pi;   % first contact angle
  alpha1 = 0.35*pi;   % first contact angle
  alpha1 = 0.5*pi;   % first contact angle
  alpha1 = 0.45*pi;   % first contact angle

  npts =64;     %  number of interfacial markers

%---
% prepare
%---

 drho = rhod-rhoa;            % density difference

 Isp = 1.0;     % orientation index
 if(drho<0)
   Isp = -1.0;
 end

 csb = cos(beta);
 snb = sin(beta);

 ROT = [csb, snb; -snb, csb];  % rotation matrix for graphics

 options = optimset('TolFun',0.0000001,'MaxFunEvals',200000,'Display','iter');

 figure(1)
 hold on

%---
% initial guess
%---

 a = sqrt(area/(alpha1-0.5*sin(2.0*alpha1)));
 Crc = 1.0/a;
 solution = [Crc  alpha1];

%---
% loop over surface tension and animate
%---

 gamma = 10.0;

%========
 for repeat=1:300
%========

% gamma = 10.0-repeat*0.05;
  gamma = gamma*0.99^2;
  repeat
  gamma

% if(repeat==1)  
%  gamma = 5.0;
% elseif(repeat==2)  
%  gamma = 3.0;
% elseif(repeat==3)  
%  gamma = 2.5;
% elseif(repeat==4)  
%  gamma = 2.0;
% elseif(repeat==5)  
%  gamma = 1.0;
% end

  capls = gamma/(gac*abs(drho));  % square of the capillary number

%---
% use a matlab solver
%---

% solution = fsolve(@drop_2di_ode, solution, options)

%---
% use newton's method
%---
  
  Niter = 10;
  eps = 0.001;

  [solution,F,Iflag] = drop_2di1_newton2 ...
   ...
     (Niter ...
     ,eps ...
     ,solution ...
     );

  if(Iflag==1) break; end

%---
% assign unknowns
%---

  C      = solution(1);
  alpha2 = solution(2);

  xplot = -y;
  yplot =  x;

%---
% plot
%---

 for i=1:npts+1
   xx = [xplot(i), yplot(i)];
   xx= ROT*xx';
   xplot(i)=xx(1);
   yplot(i)=xx(2);
 end

 if(repeat==1)  
  plot(xplot,yplot,'k--')
 else
  plot(xplot,yplot,'k-')
  patch(xplot,yplot,'y')
 end

 figure(1)
 hold on
 plot(csb*[1.0,-4.5],snb*[-1.0,4.5],'k','LineWidth',3)
 axis equal
 xlabel('x','fontsize',14)
 ylabel('y','fontsize',14)
 set(gca,'fontsize',14)
 hold off
 pause(0.1)

 if(alpha2<0.05*pi) break; end
 if(alpha2>0.95*pi) break; end

end
%===

