function [s,x,q,beta,alpha,xc,xcl,scl ...
         ,beta_flat,xc_flat ...
         ,al_scan,bt_scan,xc_scan  ...
                 ,bx_scan,xx_scan  ...   % flat plate solution
                 ,bk_scan,xk_scan] ...   % circular cylinder solution
 ...
   = flsphere (a,capl,rho1,rho2,kappa,alphain ...
              ,smax,ndiv ...
              ,mincut,maxcut ...
              ,iplot_shape)

%===================================================
% Compute families of floating sphere configurations
%
% alphain: targeted value of alpha
%          (will stop when alpha = alphain)
%===================================================

%-----------
% compute the floating angle of a sphere
%-----------

eps=0.000001;
eps=0.0000001;

tol=0.0000001;

%---
% scan the contact angle space
% with step dal (d alpha)
%---

dal = 0.005;
dal = 1.0*dal;

Nmax = 2*abs(floor(pi/dal));

%----
% prepare
%---

Drho = rho2-rho1;
Brho = 0.5*(rho1+rho2);     % mean fluid density
rhos = Brho-0.5*kappa*Drho;  % density of the sphere
capls = capl*capl;       % square of the capillary number

%----
% flat interface solution
%
% computed by solving a cubic equation using
% the "roots" matlab function (internal)
%---

C(1) = 1.0;
C(2) = 0.0;
C(3) =-3.0;
C(4) =-2.0*kappa;

cosbeta =roots(C);

if(abs(cosbeta(1))<1)
   beta = acos(cosbeta(1));
elseif(abs(cosbeta(2))<1)
   beta = acos(cosbeta(2));
elseif(abs(cosbeta(3))<1)
   beta = acos(cosbeta(3));
end

xc = -a*cos(beta);

beta_flat = beta;
xc_flat = xc;

%xc=0.05;
%alpha=0.501*pi;
%beta=0.5002*pi;   % initial guess

%---
% proceed to scan alpha
%---

Ido = 1;
Iflag = 0;
Icount = 0;
Jcount = 0;
Irecord = 1;
alpha = beta;   % flat plate contact angle

%---
while(Ido==1) % loop over contact angle
%---

Icount = Icount+1;

itermax = 20;

%---
for iter=1:itermax
%---

%---
% solve for beta using Newton's method
%---

  cs  = cos(beta);
  sn  = sin(beta);
  amb = alpha-beta;
  xc  = (2/3*cs^3 +2/3*kappa +2*capls/a^2*sn*sin(amb))/sn^2;
  xcl = xc+a*cs;
  scl =    a*sn;
  slope = tan(amb);

  [x,s,q] = flsphere_ode(ndiv,smax,capls,xcl,scl,slope);

% obj = x(ndiv+1);
  arg = s(ndiv+1)/capl;
  obj = x(ndiv+1) + capl*besselk(0,arg)/besselk(1,arg)*q(ndiv+1);

  if(abs(obj)<tol) break; end

  beta = beta+eps;

  cs = cos(beta);
  sn = sin(beta);
  amb = alpha-beta;
  xc = (2/3*cs^3 +2/3*kappa+2*capls/a^2*sn*sin(amb))/sn^2;
  xcl = xc+a*cs;
  scl =    a*sn;
  slope = tan(amb);
  [x1,s1,q1]=flsphere_ode(ndiv,smax,capls,xcl,scl,slope);

% obj1=x1(ndiv+1);
  arg1 = s1(ndiv+1)/capl;
  obj1 = x1(ndiv+1) + capl*besselk(0,arg1)/besselk(1,arg1)*q1(ndiv+1);

  beta = beta-eps; % reset

  der  = (obj1-obj)/eps;
  correction = -obj/der;
  beta = beta+correction;

  if(abs(correction)<0.0000001) break; end

end

 if(iter==itermax)
   disp ('flsphere: Newton iterations failed')
 end

 cs = cos(beta);
 sn = sin(beta);
 amb = alpha-beta;
 xc = (2/3*cs^3 +2/3*kappa+2*capls/a^2*sn*sin(amb))/sn^2;

%================================
% (optional)
%
% approximate solution
% based on an inclined flat plate
%================================
 
  iopt1 = 1;

%---
  if(iopt1==1)
%---

  bita = beta;
  eee = 0.001;

  for iter=1:20

   cs  = cos(bita);
   sn  = sin(bita);
   amb = alpha-bita;
   tmp = 0.5*amb;
   cpl = capl/a;
   fnc = cs^3 -3.0*cs-2.0*kappa-6.0*cpl*sn*(cpl*sin(amb)+sin(tmp)*sn);
   bita = bita+eee;
   cs  = cos(bita);
   sn  = sin(bita);
   amb = alpha-bita;
   tmp = 0.5*amb;
   fnc1 = cs^3 -3.0*cs-2.0*kappa-6.0*cpl*sn*(cpl*sin(amb)+sin(tmp)*sn);
   bita = bita-eee;
   der = (fnc1-fnc)/eee;
   cor = - fnc/der;
   bita = bita + cor;
   if(abs(cor)<0.0000001) break; end
  end

  if(iter==20)
   disp ('flsphere: flat-plate iterations failed')
  end

  tmp = 0.5*(alpha-bita);
  xcpl = -2.0*capl*sin(tmp)-a*cos(bita);

% cs   = cos(bita);
% sn   = sin(bita);
% amb  = alpha-bita;
% xcpl = (2/3*cs^3 +2/3*kappa+2*capls/a^2*sn*sin(amb))/sn^2;

% [beta bita xc xcpl]

%---
  end
%---

%================================
% (optional)
%
% approximate solution for a circular cylinder
%================================

  iopt2 = 1;

%---
  if(iopt2==1)
%---

  bita = beta;
  eee = 0.001;

  btc = beta;
  eee = 0.001;
  cpl = capl/a;

 %---
 for it=1:20
 %---
  amb = alpha-btc;
  tmp = 0.5*amb;
  fnc = 8.0*cpl*sin(tmp)*sin(btc)+ pi*(kappa+1.0)-2.0*btc+sin(2*btc) ...
  + 4.0*cpl^2*sin(amb);

  btc = btc+eee;

  amb = alpha-btc;
  tmp = 0.5*amb;
  fnc1 = 8.0*cpl*sin(tmp)*sin(btc)+ pi*(kappa+1.0)-2.0*btc+sin(2*btc) ...
  + 4.0*cpl^2*sin(amb);

  btc = btc-eee;

  der = (fnc1-fnc)/eee;
  corr = -fnc/der;
  btc = btc +corr;

  if(abs(cor)<0.0000001) break; end

 %---
 end
 %---

  if(iter==20)
   disp ('flsphere: cylinder iterations failed')
  end

  tmp = 0.5*(alpha-btc);
  xcc = - 2.0*capl*sin(tmp)-a*cos(btc);

%---
 end
%---

%===========================
% plot the interface profile
%===========================

%---
  if(iplot_shape==1)
%---

  figure(10)
  clf(10)
  hold on
  set(gca,'fontsize',15)
  xlabel('y/a','fontsize',15)
  ylabel('x/a','fontsize',15)
  axis equal
  axis([-2 2 -2 2])
  box on

  plot(s,x,'k');
%  plot(scl,xcl,'go');
  plot(-s,x,'k');
%  plot(-scl,xcl,'go');

%---
% plot the particle contour
%---

 ncrc=64;

 for i=1:ncrc+1
  tht=(i-1)*pi*2/ncrc;
  xcrc(i) = xc+a*cos(tht);
  scrc(i) =   a*sin(tht);
 end

 plot(scrc,xcrc,'k');
 patch(scrc,xcrc,'y');

 pause(0.1)

%=============
end % of plotting
%=============

%====
% record
%===

 if(Irecord==1)
  Jcount = Jcount +1;
  al_scan(Jcount) = alpha/pi;
  bt_scan(Jcount) = beta/pi;
  xc_scan(Jcount) = xc/a;
  bx_scan(Jcount) = bita/pi;
  xx_scan(Jcount) = xcpl/a;
  bk_scan(Jcount) = btc/pi;
  xk_scan(Jcount) = xcc/a;
 end

 if(Iflag==1)
%   pause
   break;
 end

 alpha = alpha+dal;

%-----
% change alpha scanning direction or lock
%-----

 if(alpha<mincut*pi)
  dal = abs(dal);
  Iflag = 1;
 elseif(alpha>maxcut*pi)
  dal = -abs(dal);
  Irecord = 1;
 elseif(abs(alpha-alphain)<1.0*abs(dal))  % lock on alphain
  Iflag=1;
 end

 alpha = alpha+dal;

 if(Icount>Nmax) break; end

%---
end
%---

%---
% done
%---

return
