    close all
    clear all

%-------------------------------------------
% Hydrostatic shape of a 2D meniscus between
% two vertical parallel plates computed
% by the shooting method
%-------------------------------------------

  gac  = 1.0;   % acceleration of gravity
  rhop = 1.0;  % density of the pool
  rhoa = 0.0;  % density of the ambient fluid
  b    = 1.0;  % plate half-separation

  alpha = 0.51*pi;   % contact angle
  npts  = 64;     %  number of interfacial markers
  eps   =0.001; % for the secant method

  maxiter =16; % maximum iterations
  tol = 0.000000001;  % tolerance

  gamma= 2.0; % surface tension
  gamma= 0.1; % surface tension
  gamma= 1.0; % surface tension

  capls = gamma/(gac*abs(rhop-rhoa));  % square of the capillary length
  hmax  = 3.5*capls/b;

  nalpha = 128;
  dalpha = pi/nalpha;
  cut1 = pi-(nalpha-10)*abs(dalpha);
  cut2 =    (nalpha-10)*abs(dalpha);

  h = capls*cos(alpha)/b;

%---
% loop over contact angles
%---

for repeat=1:900

  alpha = alpha+dalpha;

  if(alpha>cut1)
    dalpha=-dalpha;
  end
  if(alpha<cut2)
    dalpha=-dalpha;
  end

  [Iflag,x,y,h] = men_2d ...
 ...
  (b,gac,gamma,rhop,rhoa ...
  ,alpha,npts,eps,maxiter,tol ...
  ,h ...
  );

  d = y(npts+1); % meniscus height

%  [1-sin(alpha) (0.5*d+h)*c/capls]

%---
% shift to reset the origin
% for plotting purposes
%---

  xplot = x;
  yplot = y+h;

%---
% plot
%---

 figure(1)
 plot( xplot,yplot,'-k','LineWidth',3);
 hold on
 plot(-xplot,yplot,'-k','LineWidth',3);
 patch( [xplot b 0 0],[yplot -0.99*hmax -0.99*hmax yplot(1)] ...
       ,'-y','Edgecolor','y')
 patch(-[xplot b 0 0],[yplot -0.99*hmax -0.99*hmax yplot(1)] ...
           ,'-y','Edgecolor','y')
 plot([ b, b],[-hmax,hmax],'r','LineWidth',5)
 plot([-b,-b],[-hmax,hmax],'r','LineWidth',5)
 plot([-2*b,-b],[0,0],'k')
 plot([b,2.0*b],[0,0],'k')

 xlabel('x','fontsize',15)
 ylabel('y','fontsize',15)
 set(gca,'fontsize',15)
 axis equal
 axis([-2.0*b 2.0*b -hmax hmax])

 pause(0.2)
 hold off
 %clf(1)

end

%---
% done
%---
