function [x,x1,x2] = men_2d_ode ...
 ...
      (npts,capls,a,h)

%--------------------------------------------
% Integrate the ODEs by the modified Euler method
% with uniform step size dx
% from x=0 to a
%
% SYMBOLS:
% -------
%
% df/dx = g        (definition)
% dg/dx = (f+h) (1.0+g^2)^1.5 / capls
%
% Integration interval: (0, a)
%
% Initial condition f(0) = 0, g(0) = 0
%--------------------------------------

%--------
% prepare
%--------

 dx = a/npts; % uniform x step

  x(1) = 0.0;
 x1(1) = 0.0;
 x2(1) = 0.0;

%---------------------------------------
% integrate by the modified Euler method
%---------------------------------------

   for i=1:npts

       if(i==1) 
        x1p = 0.0;         % value at mid-plane
        x2p = h/capls;     % value at mid-plane
       else
        x1p = x2(i);
        x2p = (x1(i)+h)*sqrt( (1.0+x2(i)^2)^3 )/capls;
       end

       x1sv = x1(i);    % save
       x2sv = x2(i);
       x1psv = x1p;
       x2psv = x2p;

       x(i+1) = x(i) + dx;
       x1(i+1) = x1(i) + x1p*dx;
       x2(i+1) = x2(i) + x2p*dx;
       x1p = x2(i+1);         % second velocity evaluation
       x2p = (x1(i+1)+h)*sqrt( (1.0+x2(i+1)^2)^3 )/capls;

       x1(i+1) = x1sv + 0.5*(x1psv+x1p)*dx;
       x2(i+1) = x2sv + 0.5*(x2psv+x2p)*dx;

   end

%-----
% done
%-----

return
