function [wr,wi,b]  ...
...
 = men_3d_2p_grid (a,m,xi0,complete)

%=====
 global ab
 global Nxi Neta
 global Dxi Deta
 global xi eta
 global y z J 
 global dydxi dydeta dzdxi dzdeta
%=====

%========
% generate a grid inside a unit cell
% 
% m=6: hexagonal
% m=4: square
%
% a: radius of the inner circle
% b: radius of a hexagonal cell
%
% complete = 1 : will generate grid over
%                a complete cell
% complete = 0 : only 1/m of a cell
%========

Dxi = xi0/Nxi;
Deta = pi/(m*Neta);

if(complete==1)
 Neta = 2*m*Neta;
end

%----
% prepare
%----

im = sqrt(-1);
expn = 2.0/m;

%----
% grid lines in xi-eta space
%
% zeta = xi + i \eta
%----

for i=1:Nxi+2
  xi(i) = Dxi*(i-1.0);
end

for j=1:Neta+2
  eta(j) = Deta*(j-1.0);
end

%----
% w = exp(zeta)
%
% dz/dw = 1/(w^m - 1)^(2/m)
%----

fc = 1.0/exp(xi0);

for j=1:Neta+2
  w(1,j) = fc*exp(xi(1)+im*eta(j)+im*pi/m);
  wr(1,j) = real(w(1,j));
  wi(1,j) = imag(w(1,j));
  v(1,j) = gauss_leg_dr(0.0,w(1,j),20,m);
  wsave = w(1,j);
  for i=2:Nxi+1
   w(i,j) = fc*exp(xi(i)+im*eta(j)+im*pi/m);
   wr(i,j) = real(w(i,j));
   wi(i,j) = imag(w(i,j));
   v(i,j) = v(i-1,j)+gauss_leg_dr(wsave,w(i,j),20,m);
   wsave = w(i,j);
  end
end

for i=1:Nxi+1
  for j=1:Neta+2
    if(i==Nxi+1 & j==Neta+1)
     dvdzeta(i,j) = 0.0;
    else
     dvdzeta(i,j) = w(i,j)/((1.0-w(i,j)^m)^expn);
    end
 end
end

%---
% normalize to make the radius of the inner
% circle equal to "a"
%---

A = a*exp(-im*pi/m)/abs(v(1,1));

for i=1:Nxi+1
  for j=1:Neta+2
   v(i,j) = A*v(i,j);
   y(i,j) = real(v(i,j));
   z(i,j) = imag(v(i,j));

   dvdzeta(i,j) = A*dvdzeta(i,j);

   J(i,j) = abs(dvdzeta(i,j));
   dydxi(i,j) = real(dvdzeta(i,j));
   dzdxi(i,j) = imag(dvdzeta(i,j));
   dydeta(i,j) = -dzdxi(i,j);
   dzdeta(i,j) =  dydxi(i,j);
%   jac1 = abs(    (y(i+1,j)-y(i-1,j))/(2.0*Dxi) ...
%              +im*(z(i+1,j)-z(i-1,j))/(2.0*Dxi));
%   jac2 = abs(    (y(i,j+1)-y(i,j-1))/(2.0*Deta) ...
%              +im*(z(i,j+1)-z(i,j-1))/(2.0*Deta));
%   [jac jac1 jac2]
%   pause
  end
end

%---
% polygonal radius
%---

b = abs(v(Nxi+1,Neta+1));

%---
% done
%---

%---
return
%---
