    close all
    clear all

%-------------------------------------------
% Hydrostatic shape of an axisymmetric meniscus 
% in a vertical tube
% by a shooting method
%-------------------------------------------

  gac   = 1.0;   % acceleration of gravity
  rhop  = 1.0;  % density of the pool
  rhoa  = 0.0;  % density of the ambient fluid
  a     = 1.0;   % tube radius

  alpha = 0.10*pi;   % contact angle
  alpha = 0.51*pi;   % contact angle

  npts  = 64;     %  number of interfacial markers
  eps   =0.001;   % for the secant method

  maxiter =16; % maximum iterations
  tol = 0.000000001;  % tolerance

  gamma = 0.1; % surface tension
  gamma = 0.5; % surface tension
  gamma = 0.2; % surface tension
  gamma = 2.0; % surface tension

  capls = gamma/(gac*abs(rhop-rhoa));  % square of the capillary length
  capl  = sqrt(capls);
  hmax   = 2.0*capls/a;
  hmaxa  = hmax+a;

  nalpha = 2*64;
  dalpha = pi/nalpha;
  cut1 = pi-dalpha;
  cut2 =    dalpha;

  h = 2.0*capls*cos(alpha)/a;

%---------------
% loop over contact angles
%---------------

for repeat=1:200

  alpha = alpha+dalpha;

  h = 2.0*capls*cos(alpha)/a;

  if(alpha<cut1)
    dalpha=-dalpha;
  end
  if(alpha>cut2)
    dalpha=-dalpha;
  end

  [Iflag,x,s,h] = men_ax ...
 ...
  (a,gac,gamma,rhop,rhoa ...
  ,alpha,npts,eps,maxiter ...
  ,tol ...
  ,h ...
  );

  d = x(npts+1);

%---
% J integral
%---

  J = 0.0;

  for i=1:npts
    fp = (x(i+1)-x(i))/(s(i+1)-s(i));
    sm = 0.5*(s(i+1)+s(i));
    J = J + 1.0/sm*fp/sqrt(1+fp^2)*abs(x(i+1)-x(i));
  end

% [1-sin(alpha) (0.5*d+h)*d/capls-J]

%---
% shift to reset the origin
% for plotting purposes
%---

  xplot = x+h;

%---
% plot
%---

 figure(1)
 plot( s,xplot,'-k','LineWidth',3);
 hold on
 plot(-s,xplot,'-k','LineWidth',3);
 patch( [s a 0 0],[xplot -0.99*hmaxa -0.99*hmaxa xplot(1)],'-y','Edgecolor','y')
 patch(-[s a 0 0],[xplot -0.99*hmaxa -0.99*hmaxa xplot(1)],'-y','Edgecolor','y')
 plot([ a, a],[-hmaxa,hmaxa],'k','LineWidth',5)
 plot([-a,-a],[-hmaxa,hmaxa],'k','LineWidth',5)
 plot([-2*a,-a],[0,0],'k')
 plot([a,2.0*a],[0,0],'k')
 hold off

 xlabel('y/a','fontsize',15)
 ylabel('x/a','fontsize',15)
 set(gca,'fontsize',15)
 axis equal
 axis([-2.0*a 2.0*a -hmaxa hmaxa])

 pause(0.1)

end

%---
% done
%---
