%====
clear all
close all
%====

colormap('bone')

%====
% axisymmetric meniscus in the exterior of
% a circular cylinder
%====

Iplot_shapes = 0;
Iplot_shapes = 1;

a = 1.0;  % cylinder radius

%----
% for numerical differentiation
%----

eps = 0.0001;
eps = 0.01;
eps = 0.001;

%---
% for numerical integration 
%---

ndiv = 4*4*4*128;

%~~~~~~~~~~~~~~
%for loopcangle = 1:5   % over contact angle
%for loopcangle = 3:3   % over contact angle
%for loopcangle = 1:1   % over contact angle
%for loopcangle = 2:2   % over contact angle
 for loopcangle = 4:4   % over contact angle
%~~~~~~~~~~~~~~

if(loopcangle==1)
 cangle = 0.02*pi;
elseif(loopcangle==2)
 cangle = 0.125*pi;
elseif(loopcangle==3)
 cangle = 0.250*pi;
elseif(loopcangle==4)
 cangle = 0.375*pi;
elseif(loopcangle==5)
 cangle = 0.48*pi;
end

slope = -1.0/tan(cangle);

%---
if(Iplot_shapes==1)
%---

 figure(loopcangle)

 aa = 5.0*a;
 if(cangle<0.5*pi)
  patch([0, a, a, 0],[0, 0, aa, aa],0.1)
 else
  patch([0, a, a, 0],[-aa, -aa, 0, 0],0.1)
 end
 hold on

%---
end
%---

%~~~~~
% guess the capillary height
%~~~~~

h = 1.0*a;

if(cangle>0.5*pi) h=-h; end

%~~~~~
% loop over the capillary length
%~~~~~

capl = 8.0;
capl = 128.0;

%~~~~~~~~~~~~~
%for loopcap = 1:1
 for loopcap = 1:4*8
%~~~~~~~~~~~~~

 if(loopcap>1)
  capl = capl*0.8;
 end

 capls = capl*capl;

 % integration domain

 smax = 4.0*capl;
 cut = 32.0*a;
 if(smax>cut) smax=cut; end

 cut = 2.0*a;
 if(smax<cut) smax=cut; end

 [f,s,q] = ode(ndiv,smax,capls,h,a,slope);

 Icon = 0;

%---
for newton =1:10
%---

%  obj = f(ndiv+1);
   arg = s(ndiv+1)/capl;
   obj = f(ndiv+1) + capl*besselk(0,arg)/besselk(1,arg)*q(ndiv+1);

   [f,s,q] = ode(ndiv,smax,capls,h+eps,a,slope);

%  obj1 = f(ndiv+1);
   arg = s(ndiv+1)/capl;
   obj1 = f(ndiv+1) + capl*besselk(0,arg)/besselk(1,arg)*q(ndiv+1);

   der = (obj1-obj)/eps;

   hsave = h;
   h = h - obj/der;

   [f,s,q] = ode(ndiv,smax,capls,h,a,slope);

   error = abs(hsave-h);

   if(error<0.00001)
    Icon = 1;
    break
   end

%---
end
%---

 error
  
 if(Icon==0) 
    break;
 end

%---
 if(Iplot_shapes==1)
%---

  figure(loopcangle)

  if(loopcap==1)
   plot(s,f,'k');
  else
   plot(s,f,'k');
  end
%plot(a,h,'o')
  plot([0, smax],[0 0],'k:')

%---
 end
%---

 h2d = capl*sqrt(2.0*(1.0-sin(cangle)) );
 if(cangle>0.5*pi) h2d=-h2d; end

%plot(a,h2d,'+')

%---
% save for plotting
%---

 CAP(loopcap) = capl;
 HEIGHT(loopcap) = h;

 [capl h]

%--- ignore please
%
% mdiv = ndiv-4;
% for i=1:mdiv
%  ratio(i) = f(i)/q(i);
%  arg = s(i)/capl;
%  besse0 = besselk(0,arg);
%  besse0p = -besselk(1,arg);
%  besse(i) = capl*bess0/besse0p;
% end
%  figure(45)
%  hold on
 %  plot(s(1:mdiv),ratio)
 %  plot(s(1:mdiv),besse,'--')
%  plot(s(1:mdiv),besse0/8,'--')
%  plot(s,f,'r--')
%  capl
%--- end of ignore please

%~~~
end % of loop over cap
%~~~

%---
if(Iplot_shapes==1)

 figure(loopcangle)
 hold on

 plot([a, a],[0 aa],':')
 box on
 xlabel('\sigma/a')
 ylabel('x/a')
 axis equal

 bb  = 10.0*a;
 if(cangle<0.5*pi)
  axis([0 bb 0 aa])
 else
  axis([0 bb -aa 0])
 end

end
%---

%---
% flat plate solution
%---

capl2d = 0.01;

for i=1:32
 h2d = capl2d*sqrt(2.0*(1.0-sin(cangle)) );
 if(cangle>0.5*pi) h2d=-h2d; end
 CAP2D(i) = capl2d;
 HGT2D(i) = h2d;
 capl2d = capl2d*1.5;
 if(capl2d>1.0) break; end
end

%---
% asymptotic solution
%---

capl2dj = 1.00;

for i=1:32
 tmp = 4.0*capl2dj/(1.0+sin(cangle));
 h2dj = cos(cangle)*(log(tmp)-0.57721);
 CAP2DJ(i) = capl2dj;
 HGT2DJ(i) = h2dj;
 capl2dj = capl2dj*1.5;
end

%---
% plot h
%---

 figure(99)
 hold on

 plot(log10(CAP),HEIGHT,'k')
 plot(log10(CAP2D),HGT2D,'k:')
 plot(log10(CAP2DJ),HGT2DJ,'k--')
 xlabel('log(l/a)','fontsize',15);
 ylabel('h/a','fontsize',15);
 box on
 axis([-1 2 0 4])

%---
% clear for the next loop over the contact angle
%---

 clear CAP
 clear HEIGHT
 clear CAP2D
 clear HEIGHT2D

%======
end % of loopcangle
%======
