      program chan_2d_dr

c=========================================
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=========================================

c=========================================
c Flow in a two-dimensional channel
c confined between two parallel plane walls
c located at y = +-a
c
c SYMBOLS:
c -------
c
c nprof: number of points for a velocity profile
c a:     channel semi-width
c visc:  fluid viscosity
c den:   fluid density
c gac:   acceleration of gravity
c V1:    lower wall velocity
c V2:    upper wall velocity
c chi:   negative of the pressure gradient
c beta:  channel inclination angle
c=========================================

      Implicit Double Precision (a-h,o-z)

c----------
c constants
c----------

      pi = 3.14159 265358

      Null = 0
      None = 1
      Ntwo = 2

      zero = 0.0

c------
c input
c------

      open (1,file="chan_2d.dat",status="unknown")
        read (1,*) nprof
        read (1,*) a
        read (1,*) visc
        read (1,*) den
        read (1,*) gac
        read (1,*) V1
        read (1,*) V2
        read (1,*) chi
        read (1,*) beta
      close (1)

c--------
c prepare
c---------

      open (2,file="chan_2d.out",status="unknown")

      nprof1 = nprof+1

      beta = pi*beta

      cth0  = cos(beta)
      sth0  = sin(beta)

c--------------------------------------------
c Draw the walls so that they appear inclined
c--------------------------------------------

      write (2,900) Ntwo
      xx = -a
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  a
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

      write (2,900) Ntwo
      xx = -a
      yy =  a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  a
      yy =  a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c---------------------
c Draw a vertical line
c across the channel
c---------------------

      write (2,900) Ntwo
      xx =  zero
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = zero
      yy = a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c-----------------
c Draw the profile
c-----------------

      write (6,*) 
      write (6,*) "Velocity Profile"
      write (6,*) 

      write (2,900) nprof1

      y = -a

      step = 2.0*a/(nprof1-1.0)

      Do i=1,nprof1

       call chan_2d 
     +
     +   (a
     +   ,visc,den
     +   ,gac
     +   ,V1,V2
     +   ,chi
     +   ,beta
     +   ,y
     +   ,flow_rate
     +   ,umean
     +   ,u
     +   )

        write (6,900) i,y,u

        xx = u
        yy = y

        xprint = xx*cth0+yy*sth0
        yprint =-xx*sth0+yy*cth0

        write (2,900) i,xprint,yprint
c       write (6,900) i,xprint,yprint

        y = y+step

      End Do

      write (2,900) null
      close (2)

c----------------------------
c flow rate and mean velocity
c----------------------------

      write (6,*) 
      write (6,901) flow_rate,umean
      write (6,*) 

c-----
c Done
c-----

 900  Format (1x,i3,4(1x,f10.5))
 901  Format ("Flow rate     = ",f15.10
     +      ,/,"Mean velocity = ",f15.10
     +       )

      Stop 
      End
