      subroutine chan_2d_2l 
     +
     +  (a
     +  ,h1
     +  ,visc1
     +  ,visc2
     +  ,rho1
     +  ,rho2
     +  ,gac
     +  ,V1
     +  ,V2
     +  ,chi
     +  ,beta
     +  ,y
     +  ,yI
     +  ,uI
     +  ,xi1,xi2
     +  ,Q1,Q2
     +  ,u
     +  )

c=====================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=====================================

c=====================================
c Flow of two superposed layers in a channel
c of width 2a
c
c The channel walls are located at y = +-a
c
c Lower fluid is labeled 1, ipper fluid is labeled 2
c
c The velocity is expressed as:
c
c  u = uI + xi*(y-yI) - gf*(y-yI)^2
c
c SYMBOLS:
c -------
c
c h1:   thickness of the lower layer
c h2:   thickness of the upper layer
c a:    channel semi-width
c yI:   location of the interface
c ui:   interfacial velocity
c chi:  negative of the pressure gradient
c=====================================

      Implicit Double Precision (a-h,o-z)

c---
c constants
c---

      pi = 3.14159 265358 D0

c--------
c prepare
c--------

      gx = gac*sin(beta)

      viscr = visc2/visc1   ! viscosity ratio
      densr = rho2/rho1     ! density ratio

      h2  = 2.0*a-h1     ! thickness of the upper layer
      hr = h2/h1         ! thickness ratios

      yI  = h1-a         ! location of the interface

c---------------------
c interfacial velocity
c---------------------

      uI = 0.5*h1*h2/(viscr+hr)/visc1
     +*(chi*(1.0+hr) + rho1*gx*(1.0+densr*hr))
     +       +(hr*V1+viscr*V2)/(viscr+hr)

c------------------------
c interfacial shear rates
c------------------------

      xi1 =-0.5*(chi+rho1*gx)*h1/visc1 + (uI-V1)/h1
      xi2 = 0.5*(chi+rho2*gx)*h2/visc2 - (uI-V2)/h2

c---------------------------
c modified pressure gradient
c---------------------------

      gf1 = 0.5*(chi+rho1*gx)/visc1
      gf2 = 0.5*(chi+rho2*gx)/visc2

c-----------
c flow rates
c-----------

      Q1 = uI*h1 - 0.5*xi1*h1*h1 - gf1*h1*h1*h1/3.0
      Q2 = uI*h2 + 0.5*xi2*h2*h2 - gf2*h2*h2*h2/3.0

c-------------------
c velocity at point y
c-------------------

      if(y.lt.yI) then                       ! lower fluid
        u = uI + xi1*(y-yI) - gf1*(y-yI)*(y-yI)
      else                                   ! upper fluid
        u = uI + xi2*(y-yI) - gf2*(y-yI)*(y-yI)
      end if

c-----
c done
c-----

      return
      end
