      subroutine chan_2d_osc
     +
     +   (h
     +   ,visc
     +   ,den
     +   ,Vamp
     +   ,omega
     +   ,ot
     +   ,y
     +   ,u
     +   )

c===========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c===========================================

c===========================================
c Flow in a channel due to an oscillating wall
c
c The walls are located at y = 0, h
c
c The lower wall is oscillating with amplitude: Vamp
c so that the wall velocity is:
c
c U_wall = Vamp * cos(omega*time)
c===========================================

      Implicit Double Precision (a-h,o-z)

c----------
c constants
c----------

      pi = 3.14159 265358 D0

c--------
c prepare
c--------

      rnu   = visc/den             ! kinematic viscosity
      delta = Dsqrt(2.0D0*rnu/omega)
      cs = Dcos(ot)
      sn = Dsin(ot)
      hr   = h/delta                ! reduced height
      hr2  = 2.0*hr
      tmp1 = exp(-hr2)
      denr = 1.0-tmp1* cos(hr2)     ! real part of the denominator
      deni =    -tmp1* sin(hr2)     ! imag part of the denominator
      dens = denr*denr+deni*deni
      aar  =  denr/dens             ! real part of 1/denominator
      aai  = -deni/dens             ! imag part of 1/denominator

c---
c compute the velocity
c---

      yr   = y/delta
      tmp1 = exp(-yr)
      tmp2 = 2.0*hr-yr
      tmp3 = exp(-tmp2)
      bbr  = tmp1*cos(yr)-tmp3*cos(tmp2)
      bbi  = tmp1*sin(yr)-tmp3*sin(tmp2)
      ccr  = aar*bbr-aai*bbi
      cci  = aar*bbi+aai*bbr

      u = Vamp*( cs*ccr + sn*cci)

c-----
c Done
c-----

      Return
      End
