clear all
close all

%-----------------------------------------
% FDLIB
%
% Copyright by C. Pozrikidis, 1999
% All rights reserved
%
% This program is to be used only under the
% stipulations of the licensing agreement
%----------------------------------------

%=========
% path_lines
%
% draw path lines in a specified flow
% the velocity is provided by the function path_lines_vel
%=========

Nsteps = 400; % number of steps

Dl = 0.01;  % spatial step

xmin =-1.0;  % plotting window
xmax = 1.0;
ymin =-1.0;
ymax = 1.0;

Xstart = [0.1, 0.2, 0.3];  % starting points
Ystart = [0.1, 0.2, 0.3];

%---
% prepare to plot
%---

figure(1)
hold on
axis equal
set(gca,'fontsize',13)
box on
xlabel('x','fontsize',13)
ylabel('y','fontsize',13)

%---
for n=1:size(Xstart')    % run over initial points
%---

 X0 = Xstart(n);
 Y0 = Ystart(n);

 Xsave = X0;
 Ysave = Y0;

 clear Xplot Yplot

 Xplot(1) = X0;
 Yplot(1) = Y0;

 X = X0; Y = Y0; t=0;

%--
 for i=1:Nsteps
%--

  [ux, uy] = path_lines_vel(X,Y,t);

  um = sqrt(ux*ux+uy*uy);

  Dt = Dl/um;

  X1 = X+ux*Dt;
  Y1 = Y+uy*Dt;
  t1 = t+Dt;

  [ux1, uy1] = path_lines_vel(X1,Y1,t1);

  X = X + 0.5*Dt*(ux+ux1);
  Y = Y + 0.5*Dt*(uy+uy1);
  t = t+Dt;

  Xplot(i+1) = X;
  Yplot(i+1) = Y;

%--- stopping checks:

  closed = sqrt((X-Xsave)^2+(Y-Ysave)^2);  % check for a closed line

  if(closed < 0.9*Dl | X<xmin |X>xmax |Y<ymin|Y>ymax) 
   break
  end

end

plot(Xplot,Yplot,'k.')

%---
end  % of path lines
%---

