function spf2d

%---
% flow over a rotating disk
% using the intrinscif
% function bvp4c
%---

clear all
close all

%---
% mesh
%---

Ndiv = 128;
etamax = 10.0;

deta = etamax/Ndiv;

for i=1:Ndiv+1
  eta(i) = (i-1.0)*deta;
end

solinit = bvpinit(eta, @initialize);
solinit0 = solinit;
options = bvpset('RelTol',1e-6);

%---
% plot
%---

 sol = bvp4c(@odefun,@bcfun,solinit,options);
 solinit =sol;

 eta = sol.x(1,:);
 H   = sol.y(1,:);
 Hp  = sol.y(2,:);
 Hpp = sol.y(3,:);
 G   = sol.y(4,:);
 Gp  = sol.y(5,:);

 Hpp(1)
  Gp(1)

 F = -0.5*Hp;
 Q =  0.5*H.^2-Hp;

%---
% plot
%---

 figure(1)
 plot(eta,-H,  'k')
 hold on
 plot(eta,G,'k--')
 plot(eta,F,'k:')
 plot(eta,Q,'k-.')
 axis([0 5 0 1])
 set(gca,'fontsize',15)
 xlabel('\eta','fontsize',15)
 ylabel(' ','fontsize',15)

%---
% done
%---

return

%----
% initialize
%----

function v = initialize(eta)

  v = [ 0.0
        0.0
        0.0 
        0.0
        0.0
       ];

return

%---
% odes
%---

function dydx = odefun(l,x)

beta = 0.647900;

dydx = [x(2)
        x(3)
        x(1)*x(3) - 0.5*x(2)^2 + 2.0*x(4)^2
        x(5)
        x(1)*x(5) - x(2)*x(4)
        ];

return

%---
% boundary conditions
%---

function res = bcfun(xa,xb)

res = [ xa(1) - 0.0
        xa(2) - 0.0
        xa(4) - 1.0
        xb(2) - 0.0
        xb(4) - 0.0
       ];

return
