function spf2d

%---
% 2D stagnation-point flow
% using pvp4c
%---

clear all
close all

%---
% mesh
%---

Ndiv = 128;
etamax = 7.0;

deta = etamax/Ndiv;

for i=1:Ndiv+1
  eta(i) = (i-1.0)*deta;
end

solinit = bvpinit(eta, @initialize);
solinit0 = solinit;
options = bvpset('RelTol',1e-8);

%---
% plot
%---

figure(1)
axis equal
box
plot([-1.5 1.5],[0,0],'k')
hold on

 sol = bvp4c(@odefun,@bcfun,solinit,options);
 solinit =sol;

 eta = sol.x(1,:);
 F   = sol.y(1,:);
 Fp  = sol.y(2,:);
 Fpp = sol.y(3,:);
 G   = sol.y(4,:);
 Gp  = sol.y(5,:);
 Gpp = sol.y(6,:);

 format long
 Fpp(1)
 Gpp(1)
 format short

%---
% plot
%---

 figure(1)
 plot(eta,F,  'k')
 hold on
 plot(eta,Fp, 'k--')
 plot(eta,Fpp,'k:')
 axis([0 3 0 1.5])
 set(gca,'fontsize',15)
 xlabel('\eta','fontsize',15)
 ylabel(' ','fontsize',15)

 figure(2)
 plot(eta,G,  'k')
 hold on
 plot(eta,Gp, 'k--')
 plot(eta,Gpp,'k:')
 axis([0 3 0 3.0])
 set(gca,'fontsize',15)
 xlabel('\eta','fontsize',15)
 ylabel(' ','fontsize',15)

%---
% done
%---

return

%========================================

%----
% initialize
%----

function v = initialize(eta)

  v = [ eta-0.647900
        1.0
        0.0 
        eta
        1.0
        0.0
       ];

return

%========================================

%---
% odes
%---

function dydx = odefun(l,x)

beta = 0.647900;

dydx = [x(2)
        x(3)
        x(2)^2-x(1)*x(3)-1.0
        x(5)
        x(6)
       -x(1)*x(6)+x(2)*x(5)-beta
        ];

return

%========================================

%---
% boundary conditions
%---

function res = bcfun(xa,xb)

res = [ xa(1) - 0.0
        xa(2) - 0.0
        xa(4) - 0.0
        xa(5) - 0.0
        xb(2) - 1.0
        xb(6) - 1.0
       ];

return
