      subroutine sgf_2d_2p 
     +
     +  (Iopt
     +  ,x,y
     +  ,x0,y0
     +  ,a11,a12,a21,a22
     +  ,b11,b12,b21,b22
     +  ,ew,area
     +  ,Max1,Max2
     +  ,gxx,gxy
     +  ,gyx,gyy
     +  ,px,py
     +  ,txxx,txxy,tyxx,tyxy
     +  ,txyx,txyy,tyyx,tyyy
     +  )
 
c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c---------------------------------------------------
c Doubly-periodic Green's function  of two-dimensional
c Stokes flow
c
c Iopt = 1 computes G
c      = 2 computes G, p, and T
c
c SYMBOLS
c -------
c
c Gij:     Green's function for the velocity
c Tijk:    Green's function for the stress
c
c a_ij:    lattice base vectors
c b_ij:    wave number base vectors
c
c---------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Double Precision k1,k2,ks

      Dimension qxx (-15:15,-15:15)
      Dimension qxy (-15:15,-15:15)
      Dimension qyy (-15:15,-15:15)
      Dimension vxxx(-15:15,-15:15)
      Dimension vxxy(-15:15,-15:15)
      Dimension vyxy(-15:15,-15:15)
      Dimension vxyx(-15:15,-15:15)
      Dimension vxyy(-15:15,-15:15)
      Dimension vyyy(-15:15,-15:15)
      Dimension ppx (-15:15,-15:15)
      Dimension ppy (-15:15,-15:15)

c--------------
c common blocks
c--------------

      common/qqqq_2d/qxx,qxy,qyy
      common/vvvv_2d/vxxx,vxxy,vyxy,vxyx,vxyy,vyyy,ppx,ppy

c----------
c constants
c----------

      pi   = 3.14159 265358 D0
      piq  = 0.25D0*pi
      pih  = 0.50D0*pi
      pi2  = 2.00D0*pi
      pi4  = 4.00D0*pi
      pi6  = 6.00D0*pi
      pi8  = 8.00D0*pi
      srpi = sqrt(pi)

c---
c Prepare to run
c---

      dxx = x-x0
      dyy = y-y0

      ews = ew**2
 
c-----------------------------------
c move as close to the singular
c point as possible
c by shifting along the base vectors
c-----------------------------------

      Imove = nint(dyy)
      dxx = dxx - Imove*a21   ! move along the second base vector
      dyy = dyy - Imove*a22

      Imove = nint(dxx)
      dxx = dxx - Imove*a11   ! move along the first base vector
      dyy = dyy - Imove*a12

c-----------
c initialize
c-----------

      Gxx = 0.0D0
      Gxy = 0.0D0
      Gyy = 0.0D0

      If(Iopt.ne.1) then

        px = 0.0D0
        py = 0.0D0

        txxx = 0.0D0
        txxy = 0.0D0
        tyxx = 0.0D0
        tyxy = 0.0D0

        txyx = 0.0D0
        txyy = 0.0D0
        tyyx = 0.0D0
        tyyy = 0.0D0

      End If

c------------------
c sum in real space  
c------------------

      Do 92 i1=-Max1,Max1
      Do 92 i2=-Max1,Max1

        dx = dxx-i1*a11-i2*a21
        dy = dyy-i1*a12-i2*a22

        r2 = dx**2+dy**2
        r  = sqrt(r2)
        W  = ew*r
        W2 = W**2

        Expp  = exp(-W2)
        cc    = 0.5*expint(W2)-Expp
        ddri2 = Expp/r2

        Gxx = Gxx + cc + dx**2 * ddri2
        Gxy = Gxy +      dx*dy * ddri2
        Gyy = Gyy + cc + dy**2 * ddri2

c-------
        If(Iopt.eq.1) Go to 92
c-------

        fc = 2.0D0*Expp/r2

        px = px + dx*fc
        py = py + dy*fc

        dx2 = dx**2
        dy2 = dy**2

        fc1 = 2.0D0 * ews * Expp
        fc2 = 4.0D0 * ews/r2 * (1.0D0+1.0D0/W2)*Expp

        Txxx = Txxx + (dx+dx)*fc1 - dx2*dx*fc2
        Txxy = Txxy +  dy    *fc1 - dx2*dy*fc2
        Tyxy = Tyxy               - dy2*dx*fc2

        Txyx = Txyx               - dx2*dy*fc2
        Txyy = Txyy +  dx    *fc1 - dy2*dx*fc2
        Tyyy = Tyyy + (dy+dy)*fc1 - dy2*dy*fc2

  92  Continue

c---------------------------
c subtract off the Stokeslet
c if desired
c---------------------------
c
c     drs = dxx**2+dyy**2
c     rll = 0.5*log(drs)
c
c     Gxx = Gxx - (-rll+dxx**2 /drs)
c     Gxy = Gxy - (     dxx*dyy/drs)
c     Gyy = Gyy - (-rll+dyy**2 /drs)
c
c---------------------------

c------------------------
c sum in wavenumber space
c------------------------

      exx = 0.0D0
      exy = 0.0D0
      eyy = 0.0D0

      If(Iopt.ne.1) then

        wwx = 0.0D0
        wwy = 0.0D0

        wxxx = 0.0D0
        wxxy = 0.0D0
        wyxy = 0.0D0

        wxyx = 0.0D0
        wxyy = 0.0D0
        wyyy = 0.0D0

      End If

      Do 93 i1=-Max2,Max2
       Do 93 i2=-Max2,Max2

        k1 = i1*b11 + i2*b21
        k2 = i1*b12 + i2*b22

        ks = k1**2 + k2**2

        If(ks.le.0.0000001) Go to 93   ! skip the zero wavenumber

        arg = k1*dxx+k2*dyy

        f = cos(arg)

        exx = exx + qxx(i1,i2)*f
        exy = exy + qxy(i1,i2)*f
        eyy = eyy + qyy(i1,i2)*f

c---
      If(Iopt.eq.1) Go to 93
c---

        f = sin(arg)

        wwx = wwx + ppx(i1,i2)*f
        wwy = wwy + ppy(i1,i2)*f

        wxxx = wxxx + vxxx(i1,i2)*f
        wxxy = wxxy + vxxy(i1,i2)*f
        wyxy = wyxy + vyxy(i1,i2)*f

        wxyx = wxyx + vxyx(i1,i2)*f
        wxyy = wxyy + vxyy(i1,i2)*f
        wyyy = wyyy + vyyy(i1,i2)*f

  93  Continue

      fc = pi4/area

      Gxx = Gxx + exx*fc
      Gxy = Gxy + exy*fc
      Gyx = Gxy
      Gyy = Gyy + eyy*fc

c--------------
c stress tensor
c--------------

      If(Iopt.ne.1) then

      dxx = x-x0    ! important: do not remove
      dyy = y-y0    ! important: do not remove

      wwx = (wwx+dxx)*fc   ! add also the mean pressure gradient
      wwy = (wwy+dyy)*fc   ! add also the mean pressure gradient

      Txxx = Txxx + wxxx*fc - wwx
      Txxy = Txxy + wxxy*fc
      Tyxx = Txxy
      Tyxy = Tyxy + wyxy*fc - wwx

      Txyx = Txyx + wxyx*fc - wwy
      Txyy = Txyy + wxyy*fc
      Tyyx = Txyy
      Tyyy = Tyyy + wyyy*fc - wwy

      px = px + wwx
      py = py + wwy

      End If

c-----
c Done
c-----

      Return
      End

c================================================

      subroutine qqq_2d 
     +
     +  (b11,b12
     +  ,b21,b22
     +  ,Max2
     +  ,ew
     +  )

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-------------------------------------------------
c q_ij is an array for summing in reciprocal space
c
c Used to compute the velocity Green's function
c-------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Double Precision k1,k2,kk,ks

      Dimension qxx(-15:15,-15:15)
      Dimension qxy(-15:15,-15:15)
      Dimension qyy(-15:15,-15:15)

      common/qqqq_2d/qxx,qxy,qyy

c-----------------------------------
c scan the reciprocal lattice points
c-----------------------------------

      Do i1=-Max2,Max2
       Do i2=-Max2,Max2

       k1 = i1*b11 + i2*b21
       k2 = i1*b12 + i2*b22

       ks = k1**2+k2**2

       If(ks.gt.0.000001) then   ! skip the zero wavenumber

        kk = sqrt(ks)
        t  = kk/ew
        ts = t*t

        arg = -0.25D0 * ts
        CF = (1.0D0 + 0.25D0*ts)*exp(arg)/ks

        k1 = k1/kk
        k2 = k2/kk

        qxx(i1,i2) = (1.0D0 -k1**2)*CF
        qxy(i1,i2) =        -k1*k2 *CF
        qyy(i1,i2) = (1.0D0 -k2**2)*CF

       End If

       End Do
      End Do

c-----
c Done
c-----

      Return
      End

c================================================

      subroutine vvv_2d 
     +
     +   (b11,b12
     +   ,b21,b22
     +   ,Max2
     +   ,ew
     +   )

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c--------------------------------------------------
c v_ijl is an array for summing in reciprocal space
c used to compute the stress Green's function
c
c p_i is an array for summing in reciprocal space
c used to compute the pressure Green's function
c--------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Double Precision k1,k2,kk,ks

      Dimension vxxx(-15:15,-15:15)
      Dimension vxxy(-15:15,-15:15)
      Dimension vyxy(-15:15,-15:15)

      Dimension vxyx(-15:15,-15:15)
      Dimension vxyy(-15:15,-15:15)
      Dimension vyyy(-15:15,-15:15)

      Dimension ppx (-15:15,-15:15)
      Dimension ppy (-15:15,-15:15)

      common/vvvv_2d/vxxx,vxxy,vyxy,vxyx,vxyy,vyyy,ppx,ppy

c-----------------------------------
c scan the reciprocal lattice points
c-----------------------------------

      Do i1=-Max2,Max2
       Do i2=-Max2,Max2

       k1 = i1*b11 + i2*b21
       k2 = i1*b12 + i2*b22

       ks = k1**2 + k2**2 

        If(ks.gt.0.000001) then    ! skip the zero wavenumber

        kk = sqrt(ks)
        t  = kk/ew
        ts = t**2

        arg = -0.25D0*ts
        EXPP = exp(arg)

        CF1 = (1.0+0.25*ts)*EXPP/kk
        CF2 = EXPP/kk

        k1 = k1/kk
        k2 = k2/kk

        ppx(i1,i2) = CF2*k1
        ppy(i1,i2) = CF2*k2

        vxxx(i1,i2) = (2.0*k1*k1*k1 -k1-k1)*CF1
        vxxy(i1,i2) = (2.0*k1*k1*k2 -k2   )*CF1
        vyxy(i1,i2) = (2.0*k2*k1*k2       )*CF1

        vxyx(i1,i2) = (2.0*k1*k2*k1       )*CF1
        vxyy(i1,i2) = (2.0*k1*k2*k2 -k1   )*CF1
        vyyy(i1,i2) = (2.0*k2*k2*k2 -k2-k2)*CF1

       End If

       End Do
      End Do

c-----
c Done
c-----

      Return
      End

c============================================

      subroutine ewald_2d 
     +
     +   (a11,a12,a21,a22
     +   ,b11,b12,b21,b22
     +   ,ew,area
     +   )
 
c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c----------------------------------------
c Computes the reciprocal lattice vectors
c          the cell area
c          xi (ew) according to Beennakker
c-----------------------------------------

      Implicit Double Precision (a-h,o-z)

c----------
c constants
c----------
 
      pi   = 3.14159 265358
      pi2  = 2.0*pi
      srpi = sqrt(pi)

c----------
c cell area
c----------

      area = abs(a11*a22-a12*a21)

c-----------------------------------------
c lattice base vector in wave number space 
c-----------------------------------------

      cf = pi2/area

      b11 =  cf*a22
      b12 = -cf*a21
      b21 = -cf*a12
      b22 =  cf*a11

      ew = srpi/sqrt(area)

c--------------------------
c     write (6,*) " Reciprocal Lattice "
c     write (6,*) " ------------------ "
c     write (6,104) b11,b12
c     write (6,104) b21,b22
c     write (6,114) area,ew
c--------------------------

c-----
c Done
c-----

 104  Format (6(1x,f10.5))
 114  Format (2x," cell area = ",f10.5," ewald = ",f10.5)

      Return
      End
