      program tube_crc_dr

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c==========================================

c====================================
c Steady flow through a circular tube
c due to gravity or pressure gradient
c====================================

      Implicit Double Precision (a-h,o-z)

c---
c constants
c---

      Null = 0
      None = 1
      Ntwo = 2

      zero = 0.0

c---
c input
c---

      open  (1,file="tube_crc.dat",status="unknown")

       read (1,*) nprof   ! number of points for a profile
       read (1,*) a       ! tube radius
       read (1,*) visc    ! viscosity
       read (1,*) den     ! density
       read (1,*) gac     ! acceleration of gravity
       read (1,*) chi     ! negative of the pressure gradient
       read (1,*) beta    ! inclination angle

      close (1)

c---
c prepare
c---

      open (2,file="tube_crc.out",status="unknown")

      beta = pi*beta

      nprof1 = nprof+1
      step = a/(nprof1-1.0)

c---
c draw the wall
c---

       write (2,900) Ntwo
       write (2,900) None,-a,a
       write (2,900) Ntwo, a,a

c---
c draw the centerline
c---

       write (2,900) Ntwo
       write (2,900) None,-a,zero
       write (2,900) Ntwo, a,zero

c---
c draw a vertical line
c---

       write (2,900) Ntwo
       write (2,900) None,zero,zero
       write (2,900) Ntwo,zero,a

c---
c draw the profile
c---

      write (6,*)
      write (6,*) " Velocity profile"
      write (6,*)

      write (2,900) nprof1

      sigma = 0.0

      Do i=1,nprof1

      call tube_crc
     +
     +   (a
     +   ,visc
     +   ,den
     +   ,gac
     +   ,chi
     +   ,beta
     +   ,sigma
     +   ,flow_rate
     +   ,umean
     +   ,umax
     +   ,u
     +   )

       write (2,900) i,u,sigma
       write (6,900) i,u,sigma

       sigma = sigma + step

      End Do

c------
c print
c------

      write (6,*)
      write (6,901) flow_rate,umean,umax
      write (6,*)

c-----
c Done
c-----

      write (2,900) Null
      close (2)

 900  Format (1x,i3,4(1x,f10.5))

 901  Format ("Flow rate     = ",f15.10
     +     ,/,"Mean velocity = ",f15.10
     +     ,/,"Max  velocity = ",f15.10
     +       )

      Stop
      End
