      subroutine tube_crc_sw_trans
     +
     +  (a
     +  ,visc
     +  ,den
     +  ,omega
     +  ,time
     +  ,s
     +  ,u
     +  )

c=========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=========================================

c=========================================
c Transient swirling flow inside a circular cylinder
c due to the sudden rotation of the cylinder
c with constant angular velocity
c
c SYMBOLS:
c -------
c
c a:      tube radius
c visc:   fluid viscosity
c den:    fluid density
c omega:  angular velocity of rotation
c s:      distance from the axis
c u:      velocity
c aj1:    zero of J1
c=========================================

      Implicit Double Precision (a-h,o-z)

      Dimension aj1(20)

c------------------------------------
c aj1 are zeros of Bessel function J1
c
c Source: Abramowitz & Stegun
c         p. 409
c-----------------------------------

      aj1 (1) =  3.83171
      aj1 (2) =  7.01559
      aj1 (3) = 10.17347
      aj1 (4) = 13.32369
      aj1 (5) = 16.47063
      aj1 (6) = 19.61586
      aj1 (7) = 22.76008
      aj1 (8) = 25.90367
      aj1 (9) = 29.04683
      aj1(10) = 32.18968
      aj1(11) = 35.33231 
      aj1(12) = 38.47477
      aj1(13) = 41.61709
      aj1(14) = 44.75932
      aj1(15) = 47.90146
      aj1(16) = 51.04354
      aj1(17) = 54.18555
      aj1(18) = 57.32753
      aj1(19) = 60.46946
      aj1(20) = 63.61136

c----
c Confirm the zeros
c----
 
c     Do j=1,20
c      write (6,*) bess_J1(aj1(j))
c     End Do
c     pause
 
c----

c---
c prepare
c---

      Ntrunc = 10        ! trunction level of the infinite series

      rnu = visc/den     ! kinematic viscosity
      as  = a*a
      tr  = time*rnu/as  ! reduced time
      sr  = s/a          ! reduced radial position

      u = 0.0D0

      Do j=1,Ntrunc
       tmp0 =   aj1(j)
       tmp1 =   tmp0*sr
       tmp2 = - tmp0*tmp0 * tr
       u  = u + bess_J1(tmp1)/bess_J0(tmp0)*exp(tmp2)/tmp0
      End Do

      u = a*omega*(sr+2.0*u)

c-----
c Done 
c-----

      Return
      End
