function u = tube_crc_wom ...
  ...
  (a ...
  ,visc ...
  ,rho ...
  ,pgamp ...
  ,omega ...
  ,ot ...
  ,s ...
  )

%-------------------------------------------
% Unsteady unidirectional flow in a circular tube 
% due to an oscillating pressure gradient.
%
% The pressure gradient is:
%
% dp/dx = - pgamp * sin(omega*time)
%
% SYMBOLS:
% -------
%
% a:     tube radius
% omega: angular frequency
% ot:    omega*time
% visc:  fluid viscosity
% rho:   fluid density
% s:     sigma, distance from centerline
% u:     axial velocity
%-------------------------------------------

%--------
% prepare
%--------

  nu  = visc/rho;  %   kinematic viscosity

% delta = sqrt(2.0*nu/omega);
  delta = sqrt(nu/omega);
  ar    = a/delta;
  sr    = s/delta;

  Iopt = 0;     % need neither BERp nor BEIp

%---
% denominator
%---

 [ber,bei,berp,beip] ...
  ...
  = ber_bei_0 (Iopt,ar);

  denr =  ber;
  deni = -bei;

  dens = denr*denr + deni*deni;

  aar  = denr/dens;
  aai  =-deni/dens;

%---
% numerator
%---

 [ber,bei,berp,beip] ...
  ...
  = ber_bei_0 (Iopt,sr);

 bbr =  ber;
 bbi = -bei;

%----
% real part
%---

 Ur = aar*bbr - aai*bbi - 1.0;
 Ui = aar*bbi + aai*bbr;

 cs = cos(ot);
 sn = sin(ot);

 u = pgamp*(cs*Ur+Ui*sn)/(rho*omega);

%-----
% done
%-----

return
