      program bear_2d

c=======================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c=======================================

c=======================================
c Simulation of the normal motion of a body translating
c parallel to a plane surface at constant velocity
c under the premises of lubrication flow
c=======================================

      Implicit Double Precision (a-h,o-z)

      Dimension x(200),h(200),dhdx(200)
      Dimension trd1(200),trd2(200)
      Dimension press(200),pi1(200),pi2(200)

      Dimension time(5000),gap(5000),friction(5000)

      Parameter (Iprof=32)     ! points on a profile

c---
c constants
c---

      pi  = 3.14159 255358D0
      pi2 = 2.0D0*pi
      pih = 0.5D0*pi

      Null = 0
      None = 1
      Ntwo = 2
      zero = 0.0

c---
c preferences
c---

      write (6,*) 
      write (6,*) " Choose the data input mode"
      write (6,*) 
      write (6,*) " Enter:"
      write (6,*) 
      write (6,*) " 1 to read input from file: bear_2d.dat"
      write (6,*) " 2 to type in input"
      write (6,*) " 0 to quit"
      write (6,*) " ----------"
      read  (5,*) Ienrd

      if(Ienrd.eq.0) Go to 99

c------------------------
      if(Ienrd.eq.1) then
c------------------------

        open (1,file="bear_2d.dat")

         read (1,*) a
         read (1,*) b
         read (1,*) delta
         read (1,*) c
         read (1,*) 
         read (1,*) visc
         read (1,*) p1
         read (1,*) p2
         read (1,*) Wload
         read (1,*) V
         read (1,*) 
         read (1,*) Dt
         read (1,*) 
         read (1,*) N
         read (1,*) Nprint
         read (1,*) Npause

        close (1)

c----------
       else
c----------

        write (6,*) 
        write (6,*) " Geometry"
        write (6,*)
        write (6,*) " Enter the length: a"
        write (6,*) "           length: b"
        write (6,*) "           parabolicity: delta"
        write (6,*) "           initial clearance: c"
        write (6,*) " ------------------------------"
        read  (5,*) a,b,delta,c

        write (6,*)
        write (6,*) " Enter the fluid viscosity"
        write (6,*) " -------------------------"
        read  (5,*) visc

        write (6,*)
        write (6,*) " Enter the pressures at the end of the"
        write (6,*) "       lubrication zone: p1 and p2 "
        write (6,*) " -------------------------------------"
        read  (5,*) p1,p2

        write (6,*)
        write (6,*) " Enter the force by which the body"
        write (6,*) "       is pressing against the plane"
        write (6,*) " ---------------------------------"
        read  (5,*) Wload

        write (6,*)
        write (6,*) " Enter the velocity of translation"
        write (6,*) " ---------------------------------"
        read  (5,*) V

        write (6,*)
        write (6,*) " Enter the time step"
        write (6,*) " -------------------"
        read  (5,*) Dt

        write (6,*)
        write (6,*) " Enter the grid size N"
        write (6,*)
        write (6,*) " 32 recommended"
        write (6,*) " --------------"
        read  (5,*) N

        write (6,*)
        write (6,*) " After how many steps a printout ?"
        write (6,*) " ---------------------------------"
        read  (5,*) Nprint

        write (6,*)
        write (6,*) " After how many steps pause ?"
        write (6,*) " ----------------------------"
        read  (5,*) Npause

c----------
      end If
c----------

c---
c prepare
c---

      Dx = (a+b)/N
      Dx2 = 2.0*Dx

      Iprof1 = Iprof+1
      Iprof4 = Iprof+4

      Dxprof = (a+b)/Iprof     ! step for the finite-difference method

c---
c open output files
c---

      open (2,file="bear_2d.prof")
      open (3,file="bear_2d.press")
      open (4,file="bear_2d.minth")

c---
c print flat wall profile
c---

      write (2,100) Ntwo,time(Kstep) 
      write (2,100) None,-b,zero
      write (2,100) Ntwo, a,zero

c----------------------------
c Define nodes 
c extending from x = -b to a
c----------------------------


      Do i=1,N+1
       x(i) = -b+(i-1.0)*Dx
       h(i) = c+delta*(x(i)/a)**2
      End Do

c---------------------------
c Compute dh/dx at nodes
c using central differences
c---------------------------

      Do i=2,N
       dhdx(i) = (h(i+1)-h(i-1))/Dx2
      End Do

c---
c prepare for time stepping
c---

      Istep = 1
      Kstep = 1

      Iprint = Nprint

      time(Kstep) = 0.0
      gap(Kstep) = c


c-.-.-.-.-.-.-.-.-.-.-
c begin the simulation
c-.-.-.-.-.-.-.-.-.-.-

 98   Continue

c-------------------------------------
c Solve 2nd-order ode for the pressure 
c between x=-b and x=a
c
c pi1 is due to longintudinal motion
c pi2 is due to normal motion
c-------------------------------------

      call bear_2d_pressure 
     +
     +   (visc
     +   ,x,h
     +   ,dhdx
     +   ,Dx
     +   ,V
     +   ,N
     +   ,p1,p2
     +   ,pi1,pi2
     +   )

c--------------------------------------------
c integrate the pressure to find the normal
c force exerted on the body
c--------------------------------------------

      F1 = 0.5*(pi1(1)+pi1(N+1))
      F2 = 0.5*(pi2(1)+pi2(N+1))

      Do i=2,N
        F1 = F1 + pi1(i)
        F2 = F2 + pi2(i)
      End Do

      F1 = F1*Dx
      F2 = F2*Dx

c---
c Compute dc/dt to balance the load
c using the equation:
c
c  F1 + F2 * dcdt = Wload
c---
 
      dcdt = (Wload-F1)/F2

c---
c recover the physical pressure
c---

      Do i=1,N+1
        press(i) = pi1(i) + dcdt*pi2(i)
      End Do

c---
c Approximate the lubrication flow in the gap as the
c superposition of a Couette flow due to translation
c and Poiseuille flow due to the pressure field.
c
c Compute the friction force due to lubrication
c---

      frick = 0.0

      Do i=1,N
        hmean  = 0.5*(h(i)+h(i+1))
        dpdx = (press(i)-press(i+1))/Dx 
        shear = visc*V/hmean-0.5*dpdx*hmean
        frick = frick + shear
      End Do

      frick = frick*Dx 

      friction(Kstep) = frick

c-------------------------------------
c Printing session
c
c Will plot: wall and body contour
c            pressure distribution 
c-------------------------------------

      if(Iprint.eq.Nprint) then

c---
c record the body profile
c---

        write (2,100) Iprof4,time(kstep) 

        Do i = 1,Iprof1
          xx = -b+(i-1.0)*Dxprof
          yy = c+delta*(xx/a)**2
          write (2,100) i,xx,yy
c         write (6,100) i,xx,yy
        End Do

        yy = c+delta*(b/a)**2
        tmp = 2.0*yy
        write (2,100) i,a,tmp

        i = i+1
        xx = -b
        write (2,100) i,xx,tmp

        i = i+1
        write (2,100) i,xx,yy

c---
c record the pressure distribution
c---

c       write (6,*)
c       write (6,*) " pressure distribution"
c       write (6,*)

        write (3,100) N+1,time(kstep)

        Do i=1,N+1
          write (3,100) i,x(i),press(i),pi1(i),pi2(i)
c         write (6,100) i,x(i),press(i),pi1(i),pi2(i)
        End Do

        Iprint = 0

      End If

c----------------------------------------------
c First step of 2nd-order Runge-Kutta completed
c Proceed to second step
c----------------------------------------------

      svc    = c            ! save
      svdcdt = dcdt         ! save

      c = c + dcdt*dt

c---
c new geometry
c---

      Do i=1,N+1
       h(i) = c+delta*(x(i)/a)**2
      End Do

c---
c new pressure
c---

      call bear_2d_pressure 
     +
     +   (visc,x,h,dhdx,Dx,V
     +   ,N
     +   ,p1,p2
     +   ,pi1,pi2
     +   )

      Do  i=2,N
        F1 = F1 + pi1(i)
        F2 = F2 + pi2(i)
      End Do

      F1 = F1*Dx
      F2 = F2*Dx

c---
c new dcdt
c---

      dcdt  = (Wload-F1)/F2

c-----
c final step in RK2
c----

      c = svc + 0.5*(dcdt+svdcdt)*dt

      Do i=1,N+1
       h(i) = c+delta*(x(i)/a)**2
      End Do

c-------------------
c End of a time step
c-------------------

c---
c reset counters and time
c---

      Kstep  = Kstep  + 1
      Istep  = Istep  + 1
      Iprint = Iprint + 1

      time(kstep) = time(kstep-1)+Dt

c---
c record and display
c---

      gap(Kstep) = c

      write (6,100) Kstep,time(kstep),c

      If(Istep.lt.Npause) Go to 98

c------------------------------------------------

      write (6,*) 
      write (6,*) " Continue the simulation?"
      write (6,*) 
      write (6,*) " Anything but 0 for yes"
      write (6,*) " ----------------------"
      read (5,*) Icon

      Istep = 0

      if(Icon.ne.0) Go to 98

c---------------------
c Simulation has ended
c---------------------

      write (2,100) Null
      write (3,100) Null

c---
c Print time, minimum film thickness, friction force
c---

      write (4,100) Kstep
      Do i=1,Kstep
        write (4,102) i,time(i),gap(i),friction(i)
      End Do
      write (4,100) Null

c---
c wrap up
c---

      close (2)
      close (3)
      close (4)

c---
c Done
c---

 99   Continue

 101  Format (10(1x,f15.10))
 100  Format (1x,I3,10(1x,f15.10))
 102  Format (1x,I3,1x,f10.5,2(1x,f12.8))

      Stop
      End
