      program films

c=========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=========================================

c----------------------------------------------------------
c  This program uses the long-wave (lubrication) approximation
c  to compute the evolution of NLR layers
c  resting on a horizontal wall or flowing down
c  an inclined plane.
c
c  The layers have periodic shapes repeated with period RL.
c  The wall is allowed to have periodic corrugations
c  with same period RL.
c
c  Bottom layer is numbered 1
c  Top    layer is numbered NLR
c
c  The evolution is computed using an explicit
c  finite-difference method
c
c
c  SYMBOLS:
c  -------
c
c  NLR:	Number of layers
c  NSG:	Number of points (segments) along each interface
c
c  x(i,j):  X-position of ith point on jth interface
c  y(i,j:   Y-position of ith point on jth interface
c
c  thick0(j):	Initial thickness of the jth layer
c    amp0(j):	Initial amplitude of the jth interface
c
c  thick(i,j):	Local and instantaneous thickness
c               at theith point of the jth layer
c
c  vs:	Viscosities
c  vsr:	Viscositt ratios
c
c  Y1 = dy/dx
c  Y2 = d^2y/dx^2
c  Y3 = d^3y/dx^3
c  Y4 = d^4y/dx^4
c----------------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x (0:200,0:05),y (0:200,0:05)
      Dimension Y1(0:200,0:05),Y2(0:200,0:05)
      Dimension Y3(0:200,0:05),Y4(0:200,0:05)

      Dimension V (0:200,1:05)

      Dimension Ysave (0:200,1:05)
      Dimension Vsave1(0:200,1:05),Vsave2(0:200,1:05)
      Dimension Vsave3(0:200,1:05)

      Dimension vs(05),vsr(05),rho(05),tension(05)

      Dimension amp0(0:05),shift0(0:05),thick0(05)

      Dimension xtmp(-10:500),ytmp(-10:500)

c---
c time history
c---

      Dimension time(0:9000)
      Dimension xmax(9000,05),ymax(9000,05)
      Dimension xmin(9000,05),ymin(9000,05)
      Dimension amp (9000,05)

      Parameter (pltfc=1.20)  ! for plotting window

c---
c common blocks
c---

      common/XXYY/X,Y,Y1,Y2,Y3,Y4

      common/Int1/NLR,NLR0,NLR1,NSG,NSGa,NSG1,NSG2
      common/Int2/ICU

      common/REAL1/RL,gx,gy
      common/REAL2/vs,vsr,rho,tension

      common/piii/pi,pi2

c----------
c constants
c----------

      pi  = 3.1415926 53589793 D0
      pi2 = 2.0D0*pi

      null = 0
      None = 1

c---
c Input data
c---

c     write (6,*) 
c     write (6,*) ' Enter 0 to quit'
c     write (6,*) '       1 to type data on the keyboard'
c     write (6,*) '       2 to read them from file films.dat'
c     write (6,*) ' ----------------------------------------'
c     read  (5,*) Ienrd

      Ienrd = 2

c------------------------
      if(Ienrd.eq.1) then    ! enter data from keyboard
c------------------------

      write (6,*)
      write (6,*) ' Please enter the number of layers'
      write (6,*) ' ---------------------------------'
      read  (5,*) NLR 

      write (6,*)
      write (6,*) ' Please enter the wave length'
      write (6,*) ' ----------------------------'
      read  (5,*) RL

      write (6,*)
      write (6,*) ' Enter the plane inclination angle'
      write (6,*) ' in multiples of pi'
      write (6,*) ' ---------------------------------'
      read  (5,*)  beta

      write (6,*)
      write (6,*) ' Enter the acceleration of gravity'
      write (6,*) ' ---------------------------------'
      read  (5,*) gac

      write (6,*)
      write (6,*) ' Enter the layer thicknesses'
      write (6,*)
      write (6,*) ' Bottom to top'
      write (6,*) ' ---------------------------'
      read  (5,*) (thick0(i),i=1,NLR)

      write (6,*)
      write (6,*) ' Enter the layer viscosities'
      write (6,*)
      write (6,*) ' Bottom to top'
      write (6,*) ' ---------------------------'
      read  (5,*) (vs(i),i=1,NLR)

      write (6,*)
      write (6,*) ' Enter the layer densities'
      write (6,*)
      write (6,*) ' Bottom to top'
      write (6,*) ' ---------------------------'
      read  (5,*) (rho(i),i=1,NLR)

      write (6,*)
      write (6,*) ' Enter the interfacial tensions'
      write (6,*)
      write (6,*) ' Bottom to top'
      write (6,*) ' ------------------------------'
      read  (5,*) (tension(i),i=1,NLR)

      write (6,*)
      write (6,*) ' The inclined wall may by wavy'
      write (6,*)
      write (6,*) ' Enter the amplitude of the corrugation'
      write (6,*) ' -------------------------------------'
      read  (5,*) amp0(0)

      write (6,*)
      write (6,*)  ' Enter 0 to generate interface profiles'
      write (6,*) '        1 to read them from file: films.inp'
      write (6,*) ' -----------------------------------------'
      read  (5,*) Iread

      write (6,*)
      write (6,*) ' Enter the initial interface amplitudes'
      write (6,*)
      write (6,*) ' Bottom to top'
      write (6,*) ' -------------------------------------'
      read  (5,*) (amp0(i),i=1,NLR)

      write (6,*)
      write (6,*) ' Enter the initial interface phases'
      write (6,*) ' with respect to the designated origin'
      write (6,*) ' in multiples of pi'
      write (6,*)
      write (6,*) ' Bottom to top'
      write (6,*) ' -------------------------------------'
      read  (5,*) (shift0(i),i=1,NLR)

      If(Iread.eq.0) then

        write (6,*)
        write (6,*) ' Enter the number of finite-difference'
        write (6,*) ' divisions over a period'
        write (6,*) ' -------------------------------------'
        read  (5,*) NSG

      End If

      write (6,*)
      write (6,*) ' How many steps to get a printout ?'
      write (6,*) ' ----------------------------------'
      read  (5,*) Nprint

      write (6,*)
      write (6,*) ' Choose the order of the Runge-Kutta method'
      write (6,*)
      write (6,*) ' Enter 1, 2, or 4; default is 4'
      write (6,*) ' ------------------------------------------'
      read  (5,*) IRK

      write (6,*)
      write (6,*) ' Enter 1 for central differencing'
      write (6,*) '       2 for upwind  differencing'
      write (6,*) ' --------------------------------'
      read  (5,*) ICU

      write (6,*)
      write (6,*) ' Enter the time step'
      write (6,*) ' -------------------'
      read  (5,*) Dt

      write (6,*)
      write (6,*) ' How many steps to take ?'
      write (6,*) ' ------------------------'
      read  (5,*) Nstep

c---------
      elseif(Ienrd.eq.2) then      ! read data
c---------

      open (3,file='films.dat')

      read (3,*) NLR 
      read (3,*) RL
      read (3,*) beta
      read (3,*) gac
      read (3,*) (thick0(i),i=1,NLR)
      read (3,*) (vs    (i),i=1,NLR)
      read (3,*) (rho   (i),i=1,NLR)
      read (3,*) (tension(i),i=1,NLR)
      read (3,*) amp0(0)
      read (3,*) (amp0(i),i=1,NLR)
      read (3,*) (shift0(i),i=1,NLR)
      read (3,*) NSG
      read (3,*) Iread
      read (3,*) Nprint
      read (3,*) IRK
      read (3,*) ICU
      read (3,*) Dt
      read (3,*) Nstep

c     write (6,*) NLR 
c     write (6,*) RL
c     write (6,*) beta
c     write (6,*) gac
c     write (6,*) (thick0(i),i=1,NLR)
c     write (6,*) (vs (i),i=1,NLR)
c     write (6,*) (rho(i),i=1,NLR)
c     write (6,*) (tension(i),i=1,NLR)
c     write (6,*) amp0(0)
c     write (6,*) (amp0(i),i=1,NLR)
c     write (6,*) (shift0(I),i=1,NLR)
c     write (6,*) NSG
c     write (6,*) Iread
c     write (6,*) Nprint
c     write (6,*) IRK
c     write (6,*) ICU
c     write (6,*) Dt
c     write (6,*) Nstep

      close (3) 

c-----------
      End If         ! finished reading input data
c-----------

c---
c prepare
c---

      If(IRK.ne.1.and.IRK.ne.2
     +           .and.IRK.ne.4) then
       IRK=4
       write (6,*) 'films: Defaulted to RK4'
      End If

      NLRa = NLR-1        ! layers
      NLR1 = NLR+1

      NSG1 = NSG+1        ! points along the interfaces
      NSG2 = NSG+2

      beta = beta*pi
      cth0 = Dcos(beta)
      sth0 = Dsin(beta)

      gx =  gac*sth0
      gy = -gac*cth0

      htot = thick0(1)      ! total layer thickness

      Do i=2,NLR
        htot = htot+thick0(i)
      End Do

c---
c various definitions
c---

      RLM  = - RL

      RLO  =  pltfc*RL       ! for plotting
      RLMO =  pltfc*RLM

      shiftx = - RL*cth0     ! for plotting
      shifty =   RL*sth0

      rho(NLR1) = 0.0          ! density of air
      vs (NLR1) = 0.0          ! viscosity of air

      Do i=1,NLR               
       vsr(i) = vs(i+1)/vs(i)  ! viscosity ratios
      End Do

      Do i=1,NLR
      End Do

      open (1,file='films.xy')
      open (4,file='films.diag')

c------------------------
      If(Iread.eq.0) then
c------------------------

c---------------------------
c Generate interfacial shapes 
c
c sinusoidal waves with amplitude: amp0
c and phase-lag with respect to the 
c x origin: shift0
c---------------------------

      time(1) = 0.0D0

      wn = pi2/RL      ! wave number
      Dx = RL/(NSG1-1.0D0)

      h = 0.0         ! unperturbed interface height

      Do j=1,NLR     ! loop over interfaces

      shift0(j) = shift0(j)*pi

      h = h + thick0(j)

c      write (6,101) NSG,j 

       Do i=0,NSG1     ! loop over points
        x(i,j) = (i-1.0D0)*Dx
        arg    = wn*x(i,j)-shift0(j)
        y(i,j) = h + amp0(j)*Dcos(arg)
c       write (6,104) i,x(i,j),y(i,j)
       End Do

      shift0(j) = shift0(j)/pi          ! unreduce to record

      End Do

c----------------------------------------
c read initial shape from file: films.inp
c----------------------------------------

      Else If(Iread.eq.1) then

      open (2,file='films.inp')

      Do j=1,NLR                  ! over interfaces

       read (2,*) NSG1,idle,time(1)

       write (6,*) "Reading Interface ",j

       Do i=1,NSG1
         read (2,*)   k,X(i,j),Y(i,j)
         X(i,j) = X(i,j)-shiftx            ! shift
         Y(i,j) = Y(i,j)-shifty
         tmx    = X(i,j)*cth0-Y(i,j)*sth0  ! rotate
         tmy    = X(i,j)*sth0+Y(i,j)*cth0
         X(i,j) = tmx
         Y(i,j) = tmy
         write (6,104) i,x(i,j),y(i,j)   ! display
       End Do

      End Do

      NSG  = NSG1-1 
      NSG2 = NSG1+1

      close (2)

      End If

c------------------------
c compute wall points and
c derivatives
c--------------------------

      Do i=0,NSG1
       x(i,0) = (i-1.0D0)*Dx
       arg    = wn*x(i,0)
       cs = cos(arg)
       sn = sin(arg)
        Y(i,0) =  amp0(0)*cs
       Y1(i,0) = -amp0(0)*sn*wn
       Y2(i,0) = -amp0(0)*cs*wn**2
       Y3(i,0) =  amp0(0)*sn*wn**3
       Y4(i,0) =  amp0(0)*cs*wn**4
      End Do

c----------------------------
c print wall profile
c so that it appears inclined
c----------------------------

      write (1,101) NSG1,null,time(Kstep)

      Do i=1,NSG1
        tmx =  x(i,0)*cth0+y(i,0)*sth0
        tmy = -x(i,0)*sth0+y(i,0)*cth0
        tmx = tmx + shiftx
        tmy = tmy + shifty
        write (1,105) i,tmx,tmy
      End Do

c---
c prepare
c---

      Dth = 0.5*Dt

      Kstep  = 1
      Iprint = Nprint

      Itry = 1          ! graphics

c-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
c  Time integration begins here
c-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-

  97  Continue

c---
c  Compute maximum and minimum interface elevations 
c  and interface amplitude
c---

      Do j=1,NLR

       Do i=0,NSG1
        xtmp(i) = x(i,j)
        ytmp(i) = y(i,j)
       End Do

       call elv_max_min
     +
     +  (NSG
     +  ,xtmp,ytmp
     +  ,xmax(Kstep,j)
     +  ,ymax(Kstep,j)
     +  ,xmin(Kstep,j)
     +  ,ymin(Kstep,j)
     +  )

       amp(Kstep,j) = 0.5*(ymax(Kstep,j)-ymin(Kstep,j))

      End Do

c---
c if amplitudes are too large, stop
c---

      Do i=1,NLR 
       h = 0.0D0              ! compute height of ith interface
        Do k=1,i
         h = h+thick0(k)
        End Do
        If(amp(Kstep,i).gt.h) then
          write (6,*) ' Something went wrong'  
          Go to 99
        End If
      End Do

c---
c Reduce local interface amplitude
c by initial amplitude 
c---

      Do j=1,NLR 
        If(amp0(j).ne.0) 
     +  amp(Kstep,j) = amp(Kstep,j)/amp0(j)
      End Do

c-----------------
c Printing session
c-----------------

      write (6,110) Kstep,time(Kstep),(amp(Kstep,i),i=1,NLR)
      write (4,110) Kstep,time(Kstep),(amp(Kstep,i),i=1,NLR)

c---
c print rotated and shifted interfaces
c---

      If(Iprint.eq.Nprint) then

        Do j=1,NLR

          write (1,101) NSG1,j,time(Kstep)

          Do i=1,NSG1
           tmx =  X(i,j)*cth0+Y(i,j)*sth0
           tmy = -X(i,j)*sth0+Y(i,j)*cth0
           tmx = tmx + shiftx
           tmy = tmy + shifty
           write (1,105) i,tmx,tmy
          End Do

        End Do

        Iprint = 0

      End If

c---
c compute phase-space velocity: v=dh/dt
c---

       call films_pde(v)

c---------------
c first order RK
c---------------

      If(IRK.eq.1) then

       Do j=1,NLR
         Do i=0,NSG1
           Y(i,j) = Y(i,j)+v(i,j)*Dt
         End Do
       End Do

c----------------
c Second order RK
c----------------

      Else If(IRK.eq.2) then

       Do j=1,NLR 
         Do i=0,NSG1
           Ysave (i,j) = Y(i,j)
           Vsave1(i,j) = V(i,j)
                Y(i,j) = Y(i,j)+V(i,j)*Dt
         End Do
       End Do

       call films_pde(v)

       Do j=1,NLR 
         Do i=0,NSG1
           Y(i,j) = Ysave(i,j)
     +            +(Vsave1(i,j)+V(i,j))*Dth
         End Do
       End Do

c----------------
c Fourth order RK
c----------------

      Else If(IRK.eq.4) then

       Do j=1,NLR 
         Do i=0,NSG1
           Ysave (i,j) = Y(i,j)
           Vsave1(i,j) = V(i,j)
                Y(i,j) = Y(i,j)+V(i,j)*Dth
         End Do
       End Do

       call films_pde(v)

       Do j=1,NLR 
         Do i=0,NSG1
           Vsave2(i,j) = V(i,j)
                Y(i,j) = Ysave(i,j)+V(i,j)*Dth
         End Do
       End Do

       call films_pde(v)

       Do j=1,NLR 
         Do i=0,NSG1
           Vsave3(i,j) = V(i,j)
                Y(i,j) = Ysave(i,j)+V(i,j)*Dt
         End Do
       End Do

       call films_pde(v)

       Do j=1,NLR
         Do i=0,NSG1
           Y(I,J)=Ysave(I,J)
     +          +Dt*(Vsave1(I,J)+2.0D0*Vsave2(I,J)+
     +           2.0D0*Vsave3(I,J)+V(I,J))/6.0D0
         End Do
       End Do

      End If

c----------------------
c End of RK Integration
c----------------------

  91  Continue

c     Do i=1,NSG
c       write (6,102) i,(y(i,j),j=1,NLR)
c     End Do

c---
c reset counters and time
c---

      Kstep  = Kstep +1
      Iprint = Iprint+1

      time(Kstep) = time(Kstep-1)+DT

      If(Kstep.le.Nstep) Go to 97

c-.-.-.-.-.-.-.-.-.-.-.-
c SIMULATION HAS ENDED
c-.-.-.-.-.-.-.-.-.-.-.-

  99  Continue

c---
c printing session
c---

      Do j=1,NLR
       write (1,101) NSG1,j,time(Kstep)
       Do i=1,NSG1
         tmx =  X(i,j)*cth0+Y(i,j)*sth0
         tmy = -X(i,j)*sth0+Y(i,j)*cth0
         tmx = tmx + shiftx
         tmy = tmy + shifty
         write (1,105) i,tmx,tmy
       End Do
      End Do

      write (1,105) null,null,null

c---
c  Compute maximum and minimum interface elevations 
c  and reduce
c---

      Do j=1,NLR

       Do i=0,NSG1
        xtmp(i) = x(i,j)
        ytmp(i) = y(i,j)
       End Do

       call elv_max_min
     +
     + (NSG
     + ,xtmp,ytmp
     + ,xmax(Kstep,j)
     + ,ymax(Kstep,j)
     + ,xmin(Kstep,j)
     + ,ymin(Kstep,j)
     + )
       amp(Kstep,j) = 0.5*(ymax(Kstep,j)-ymin(Kstep,j))
      End Do

      Do j=1,NLR 
        If(amp0(j).ne.0) 
     +  amp(Kstep,j) = amp(Kstep,j)/amp0(j)
      End Do

      write (6,110) Kstep,time(Kstep),(amp(Kstep,i),i=1,NLR)
      write (4,110) Kstep,time(Kstep),(amp(Kstep,i),i=1,NLR)
      write (4,*) null,null,null

c---
c Record and stop
c---

      write (1,*)
      write (1,*) " PROGRAM: films"
      write (1,*)
      write (1,600) NLR 
      write (1,601) RL
      write (1,602) beta
      write (1,603) gac
      write (1,604) (thick0(i),i=1,NLR)
      write (1,605) (vs   (i),i=1,NLR)
      write (1,606) (rho   (i),i=1,NLR)
      write (1,607) (tension(i),i=1,NLR)
      write (1,608) amp0(0)
      write (1,609) (amp0(i),i=1,NLR)
      write (1,610) (shift0(i),i=1,NLR)
      write (1,611) NSG
      write (1,612) Iread
      write (1,613) Nprint
      write (1,614) IRK
      write (1,615) ICU
      write (1,616) Dt
      write (1,617) Nstep

      close (4) 

c-----
c Done
c-----

  100 Format (2X,F12.5,2X,F12.5,2X,F12.5,2X,F12.5,2X,F12.5)

  101 Format (2X,I3,2X,I3,2x,F10.5)                             
  102 Format (1X,I2,1X,F10.5,1X,F10.5,1X,F10.5,1X,F10.5,1X,F10.5)
  104 Format (2X,I3,2X,F10.5,2X,F10.5)
  105 Format (1X,I3,10(2X,F12.5))
  109 Format (2X,'AMPLITUDE OF INTERFACE:',I1,' =',F10.5)

  110 Format (1X,I5,10(1x,f8.5))

  501 Format (4(1x,f10.5))
  502 Format (3(1x,f10.5),1x,I1)

 600  Format (' NLR    = ',i3)
 601  Format (' L      = ',f10.5)
 602  Format (' beta   = ',f10.5)
 603  Format (' gac    = ',f10.5)
 604  Format (' thick0 = ',20(1x,f10.5))
 605  Format (' vs     = ',20(1x,f10.5))
 606  Format (' rho    = ',20(1x,f10.5))
 607  Format (' tension= ',20(1x,f10.5))
 608  Format (' a_wall = ',f10.5)
 609  Format (' amp    = ',20(1x,f10.5))
 610  Format (' shift  = ',20(1x,f10.5))
 611  Format (' NSG    = ',I3)
 612  Format (' Iread  = ',I3)
 613  Format (' Nprint = ',I3)
 614  Format (' Irk    = ',I3)
 615  Format (' Icu    = ',I3)
 616  Format (' Dt     = ',f10.5)
 617  Format (' Nstep  = ',I5)

      Stop
      End
