%---
close all
clear all
%---

%-----------------------------
% Generate a Joukowski airfoil
%-----------------------------

xc = 0.1;
yc = 0.2;
n = 32;
sigma = 1.0;

%---
% prepare
%---

rad = sqrt((xc+sigma)^2+yc*yc);
phi = acos((xc+sigma)/rad);
thstart = -pi+phi;
dth = 2*pi/n;

%---
% run around
%---

for i=1:n+1
  th = thstart+(i-1)*dth;
  xi = xc+rad*cos(th);
  et = yc+rad*sin(th);
  r2 = xi*xi+et*et;
  x(i) = xi*(1.0+sigma^2/r2);
  y(i) = et*(1.0-sigma^2/r2);
end

%---
% plotting
%---

figure(1)
hold on
axis equal
axis(sigma*[-2.2 2.2 -0.5 1])
xlabel('x','fontsize',13)
ylabel('y','fontsize',13)
set(gca,'fontsize',13)
box on

plot(-x,y,'ko-')
%plot(-x(1),y(1),'ro')
%plot(-x(10),y(10),'co')

