clear all
close all
hold on

%---
% parameters
%---

ax=0.0;
bx=1.0;
ay=0.0;
by=0.4;

veltop=1.0;

Nx=32;
Ny=16;

%--------
% prepare
%--------

Dx = (bx-ax)/Nx;  % grid spacing
Dy = (by-ay)/Ny;  % grid spacing
beta =(Dx/Dy)^2;

N = Ny*(Nx+1);    % size of the matrix

%------------------
% generate the grid
%------------------

[glx,gly,gx,gy] = grid_2d (ax,bx,ay,by,Nx,Ny);

%-------------------------------------
% specify the potential at the top row
%-------------------------------------

 for i=1:Nx+1
   phitop(i) = veltop*glx(i);
 end

%-------------------
% coefficient matrix
%-------------------

 A = cvt_2d_fdm (Nx,Ny,beta);

%----------------
% right-hand side
%----------------

 for i=1:N-Nx-1
   rhs(i) = 0.0;
 end
 for i=1:Nx+1
   rhs(N-Nx-1+i) = -beta*phitop(i);
 end

%------------------------
% solve the linear system
%------------------------

 sln=rhs/A';

%-------------------------
% assign solution to nodes
%-------------------------

 Ic = 1;  % counter

 for j=1:Ny
   for i=1:Nx+1
     ptl(i,j) = sln(Ic);
     Ic = Ic+1;
  end
 end

 for i=1:Nx+1
    ptl(i,Ny+1) = phitop(i);
 end

%-------------
% surface plot
%-------------

figure(1)
surf(glx,gly,ptl')
xlabel('x','fontsize',15)
ylabel('y','fontsize',15)
zlabel('\phi','fontsize',15)
set(gca,'fontsize',15)
view(-33,42)
box on

%-------------------------------------------
% compute the velocity at the internal nodes
% by numerical differentiation
% using central differences
%-------------------------------------------

 for i=2:Nx
  for j=2:Ny
    gux(i,j) =  (ptl(i+1,j) -ptl(i-1,j)) ...
              /(gx(i+1,j)-gx(i-1,j));
    guy(i,j) =  (ptl(i,j+1) -ptl(i,j-1)) ...
              /(gy(i,j+1)-gy(i,j-1));
  end
 end

%----------------------------------
% Compute the velocity on the walls
% by numerical differentiation
%----------------------------------

%---
% left wall: i=1
%---

 for j=2:Ny
  gux(1,j) = 0.0;
  guy(1,j) = ( ptl(1,j+1)- ptl(1,j-1)) ...
            /(gy(1,j+1)-gy(1,j-1));
 end

%---
% bottom wall: j=1
%---

  for i=2:Nx
   gux(i,1) = ( ptl(i+1,1)- ptl(i-1,1)) ...
               /(gx(i+1,1)-gx(i-1,1));
   guy(i,1) = 0.0;
  end

%---
% right wall: i=Nx+1
%---

 for j=2:Ny
  gux(Nx+1,j) = 0.0;
  guy(Nx+1,j) = ( ptl(Nx+1,j+1) -ptl(Nx+1,j-1)) ...
               /(gy(Nx+1,j+1)-gy(Nx+1,j-1));
  end

%---
% top wall: j=Ny1
%---

  for i=2:Nx
     gux(i,Ny+1) = veltop;
     guy(i,Ny+1) = ( ptl(i,Ny+1)- ptl(i,Ny)) ...
                  /(gy(i,Ny+1)-gy(i,Ny));
   end

%---
% four corners
% velocity is zero or singular;
% set it to zero
%---

 gux(1  ,1)   = 0.0;
 gux(Nx+1,1)   = 0.0;
 gux(1  ,Ny+1) = 0.0;
 gux(Nx+1,Ny+1) = 0.0;

 guy(1  ,1)   = 0.0;
 guy(Nx+1,1)   = 0.0;
 guy(1  ,Ny+1) = 0.0;
 guy(Nx+1,Ny+1) = 0.0;

figure(2)
hold on
[glx,gly,gx,gy] = grid_2d (ax,bx,ay,by,Nx,Ny);
quiver(gx,gy,gux,guy,'k')
axis equal
axis([ax,bx,ay,by])
xlabel('x','fontsize',15)
ylabel('y','fontsize',15)
set(gca,'fontsize',15)
box on
