function A = cvt_2d_fdm (Nx,Ny,beta)

%=================================
% Generates the coefficient matrix
% of a linear system for the potential
%=================================

%--------
% prepare
%--------

N = Ny*(Nx+1);  % matrix size
A = zeros(N,N);
cf = -2.0*(1.0+beta);

%-----------------------
% set the five diagonals
%-----------------------

A(1,1)   = cf;   % first row
A(1,2)   = 2.0;  
A(1,Nx+2) = 2.0*beta;
for i=2:Nx+1
 A(i,i)     = cf;  % first block
 A(i,i+1)   = 1.0;
 A(i,i-1)   = 1.0;
 A(i,i+Nx+1) = 2.0*beta;
end

for i=Nx+2:N-Nx-1     % intermediate blocks
  A(i,i)     = cf;
  A(i,i+1)   = 1.0;
  A(i,i-1)   = 1.0;
  A(i,i+Nx+1) = beta;
  A(i,i-Nx-1) = beta;
end
for i=N-Nx:N-1
  A(i,i)      = cf;  % last block
  A(i,i+1)    = 1.0;
  A(i,i-1)    = 1.0;
  A(i,i-Nx-1)  = beta;
end

A(N,N)     = cf;   % last row
A(N,N-1)   = 2.0; 
A(N,N-Nx-1) = beta; 

%------------------------------
% reset the ones to twos
% and the faulty betas to zeros
%------------------------------

for i=2:Ny     % run over horizontal partitions
  loc = (i-1)*(Nx+1)+1;
  A(loc,loc-1) = 0.0;
  A(loc,loc+1) = 2.0;
end

for i=1:Ny-1  % run over horizontal partitions
  loc = i*(Nx+1);
  A(loc,loc-1) = 2.0;
  A(loc,loc+1) = 0.0;
end

%-----
% done
%-----

return
