      program chan2l0_dr

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c----------------------------------------
c  Complex phase velocity 
c  for two-layer Stokes flow in a channel
c  confined between two parallel walls
c
c  SYMBOLS:
c  -------
c 
c  h:   walls are located at y = +- h
c  yI:  position of the interface
c-----------------------------------------

      Implicit Double Precision (a-h,o-z)

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

      Null = 0

c----------------
c read parameters
c----------------

      open (1,file="chan2l0.dat")

        read (1,*) h
        read (1,*) yI
        read (1,*) th0
        read (1,*)
        read (1,*) RL
        read (1,*)
        read (1,*) vs1 
        read (1,*) vs2
        read (1,*)
        read (1,*) rho1 
        read (1,*) rho2
        read (1,*)
        read (1,*) gamma
        read (1,*)
        read (1,*) U1
        read (1,*) U2
        read (1,*)
        read (1,*) pg
        read (1,*)
        read (1,*) gac
        read (1,*)
        read (1,*) Iframe
        read (1,*)
        read (1,*) Ishow
        read (1,*)
        read (1,*) Nstp
        read (1,*) wnmax

      close (1)

c------------
c preferences
c------------

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 0 to quit"
      write (6,*) " 1 for one evaluation "
      write (6,*) " 2 for a graph with respect to the"
      write (6,*) "   wave number"
      write (6,*) " -------------"
      read  (5,*) menu

      If(menu.eq.0) Go to 99

c--------
c prepare
c--------

      th0 = th0*pi

c----------------------
      If(menu.eq.1) then
c----------------------

 97   Continue

      call chan2l0
     +
     +  (RL,h,yI,th0
     +  ,vs1,vs2
     +  ,rho1,rho2
     +  ,gamma
     +  ,U1,U2
     +  ,pg
     +  ,gac
     +  ,Iframe
     +  ,Ishow
     +  ,xi1,xi2
     +  ,uI
     +  ,cr,ci
     +  ,sr,si
     +  )

c     write (6,*) 
c     write (6,105) uI,xi1,xi2
c     write (6,*) 

      write (6,101) cr,sr
      write (6,102) ci,si

      write (6,*) 
      write (6,*) ' Current value of L is: ',RL
      write (6,*) ' Please enter a new value'
      write (6,*) ' 0 to quit'
      write (6,*) ' ----------'
      read  (5,*) RL

      If(RL.eq.0) Go to 99

      Go to 97

c----------------------
      Else If(menu.eq.2) then
c----------------------

      open (9,file="chan2l0.out")

 98   Continue

      Nstp1 = Nstp+1
      wnmin = 0.0001D0
      step = (wnmax-wnmin)/Nstp

      wn = wnmin

      write (9,100) Nstp1,yI

      Do i=1,Nstp1

       RL = pi2/wn

       call chan2l0
     +
     +  (RL,h,yI,th0
     +  ,vs1,vs2
     +  ,rho1,rho2
     +  ,gamma
     +  ,U1,U2
     +  ,pg
     +  ,gac
     +  ,Iframe
     +  ,Ishow
     +  ,xi1,xi2
     +  ,uI
     +  ,cr,ci
     +  ,sr,si
     +  )

      siing = -0.5D0/(vs1+vs2) * ( (rho1-rho2)*gac/wn + wn*gamma)

      write (6,100) i,wn,cr,si,siing
      write (9,100) i,wn,cr,si,siing

      wn = wn+step

      End Do

      write (6,*) 
      write (6,*) ' Please enter new value of yI'
      write (6,*) ' 99 to quit '
      write (6,*) '------------'
      read  (5,*) yI

      If(yI.ne.99) Go to 98

      write (9,100) Null

      close (9)

c-----------
      End If
c-----------

c-----
c Done
c-----

  99  Continue

 100  Format (1x,I3,10(1x,f12.8))
 101  Format ("cr = ",f16.10," sr = ",f16.10)
 102  Format ("ci = ",f16.10," si = ",f16.10)
 105  Format ("uI = ",f12.8,", xi1= ",f12.8,", xi2= ",f12.8)

      Stop
      End
