      subroutine cramer_33c
     +
     +   (AR,AI
     +   ,BR,BI
     +   ,XR,XI
     +   )

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c------------------------------------------------
c This program accompanies the book:
c C. Pozrikidis
c ``Numerical Computation in Science and Engineering''
c Oxford University Press, 1998
c------------------------------------------------

c---------------------------------------
c  SOLUTION OF A 3x3 COMPLEX SYSTEM
c  BY CRAMER'S RULE
c
c  LEGEND:
c  ------
c
c  AR:  real part of the matrix
c  AI:  imaginary part of the matrix
c
c  BR:  real part of the rhs
c  BI:  imaginary part of the rhs
c
c  XR:  real part of the solution
c  XI:  imaginary part of the solution
c
c  DetR:  real part of the determinant
c  DetI:  imaginary part of the determinant
c
c------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension AR(3,3),AI(3,3)

      Dimension BR(3),BI(3)
      Dimension XR(3),XI(3)

      Dimension DETR(3),DETI(3)
      Dimension savR(3),savI(3)

      Parameter (tol=0.00000001D0)

c---
c display
c---

      Go to 333

      write (6,*)
      write (6,101) AR(1,1),AR(1,2),AR(1,3),BR(1)
      write (6,101) AR(2,1),AR(2,2),AR(2,3),BR(2)
      write (6,101) AR(3,1),AR(3,2),AR(3,3),BR(3)
      write (6,*)
      write (6,101) AI(1,1),AI(1,2),AI(1,3),BI(1)
      write (6,101) AI(2,1),AI(2,2),AI(2,3),BI(2)
      write (6,101) AI(3,1),AI(3,2),AI(3,3),BI(3)

 333  Continue

c-----------
c Compute the determinant
c-----------

      call Det_33c
     +
     +   (AR,AI
     +   ,DetR0,DetI0
     +   )

c-----------------
c loop over minors
c-----------------

      Do K=1,3

        Do j=1,3
         savR(j) = AR(j,K)
         savI(j) = AI(j,K)
         AR(j,K) = BR(j)
         AI(j,K) = BI(j)
        End Do

        call Det_33c
     +
     +   (AR,AI
     +   ,DetR(K),DetI(K)
     +   )

c--------
c restore
c--------

        Do j=1,3
         AR(j,K) = savR(j)
         AI(j,K) = savI(j)
        End Do

      End Do

c--------------
c Cramer's rule
c--------------

      Den =  DetR0**2+DetI0**2

      DtR =  DetR0/Den 
      DtI = -DetI0/Den 

      Do K=1,3
       XR(K) = DetR(K)*DtR - DetI(K)*DtI
       XI(K) = DetR(K)*DtI + DetI(K)*DtR
      End Do

c-----
c verify
c-----

      Do K=1,3
       VR = 0.0D0
       VI = 0.0D0
       Do J=1,3
        VR = VR + AR(K,J)*XR(J)-AI(K,J)*XI(J)
        VI = VI + AR(K,J)*XI(J)+AI(K,J)*XR(J)
       End Do
       VR = VR - BR(K)
       VI = VI - BI(K)

       If(Dabs(VR).gt.tol.or.Dabs(VI).gt.tol) then
        write (6,*) " cramer_33c: failed",Dabs(VR),Dabs(VI)
        stop
       End If

c     write (6,*) " cramer_33c: OK"

      End Do
      
c-----
c Done
c-----

  100 Format (1x,I3,10(1x,f10.5))
  101 Format (10(1x,f10.5))

      Return
      End
