      program ifsf0_s_dr

c=========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=========================================

c----------------------------------------
c Growth rate of the istability at the interface
c between two superposed fluids in shear flow
c in Stokes flow
c in the presence of surfactants
c
c Fluid 1 is at the bottom
c Fluid 2 is on top
c-----------------------------------------

      Implicit Double Precision (a-h,o-z)
      Double Precision Ma

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

      Null = 0

c----------------
c read parameters
c----------------

      open (1,file="ifsf0_s.dat")

       read (1,*) vs1    ! viscosity of lower fluid
       read (1,*) vs2    ! viscosity of upper fluid
       read (1,*) rho1   ! density of lower fluid
       read (1,*) rho2   ! density of upper fluid
       read (1,*) gamma0 ! surface tension
       read (1,*) Ma     ! Marangoni number
       read (1,*) Ds     ! surfactant diffusivity
       read (1,*)
       read (1,*) xi     ! shear rate
       read (1,*)
       read (1,*) RL     ! wave length
       read (1,*)
       read (1,*) gac    ! accelation of gravity
       read (1,*) Nstp
       read (1,*) wnmax

      close (1)

c------------
c preferences
c------------

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 for one evaluation "
      write (6,*) " 2 for a graph with respect to the wave number"
      write (6,*) " 0 to quit"
      write (6,*) "----------"
      read  (5,*) menu

      if(menu.eq.0) Go to 99

c----------------------
      if(menu.eq.1) then
c----------------------

 97   Continue

      call ifsf0_s
     +
     +  (RL
     +  ,vs1,vs2
     +  ,rho1,rho2
     +  ,gamma0
     +  ,Ma
     +  ,Ds
     +  ,xi
     +  ,gac
     +  ,cr1,si1
     +  ,cr2,si2
     +  )

      write (6,*) 
      write (6,101) cr1,si1
      write (6,101) cr2,si2

      write (6,*) 
      write (6,*) ' Please enter a new value for L'
      write (6,*) ' 0 to quit'
      write (6,*) '----------'
      read  (5,*) RL

      If(RL.eq.0) Go to 99
      Go to 97

c----------------------
      else if(menu.eq.2) then
c----------------------

      open (9,file="ifsf0_s.out")

      Nstp1 = Nstp+1
      wnmin = 0.0001
      step = (wnmax-wnmin)/Nstp

      wn = wnmin

      write (9,100) Nstp1

      Do i=1,Nstp1

       RL = pi2/wn

      call ifsf0_s
     +
     +  (RL
     +  ,vs1,vs2
     +  ,rho1,rho2
     +  ,gamma0
     +  ,Ma
     +  ,Ds
     +  ,xi
     +  ,gac
     +  ,cr1,si1
     +  ,cr2,si2
     +  )

      write (6,100) i,wn,cr1,si1,cr2,si2
      write (9,100) i,wn,cr1,si1,cr2,si2

      wn = wn+step

      End Do

      write (9,100) Null

      close (9)

c-----------
      End If
c-----------

c-----
c Done
c-----

  99  Continue

 100  Format (1x,I3,10(1x,f12.8))
 101  Format ("cr = ",f12.8,3x,"si = ",f12.8)

      Stop
      End
