      program layer0_dr

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c----------------------------------------
c Instability of a liquid layer (1)
c resting on a plane wall
c underneath a semi-infinite fluid (2)
c for Stokes flow.
c
c Fluid 1 is at the bottom
c Fluid 2 is at the top
c
c Method = 1 for analytical solution of Newhouse and Pozrikidis
c          2 for the matrix-impulse method
c
c LEGEND:
c ------
c
c b: layer thickness
c
c-----------------------------------------

      Implicit Double Precision (a-h,o-z)

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

      Null = 0

c----------------
c read parameters
c----------------

      open (1,file="layer0.dat")

        read (1,*) vs1    ! viscosity of lower fluid
        read (1,*) vs2    ! viscosity of upper fluid
        read (1,*) rho1   ! density of lower fluid
        read (1,*) rho2   ! density of upper fluid
        read (1,*) gamma  ! surface tension
        read (1,*)
        read (1,*) b      ! thickness of layer next to wall
        read (1,*)
        read (1,*) RL     ! wave length
        read (1,*)
        read (1,*) gac    ! accelation of gravity
        read (1,*)
        read (1,*) method
        read (1,*)
        read (1,*) Nstp
        read (1,*) wnmax

      close (1)

c------------
c preferences
c------------

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 0 to quit"
      write (6,*) " 1 for one evaluation "
      write (6,*) " 2 for a graph with respect to kh"
      write (6,*) " --------------------------------"
      read  (5,*) menu

      If(menu.eq.0) Go to 99

c--------
c prepare
c--------

      vsr = vs2/vs1  ! viscosity ratio

c----------------------
      If(menu.eq.1) then
c----------------------

 97   Continue

      call layer0
     +
     +   (b
     +   ,RL
     +   ,vs1,vs2
     +   ,rho1,rho2
     +   ,gamma
     +   ,gac
     +   ,method
     +   ,Phi
     +   ,si
     +   )

      write (6,*) 
      write (6,101) si

      write (6,*) 
      write (6,*) ' Currently L = ',RL
      write (6,*) ' Please enter new value of L'
      write (6,*) ' 0 to quit '
      write (6,*) '-----------'
      read  (5,*) RL

      If(RL.eq.0) Go to 99
      Go to 97

c----------------------------
      Else If(menu.eq.2) then      ! a graph
c----------------------------

      open (9,file="layer0.out")

      Nstp1 = Nstp+1
      wnmin = 0.0001
      step  = (wnmax-wnmin)/Nstp
      wn    = wnmin

c     write (9,100) Nstp1,vsr
      write (9,100) Nstp1,vsr

      Do i=1,Nstp1

       RL = pi2/wn

       call layer0
     +
     +   (b
     +   ,RL
     +   ,vs1,vs2
     +   ,rho1,rho2
     +   ,gamma
     +   ,gac
     +   ,method
     +   ,Phi
     +   ,si
     +   )

      write (6,100) i,wn,si
      write (9,100) i,wn,si
c     write (6,100) i,wn,Phi
c     write (9,100) i,wn,Phi

      wn = wn+step

      End Do

      write (9,100) Null

      close (9)

c-----------
      End If
c-----------

  99  Continue

c-----
c Done
c-----

 100  Format (1x,I3,10(1x,f12.8))

 101  Format ("Growth rate (sigma_i) = ",f15.10)

      Stop
      End
