       subroutine layersf0_s
     +
     +  (RL          ! wave length
     +  ,h           ! channel semi-width
     +  ,xi
     +  ,vs1,vs2     ! viscosities
     +  ,den1,den2   ! densities
     +  ,gamma0
     +  ,Ma
     +  ,Ds
     +  ,gac
     +  ,cr1,ci1
     +  ,sr1,si1
     +  ,cr2,ci2
     +  ,sr2,si2
     +  )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-----------------------------------------------------------
c  Stability of shear flow past a layer coated on a wall
c
c  1 refers to lower fluid, 2 to upper fluid.
c
c            Fluid 2
c   --------------------------
c            Fluid 1
c   =========================
c
c-----------------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension QR0(4,4),QI0(4,4)
      Dimension QR (4,4),QI (4,4)

      Double Precision Ma
      Double Precision k,ks
      Double Precision l,lc,LAMR,LAMI


c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

c-------- 
c prepare
c-------- 

      uI   = l*xi*h  ! interfacial velocity
      k    = pi2/RL        ! wave number
      ks   = k*k
      l    = vs2/vs1       ! viscosity ratio
      lc   = 1.0D0-l
      Drho = den1-den2 ! density difference

      hh   = k*h
      ch = Dcosh(hh)
      sh = Dsinh(hh)

      PIC = (gamma0+ Drho*gac/ks)/(2.0D0*vs1)  ! capital Pi

c----------------------------
c  initialize the secular matrix
c----------------------------

      Do i=1,4
       Do j=1,4
        QR0(i,j) = 0.0D0   ! real
        QI0(i,j) = 0.0D0   ! imaginary
       End Do
      End Do

c---------
c zeta = 0
c---------

      zR = 0.0D0
      zI = 0.0D0

c---
c real part
c---

      QR0(1,1) = ch+l*sh
      QR0(1,2) = -hh*ch
      QR0(1,3) =  hh*sh
      QR0(1,4) =  0.0D0

      QR0(2,1) = -sh-l*ch
      QR0(2,2) =  ch+hh*sh
      QR0(2,3) = -sh-hh*ch
      QR0(2,4) =  0.0D0

      QR0(3,1) =  lc 
      QR0(3,2) =  1.0D0
      QR0(3,3) =  0.0D0
      QR0(3,4) = -1.0D0

      QR0(4,1)  = lc
      QR0(4,2) = 0.0D0
      QR0(4,3) = 1.0D0
      QR0(4,4) = l

c--------------------
c compute the determinant
c--------------------

      call det_44c
     +
     +   (QR0,QI0
     +   ,detR0,detI0
     +   )

c---
c      Do i=1,4
c        write (6,101) (QR0,j),QI0(i,j),j=1,4)
c      End Do
c      write (6,102) DetR0,DetI0
c      pause
c---

c---------
c zeta = pm 1
c---------

c===
      Do irepeat=1,3
c===

      if(irepeat.eq.1) then
       zR = 1.0D0
       zI = 0.0D0
      elseif(irepeat.eq.2) then
       zR =-1.0D0
       zI = 0.0D0
      else
       zR3 = 3.8734D0 ! arbitrary
       zI3 =-1.9875D0 ! arbitrary
       zR = zR3
       zI = zI3
      end if

c Lambda:

      den  = (1.0-Ds*k*zI)**2 + (Ds*k*zR)**2
      cf   = 0.5*Ma*gamma0/(vs1*den)
      TMPR = -cf*Ds*k*zR
      TMPI = -cf*(1.0-Ds*k*zI)
      LAMR = zR*TMPR - zI*TMPI
      LAMI = zR*TMPI + zI*TMPR

c secular matrix:

      Do i=1,4
       Do j=1,4
        QR(i,j) = QR0(i,j) 
        QI(i,j) = QI0(i,j)
       End Do
      End Do

      XIC = xi/k

      QR(1,1) = QR(1,1) - PIC*zI*sh
      QI(1,1) = QI(1,1) + PIC*zR*sh

      QR(2,1) = QR(2,1) + PIC*zI*ch
      QI(2,1) = QI(2,1) - PIC*zR*ch

      QR(3,1) = QR(3,1) -zR*XIC*lc +zI*PIC
      QI(3,1) = QI(3,1) -zI*XIC*lc -zR*PIC

      tR = XIC*zR-1.0D0
      tI = XIC*zI
      QR(4,1) = QR(4,1) - (LAMR*tR-LAMI*tI)
      QI(4,1) = QI(4,1) - (LAMR*tI+LAMI*tR)
      QR(4,4) = QR(4,4) -  LAMR
      QI(4,4) = QI(4,4) -  LAMI

      call det_44c
     +
     +   (QR,QI
     +   ,detR,detI
     +   )

c---
c multiply the determinant by 1 + D_s k i zeta
c to render it a binomial in zeta
c where i is the imaginary unit
c---

      tmpR = 1.0-Ds*k*zI
      tmpI =     Ds*k*zR

      savR = detR*tmpR-detI*tmpI
      savI = detR*tmpI+detI*tmpR
      detR = savR
      detI = savI

c---
c       Do i=1,4
c         write (6,101) (QR(i,j),QI(i,j),j=1,4)
c       End Do
c      write (6,102) detR, detI
c---

c---

      if(irepeat.eq.1) then
       detR1 = detR
       detI1 = detI
      elseif(irepeat.eq.2) then
       detR2 = detR
       detI2 = detI
      else
       detR3 = detR
       detI3 = detI
      end if


c============
      End do
c============

c     write (6,*) detR0,DetI0
c     write (6,*) detR1,detI1
c     write (6,*) detR2,detI2
c     write (6,*) detR3,detI3

c===
c secular equation is: det(Q) = A Z^2 + B Z + det0 = 0
c
c where:
c
c   A + B = det1-det0
c   A - B = det2-det0
c===

      AR = 0.5D0*(detR1+detR2-2.0D0*detR0)
      AI = 0.5D0*(detI1+detI2-2.0D0*detI0)
      BR = 0.5D0*(detR1-detR2)
      BI = 0.5D0*(detI1-detI2)
      CR = detR0
      CI = detI0

c     write (6,*) "binomial coefficients:"
c     write (6,104) AR,AI,BR,BI,CR,CI

c====
c confirm the secular equation
c====

      zR3s = zR3*zR3-zI3*zI3
      zI3s = 2.0D0*zR3*zI3

      QuadR = AR*zR3s-AI*zI3s + BR*zR3-BI*zI3 + CR - detR3
      QuadI = AR*zI3s+AI*zR3s + BR*zI3+BI*zR3 + CI - detI3

c     write (6,*) "these should be zero:"
c     write (6,*) QuadR,QuadI

c====
c roots of the quadratic
c====

      call quadc
     +
     +  (AR,AI,BR,BI,CR,CI
     +  ,ZR1,ZI1
     +  ,ZR2,ZI2
     +  )

c====
c confirm the roots
c====

      zR1s = zR1*zR1-zI1*zI1
      zI1s = 2.0D0*zR1*zI1
      QuadR = AR*zR1s-AI*zI1s + BR*zR1-BI*zI1 + CR
      QuadI = AR*zI1s+AI*zR1s + BR*zI1+BI*zR1 + CI
c     write (6,*) "these should be zero:"
c     write (6,*) QuadR,QuadI

      zR2s = zR2*zR2-zI2*zI2
      zI2s = 2.0D0*zR2*zI2
      QuadR = AR*zR2s-AI*zI2s + BR*zR2-BI*zI2 + CR
      QuadI = AR*zI2s+AI*zR2s + BR*zI2+BI*zR2 + CI
c     write (6,*) "these should be zero:"
c     write (6,*) QuadR,QuadI

c-------------------------------------
c complex phase velocity: c = uI + 1/z
c-------------------------------------

      Dnm = ZR1*ZR1+ZI1*ZI1
      cr1 =  ZR1/Dnm
      ci1 = -ZI1/Dnm

      Dnm = ZR2*ZR2+ZI2*ZI2
      cr2 =  ZR2/Dnm
      ci2 = -ZI2/Dnm

c     cr1 = cr1+uI
c     cr2 = cr2+uI

c----
c complex growth rate
c--------------------

      sr1 = k*cr1
      si1 = k*ci1
      sr2 = k*cr2
      si2 = k*ci2

c-----
c Done
c-----

  101 Format (1x,4(3x,f8.5,1x,f8.5))
  102 Format ("Determinant:",1x,f15.10,1x,f15.10)
  103 Format ("zeta       :",1x,f15.10,1x,f15.10)
  104 Format (10(1x,f10.8))

      Return 
      End
