      program wave

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999.
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the FDLIB licensing agreement.
c----------------------------------------

c--------------------------------------
c Fourier analysis and Prony-fitting
c of periodic waves.
c
c The Fourier coefficients are fitted
c into normal modes.
c
c SYMBOLS:
c -------
c
c RL: spatial period
c
c x,y:	coordinates of points along the wave
c
c csf:	cosine Fourier coefficients
c snf:	sine   Fourier coefficients
c
c--------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension     x(200),    y(200)
      Dimension    sr(200),   si(200)
      Dimension   amp(200),  phi(200)
      Dimension csf(0:200),snf(0:200)

      Dimension   time(900),signal(900)
      Dimension sgn_cs(900),sgn_sn(900)

      Dimension z_cs(20,2),sigma_cs(20,2),c_cs(20,2)
      Dimension z_sn(20,2),sigma_sn(20,2),c_sn(20,2)

      Dimension amp_cs(20),phi_cs(20)
      Dimension amp_sn(20),phi_sn(20)

      Integer p

c----------
c constants
c----------

      pi  = 3.14159 265358D0
      pi2 = 2.0D0*pi

      Null = 0

c------
c Flags
c------

      Isee = 1

c------
c Input
c------

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 0 to quit"
      write (6,*) " 1 to generate the wave profiles"
      write (6,*) " 2 to read profiles from file: wave.inp"
      write (6,*) " --------------------------------------" 
      read  (5,*) Igen

      If(Igen.eq.0) Go to 99

c-----------------------
      If(Igen.eq.1) then
c-----------------------

        open (4,file="wave.dat")

        read (4,*) npts
        read (4,*) RL
        read (4,*) Dt
        read (4,*) Nprof   ! Number of wave profiles to be generated
        read (4,*) Nexp    ! Number of normal modes
        read (4,*) Ntype   ! progressive (1) or standing (2) wave
        read (4,*) Isee

        Do j=1,Nexp
          read (4,*)
          read (4,*) sr(j)
          read (4,*) si(j)
          read (4,*) amp(j)
          read (4,*) phi(j)
          phi(j) = phi(j)*pi
        End Do

        close (4)

        npts1 = npts+1
        npts2 = npts+2
        wn    = pi2/RL    ! wave number
        step   = RL/npts

c---------
      Else
c---------

        write (6,*)
        write (6,*) " How many normal modes to fit?"
        write (6,*)
        write (6,*) " Choose from 1, 2, 3"
        write (6,*) " -------------------" 
        read  (5,*) Nexp

        write (6,*)
        write (6,*) " Please enter Dt"
        write (6,*) " ----------------" 
        read  (5,*) Dt

        open (3,file="wave.inp")

c-----------
      End If
c-----------

c--------
c prepare
c--------

      Nexp2 = 2*Nexp
      Nexp4 = 4*Nexp
      Nmin  = Nexp4-1   ! minimum number of time-points needed

c-------------------------------------------
c Ready to generate the profiles and compute
c the Fourier coefficients
c-------------------------------------------

      Ic = 0     ! count the number of generated profiles

  98  Continue   ! return for another profile

      Ic = Ic+1

c------------------------
      If(Igen.eq.1) then  ! Generate profile
c------------------------

        If(Ic.eq.Nprof+1) Go to 93    ! Done generating profiles

        time(Ic) = (Ic-1.0D0)*Dt

        t = time(Ic)

c-------
        If(Ntype.eq.1) then    ! progressive waves
c-------

         Do i=1,Npts1

            x(i) = step*(i-1.0D0)
            wnx  = wn*x(i)
            y(i) = 0.0D0

            Do j=1,Nexp
             arg = sr(j)*t + phi(j)
             cs   = Dcos(arg)
             sn   = Dsin(arg)
             gr   = Dexp(si(j)*t)
             ampc = amp(j)*cs*gr
             amps = amp(j)*sn*gr
             y(i) = y(i) + ampc * Dcos(wnx)+amps * Dsin(wnx)
            End Do

         End Do

c-------
        Else               ! standing waves
c-------

          Do i=1,Npts1
            x(i) = step*(i-1.0D0)
            wnx = wn*x(i)
            y(i) = 0.0D0
            argt = sr(j)*t
            arge = si(j)*t
            Do j=1,Nexp
            y(i) = y(i) + amp(j) * Dcos(wnx) * Dcos(argt) * Dexp(arge)
            End Do
          End Do

c-------
        End If
c-------

c---------------------------
      Else If(Igen.eq.2) then   ! read profile from file: wave.inp
c---------------------------

        read (3,*) Npts1,time(Ic)

        If(Npts1.eq.0) Go to 93    ! Done reading

        Do i=1,Npts1
          read (3,*) j,x(i),y(i)
        End Do

        Npts  = Npts1-1
        Npts2 = Npts1+1

        RL = x(Npts1)-x(1)   ! period
        wn = pi2/RL          ! wave number

c-------------
        End If   ! end of generating a profile
c-------------

c-----------------------------
c Compute Fourier Coefficients
c-----------------------------

      call fourier (RL,Npts,x,y,M,csf,snf)

c-----------------------------
c Display Fourier Coefficients
c-----------------------------

c     write (6,*)
c     write (6,*) " Fourier Coefficients"
c     write (6,*)
c     write (6,*) "  p, csf(p), snf(p) "
c     write (6,*) " -------------------"
c     write (6,*)
c     Do p=0,M
c       write (6,101) p,csf(p),snf(p)
c     End Do
c     write (6,*)
c     write (6,*)
 
c-----------------------------------
c store the first-order coefficients
c-----------------------------------

      sgn_cs(Ic) = csf(1)
      sgn_sn(Ic) = snf(1)

      Go to 98  ! return for another profile

c---------------------------------------------
c time series of Fourier coefficients computed
c---------------------------------------------

 93   Continue

      Ic = Ic-1

c-----------------------------------------
c Printing session
c
c Print first cosine and sine coefficients
c-----------------------------------------

      open (9,file="wave.out")

      write (9,*) Ic," cosine Fourier signal"
      Do i=1,Ic
         write (9,101) i,time(i),sgn_cs(i)
      End Do
      write (9,*) Ic," sine Fourier signal"
      Do i=1,Ic
         write (9,101) i,time(i),sgn_sn(i)
      End Do

      If(Isee.eq.1) then

      write (6,*) Ic," cosine Fourier signal:"
      Do i=1,Ic
         write (6,101) i,time(i),sgn_cs(i)
      End Do
      write (6,*) Ic," sine Fourier signal:"
      Do i=1,Ic
         write (6,101) i,time(i),sgn_sn(i)
      End Do

      End If

c-------------------------------
c Proceed with the Prony fitting
c
c shift the time series 
c-------------------------------

 96   Continue

      write (6,*) 
      write (6,*) Ic," points are available"
      write (6,*) 
      write (6,*) " How many initial points shall I skip?"
      write (6,*) " Feel free to enter 0"
      write (6,*) " --------------------"
      read  (5,*) Iskip

      t0 = time(Iskip+1)
      Npoints = Ic-Iskip

c-----------------------------------------------
c Choose the number of points in the time series
c-----------------------------------------------

      write (6,*)
      write (6,*)  Npoints," time-points are available"
      write (6,*)
      write (6,*) " Please redefine the size of the set"
      write (6,*) " Must be greater than ",Nmin," otherwise I quit" 
      write (6,*) " ----------------------------------------------"
      read  (5,*) Npoints

      If(Npoints.lt.Nmin) Go to 97

c-----------------------------------------
c Prony fitting of the COSINE coefficients
c-----------------------------------------

      Do i=1,Npoints
        signal(i) = sgn_cs(i+Iskip)
      End Do

      call prony 
     +
     +   (Npoints
     +   ,signal
     +   ,Dt,Nexp
     +   ,z_cs
     +   ,sigma_cs
     +   ,c_cs
     +   )

      write (6,*) 
      write (6,*) " -------------------"
      write (6,*) " Cosine coefficients:"
      write (6,*)
      write (6,*) " sigma_r, sigma_i, a_nm, phi_nm"
      write (6,*) 

      Do i=1,Nexp2
       cfr = c_cs(i,1)
       cfi = c_cs(i,2)
       amp_cs(i) = Dsqrt(cfr**2+cfi**2)
       phi_cs(i) = Dacos(cfr/amp_cs(i))
       If(cfi.lt.0)  phi_cs(i) = - phi_cs(i)
       phi_cs(i) = - phi_cs(i)
       write (6,104) i,sigma_cs(i,1),sigma_cs(i,2)
     +                ,amp_cs(i),phi_cs(i)/pi
c    +                ,c_cs(i,1),    c_cs(i,2)
      End Do

c---------------------------------------
c Prony fitting of the SINE coefficients
c---------------------------------------

      Do i=1,Npoints
        signal(i) = sgn_sn(i+Iskip)
      End Do

      call prony 
     +
     +   (Npoints
     +   ,signal
     +   ,Dt,Nexp
     +   ,z_sn,sigma_sn
     +   ,c_sn
     +   )

      write (6,*) 
      write (6,*) " -----------------"
      write (6,*) " Sine coefficients:"
      write (6,*)
      write (6,*) " sigma_r, sigma_i, a_nm, phi_nm"
      write (6,*) 

      Do i=1,Nexp2
       cfr = c_sn(i,1)
       cfi = c_sn(i,2)
       amp_sn(i) = sqrt(cfr**2+cfi**2)
       phi_sn(i) = acos(cfi/amp_sn(i))
       If(cfr.lt.0) phi_sn(i) = - phi_sn(i)
       write (6,104) i,sigma_sn(i,1),sigma_sn(i,2)
     +                ,amp_sn(i),phi_sn(i)/pi
c    +                ,c_sn(i,1),c_sn(i,2)
      End Do

c---------------------------------------------------
c To find the proper sigma, compare the coefficients
c "amp" and phase shift "phi" of the cosine and sine
c signals, and pick up the matching pair
c---------------------------------------------------

c--
c Repeat with a different set 
c---

      write (6,*)
      write (6,*) " Repeat with a different set of points ? "
      write (6,*)
      write (6,*) " Enter 0 to decline"
      write (6,*) " ------------------"
      read  (5,*) Irepeat

      If(Irepeat.ne.0) Go to 96

  97  Continue

c---------------------------------
c Reconstruct the Fourier coefficients
c based on the Prony fitting
c---------------------------------

      write (9,101) Ic-Iskip

      Do i=1,Ic

        recc  = 0.0D0
        recs  = 0.0D0

        th = time(i)-t0

        If(th.ge.0) then

        Do j=1,Nexp
          loc = 2*j-1

c         cosine:
          argt = sigma_cs(loc,1)*th + phi_cs(loc)
          arge = sigma_cs(loc,2)*th
          recc = recc + amp_cs(loc)*cos(argt)*exp(arge)

c         sine:
          argt = sigma_sn(loc,1)*th + phi_sn(loc)
          arge = sigma_sn(loc,2)*th
          recs = recs + amp_sn(loc)*sin(argt)*exp(arge)

        End Do

        write (9,101) i,time(i),recc,recs

        End If
      End Do

c---------
c Done
c---------

  99  Continue

      write (9,*) null

      close (9)

 100  Format (1x,f12.7,10(1x,f12.7,1x))
 101  Format (1x,I3,10(1x,f12.7))
 103  Format (10(1x,f9.6))
 104  Format (1x,i2,10(1x,f11.7))

      Stop
      End
