      program rayleigh

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c==========================================

c--------------------------------------------
c  Normal mode analysis of unidirectional inviscid
c  shear flow along the x axis
c
c  The flow is confined between two parallel walls
c  located at y = a and -b
c
c  SYMBOLS:
c  -------
c
c  U:   velocity, function of y
c  Upp:	second derivative of U with respect to y
c  N:   level of discretization of y axis within (a,b)
c
c  kdiv:  number of graph-points
c         will span the wave-number range (0, kmax)
c
c  epsr: step for Newton corrections
c  epsi: step for Newton corrections
c  tole: tolerance for convergence	
c
c  cr, ci:	real and imaginary parts of c
c-------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension U(257),Up(257),Upp(257), Upp_num(257)

      Double Precision k,kstart,kend

c----------
c constants
c----------

      null = 0

c------
c input
c------

 98   Continue

      write (6,*) 
      write (6,*) " Enter:"
      write (6,*) 
      write (6,*) " 1 to read parameters from file: rayleigh.dat"
      write (6,*) " 2 to type in parameters (not recommended)"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) Ienrd

      if(Ienrd.eq.0) Go to 99

c------------------------
      if(Ienrd.eq.2) then  !  read parameters
c------------------------

      write (6,*)
      write (6,*) " Select the velocity profile"
      write (6,*) 
      write (6,*) ' 1 for U=U0*(d*tanh(y/b)+(d-1)*exp(-y^2/b^2))'
      write (6,*) ' 2 for U=U0/cosh(y/b)**2'
      write (6,*) ' 0 to quit'
      write (6,*) ' ---------'
      read  (5,*) menu
      if(menu.eq.0) Go to 99

      if(menu.eq.1) then

        write (6,*)
        write (6,*) ' Please enter: U0, delta, alpha'
        write (6,*) ' ----------------------------'
        read  (5,*) U0,delta,alpha

      else if(menu.eq.2) then

        write (6,*)
        write (6,*) ' Please enter the constants U0, delta'
        write (6,*) ' --------------------------------'
        read  (5,*) U0,delta

      End If

      write (6,*) 
      write (6,*) " Flow confined between y=a and -b"
      write (6,*) " Enter a and b"
      write (6,*) " ------------"
      read  (5,*) a,b

      write (6,*)
      write (6,*) " The number of finite-difference divisions is N"
      write (6,*)
      write (6,*) " Please enter N"
      write (6,*) " --------------"
      read  (5,*) N

      write (6,*)
      write (6,*) " Stability graph will span the wavenumber"
      write (6,*) " interval (kstart, kend) with N divisions"
      write (6,*)
      write (6,*) " Please enter kstart"
      write (6,*) " ------------------"
      read  (5,*) kstart

      write (6,*)
      write (6,*) " Please enter kend"
      write (6,*) " -----------------"
      read  (5,*) kend

      write (6,*)
      write (6,*) " Enter the number of increments in k"
      write (6,*) ' 0 to quit'
      write (6,*) ' ---------'
      read  (5,*) kdiv
      if(kdiv.eq.0) Go to 99

      write (6,*)
      write (6,*) " Enter initial guess for the real and" 
      write (6,*) "       imaginary part of the complex"
      write (6,*) "       phase velocity, cr ,ci"
      write (6,*) " ------------------------------------"
      read  (5,*) cr,ci

      newter = 20        ! max Newton iterations
      epsr = 0.01    ! for Newton iterations
      epsi = 0.01    ! for Newton iterations
      tole  = 0.000001  ! for stopping Newton iterations

c---------
      else
c---------

      open (1,file="rayleigh.dat")

       read (1,*) a,b
       read (1,*)
       read (1,*) menu
       read (1,*) U0 
       read (1,*) delta
       read (1,*) alpha
       read (1,*)
       read (1,*) N
       read (1,*) kstart
       read (1,*) kend
       read (1,*) kdiv
       read (1,*) cr,ci
       read (1,*) 
       read (1,*) newter
       read (1,*) epsr
       read (1,*) epsi
       read (1,*) tole

      close (1)

c     write (6,*) U0,delta,alpha

c-----------
      end if
c-----------

c--------
c prepare
c--------

      open (2,file="rayleigh.out")

      denn= epsr**2 + epsi**2 
      Dk = (kend-kstart)/(kdiv+1.0D0-1.0D0)
      Dy  = (a+b)/N
      Dys = Dy**2

c-----------------------------------
c compute the velocity profile
c and the second derivative Upp
c by numerical differentiation
c-----------------------------------

      write (6,*)
      write (6,*) "Velocity profile: i, U, Upp"
      write (6,*)


      Do i=1,N+1

       y = -b+(i-1.0D0)*Dy

       call rayleigh_vel (menu,U0,delta,alpha,y,U(i),Up(i),Upp(i))

       Ido = 0
       if(Ido.eq.1) then
        epsU   = 0.00001
        epsUs  = epsU**2
        y1 = y+epsU
        y2 = y-epsU
        call rayleigh_vel (menu,U0,delta,alpha,y1,U1,Up1,Upp1)
        call rayleigh_vel (menu,U0,delta,alpha,y2,U2,Up2,Upp2)
        Upp_num(i) = (U1-2.0D0*U(i)+U2)/epsUs
        Upp(i) = Upp_num(i)
       end if

       write (6,101) i,y,U(i),Upp(i),Upp_num(i)

      End Do

c--------------------------------------------------
c scan the wave-number axis from k = kstart to kend
c--------------------------------------------------


      write (6,*) 
      write (6,*) "  k, cr, ci, growth rate, number of iterations"
      write (6,*) 
      write (2,100) kdiv+1

c--------------------------------------------------
c scan the wave-number axis from k = kstart to kend
c--------------------------------------------------

c=======================
      Do kloop=1,kdiv+1    ! scan wave number axis
c=======================

      k = kstart+(kloop-1)*Dk;

c---
c  Newton iterations
c---

        Do iter=1,newter   ! Newton iterations 

         call rayleigh_sys (N,Dys,U,Upp,k,cr,ci,detr,deti)

c        write (6,*) N,k,detr,deti
c        stop

         cr = cr + epsr
         ci = ci + epsi

         call rayleigh_sys (N,Dys,U,Upp,k,cr,ci,detr1,deti1)

c        write (6,*) N,k,detr,deti
c        write (6,*) epsr,epsi,detr1,deti1
c        stop

         cr = cr - epsr   ! reset
         ci = ci - epsi   ! reset

c---
c compute the derivative d(det(E))/dc
c---

         Ddetr = detr1-detr
         Ddeti = deti1-deti
         DdetDcr = ( Ddetr*epsr + Ddeti*epsi)/denn
         DdetDci = (-Ddetr*epsi + Ddeti*epsr)/denn

c        write (6,*) DdetDcr,DdetDci
c        stop

c---
c compute the correction
c---

         den = DdetDcr**2+DdetDci**2

         corrr = ( detr*DdetDcr+deti*DdetDci)/den
         corri = (-detr*DdetDci+deti*DdetDcr)/den

c---
c make the correction
c---

        cr = cr - corrr
        ci = ci - corri

        corr = sqrt(corrr**2+corri**2)
        if(corr.lt.tole) Go to 92    ! success

c---
      End Do ! end of newton iterations
c---

      write (6,*) " rayleigh: Newton terations did not converge"
      Go to 99

 92   Continue

c---
c printing session
c---

      sigma = ci*k

      write (2,100) kloop,k,cr,ci,sigma,iter
      write (6,100) kloop,k,cr,ci,sigma,iter

      k = k+dk

c----
      End Do  ! end of scanning the wave number
c----

      Go to 98

c-----
c Done
c-----

  99  Continue

      write (2,100) Null
      close (2)

  100 Format (1x,i3,4(1x,f10.5),1x,i3)
  101 Format (1x,i3,10(f10.5),1x,i3)

      stop
      end
