      subroutine rayleigh_sys (N,Dys,U,Upp,k,creal,cimag,detr,deti)

c========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c========================================

c---------------------------------
c Compile the (N-1)x(N-1) tridiagonal matrix E
c and compute the determinant
c
c matrix has the structure:
c
c       |a b
c       |  c a b
c E =   |    c a b
c       |      c a b
c
c ar is the real part of a
c ai is the imaginary part of a
c---------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension U(257),Upp(257)

      Dimension ar(255),ai(255)
      Dimension br(255),bi(255)
      Dimension cr(255),ci(255)
      Dimension Pr(255),Pi(255)

      Double Precision k

c---
c generate the tridiagonal matrix
c---

      Do i=2,N

         den   = (U(i)-creal)**2+cimag**2
         ar(i-1) = -2.0-Dys*(k**2 + Upp(i)*(U(i)-creal)/den)
         br(i-1) = 1.0D0
         cr(i-1) = 1.0D0

         ai(i-1) = - Dys*Upp(i)*cimag/den
         bi(i-1) = 0.0D0
         ci(i-1) = 0.0D0

c         write (6,*) creal,cimag,den
c         write (6,*) i,ar(i),br(i),cr(i)
c         write (6,*) i,ai(i),bi(i),ci(i)

      End Do

c---
c Compute the determinant of the 
c complex matriix E
c using the nifty method
c---

      Pr(1) = ar(1)
      Pi(1) = ai(1)
      tmpr  = br(1)*cr(2)-bi(1)*ci(2)
      tmpi  = br(1)*ci(2)+bi(1)*cr(2)
      Pr(2) = ar(2)*Pr(1)-ai(2)*Pi(1)-tmpr
      Pi(2) = ar(2)*Pi(1)+ai(2)*Pr(1)-tmpi

      Do i=3,N-1
          ia    = i-1
          tmpr  = br(ia)*cr(i)-bi(ia)*ci(i)
          tmpi  = br(ia)*ci(i)+bi(ia)*cr(i)
          ib    = i-2
          auxr  = tmpr*Pr(ib)-tmpi*Pi(ib)
          auxi  = tmpr*Pi(ib)+tmpi*Pr(ib)
          Pr(i) = ar(i)*Pr(ia)-ai(i)*Pi(ia)-auxr
          Pi(i) = ar(i)*Pi(ia)+ai(i)*Pr(ia)-auxi
      End Do

      detr = Pr(N-1)
      deti = Pi(N-1)

c-----
c Done
c-----

      return
      end
