      subroutine thread0
     +
     + (a
     + ,RL
     + ,vst
     + ,vss
     + ,srtn
     + ,gr
     + )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-------------------------------------------------------
c Rayleigh (capillary) instability of an infinite thread
c of a viscous fluid surrounded by another viscous fluid
c
c SYMOBOLS
c
c srtn: surface tension
c vst:	viscosity of the thread
c vss:	viscosity of the surrounding fluid
c
c RL:	wave length
c a:	thread radius
c-------------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Double Precision I0,I1,K0,K1

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

c--------
c prepare
c--------

      x  = a*pi2/RL
      xs = x*x

      Iselect = 2    ! need I and K

      call bess_I01K01 
     +
     +   (x
     +   ,Iselect
     +   ,I0,I1
     +   ,K0,K1
     +   )

c--------------------
c Tomotika's formulae
c--------------------

c---------------------------
      If(vss.lt.0.000000001) then    ! equation (36)
c---------------------------

        factor = 0.5D0*srtn/(a*vst)
        phi    = (xs-1.0D0)/(xs+1.0D0-xs*I0*I0/I1**2)
        gr     = factor*phi

c---------
      Else
c---------

      vsr = vss/vst   ! viscosity ratio

      a11 = x*I0-I1
      a12 = K1
      a13 = -x*K0-K1

      a21 = I0+x*I1
      a22 = -K0
      a23 = -K0+x*K1

      a31 =  x*I0
      a32 =  vsr*K1
      a33 = -vsr*x*K0 

      Det1 = a11*(a22*a33-a23*a32)
     +     - a12*(a21*a33-a23*a31)
     +     + a13*(a21*a32-a22*a31) 

      a11 = I1
      a21 = I0
      a31 = I1
      
      Det2 = a11*(a22*a33-a23*a32)
     +     - a12*(a21*a33-a23*a31)
     +     + a13*(a21*a32-a22*a31) 

      a12 = x*I0-I1
      a22 = I0+x*I1
      a32 = x*I0

      Det3 = a11*(a22*a33-a23*a32)
     +     - a12*(a21*a33-a23*a31)
     +     + a13*(a21*a32-a22*a31) 

      a13 = K1
      a23 = -K0
      a33 = vsr*K1

      Det4 = a11*(a22*a33-a23*a32)
     +     - a12*(a21*a33-a23*a31)
     +     + a13*(a21*a32-a22*a31) 

      uppp = I1*det1 - (x*I0-I1)*Det2

      down = (x*I0-I1)                * Det1
     +     - ((xs+1.0D0)*I1-x*I0)     * Det2 
     +     - vsr*( x*K0+K1)           * Det3
     +     - vsr*((1.0D0+xs)*K1+x*K0) * Det4

      phi    = uppp/down

      factor = 0.5D0*srtn/(a*vst)
      gr     = factor*(1.0D0-xs)*phi

c-----------
      End If
c-----------

c-----
c Done
c-----

      Return 
      End
