      program pvm

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-----------------------------------
c Motion of a group of point vortices
c
c SYMBOLS:
c -------
c
c x, y:   coordinates of the point vortices
c circ:   circulation of the point vortices
c---------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x(-10:100),y(-10:100),circ(-10:100)
      Dimension u(-10:100),v(-10:100)

c---
c for Runge-Kutta
c---

      Dimension xsv(-10:100),ysv(-10:100)

      Dimension usv1(-10:100),vsv1(-10:100)
      Dimension usv2(-10:100),vsv2(-10:100)
      Dimension usv3(-10:100),vsv3(-10:100)

c--------------
c common blocks
c--------------

      common/xxyy/x,y,circ
      common/uuvv/u,v

c----------
c constants
c----------

      pi  = 3.14159265358
      pi2 = 2.0*pi

c----------
c main menu
c----------

      write (6,*) 
      write (6,*)  "            MENU"
      write (6,*) 
      write (6,*) "Choose the flow arrangement."
      write (6,*) 
      write (6,*) "Enter 0 to quit"
      write (6,*) "      1 for flow in infinite space"
      write (6,*) '----------------------------------'
      read  (5,*) Iflow

      If(Iflow.eq.0) Go to 99

c---------------
c Set parameters
c---------------

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 0 to quit"
      write (6,*) " 1 to read parameters from file: pvm.dat"
      write (6,*) " 2 to enter parameters through the keyboard"
      write (6,*) '-------------------------------------------'
      read  (5,*) Ienrd

      If(Ienrd.eq.0) Go to 99

c------------------------
      If(Ienrd.eq.2) then
c------------------------

      write (6,*) 
      write (6,*) 'How many point vortices ?'
      write (6,*) '-------------------------'
      read  (5,*) Npv

      write (6,*)
      write (6,*) 'Please enter the order of the Runge-Kutta method'
      write (6,*) 'for time integration'
      write (6,*)
      write (6,*) 'Choose from 1, 2, 4'
      write (6,*) '-------------------'
      read  (5,*) IRK

      write (6,*)
      write (6,*) 'Please enter time step'
      write (6,*) '---------------------'
      read  (5,*) Dt

      write (6,*)
      write (6,*) 'After how many steps print the position ?'
      write (6,*) '-----------------------------------------'
      read  (5,*) Nprint

      write (6,*)
      write (6,*) 'Enter number of time steps till pause'
      write (6,*) '-------------------------------------'
      read  (5,*) Nstep

      Do i=1,Npv
        write (6,*) 'Point vortex:',i
        write (6,*) 
        write (6,*) 'Please enter:' 
        write (6,*) 
        write (6,*) '   the point vortex x and y coordinates'
        write (6,*) '   the circulation around the point vortex'
        write (6,*) '-------------------------------------------'
        read (5,*) x(i),y(i),circ(i)
      End Do

c---------
      Else
c---------

       open (3,file="pvm.dat")

        read (3,*) IRK
        read (3,*) Dt
        read (3,*) Nprint
        read (3,*) Nstep

       close (3)

       open (3,file="pvm_pv.dat")

        read (3,*) Npv,time
        Do i=1,Npv
          read (3,*) idle,x(i),y(i),circ(i)
        End Do

       close (3)

c-----------
      End If
c-----------

c----------------
c prepare to run
c----------------

      open (8,file="pvm.xy")

      Iprint = Nprint
      Jprint = 0

c-----------------------
c begin time integration
c-----------------------

      Kstep = 1      ! Global counter
      Istep = 1      ! Batch  counter

  98  Continue

c---
c compute the velocity
c---

      call pvm_vel
     +
     +   (Npv
     +   ,Iflow
     +   )

c-----------------
c Printing session
c-----------------

      If(Iprint.eq.Nprint) then

	Jprint = Jprint + 1

        write (8,100) Npv,time
        write (6,100) Npv,time

        Do i=1,Npv
          write (8,100) i,x(i),y(i),u(i),v(i)
          write (6,100) i,x(i),y(i),u(i),v(i)
        End Do

        Iprint = 0

      End If

c-------------------------------------
c time integration by the Euler method
c-------------------------------------

      If(IRK.eq.1) then

      Do i=1,Npv
       x(i) = x(i) + u(i)*Dt
       y(i) = y(i) + v(i)*Dt
      End Do

c----------------------------
c time integration by the RK2
c----------------------------

      Else If(IRK.eq.2) then

      Do i=1,Npv
        xsv(i) = x(i)        ! save position
        ysv(i) = y(i)        ! save position
       usv1(i) = u(i)        ! save velocity
       vsv1(i) = v(i)        ! save velocity
       x(i) = xsv(i) + u(i)*Dt
       y(i) = ysv(i) + v(i)*Dt
      End Do

      call pvm_vel               ! second velocity
     +            (Npv
     +            ,Iflow
     +            )

      Do i=1,Npv
       u(i) = 0.5*(usv1(i)+u(i))    ! average velocity
       v(i) = 0.5*(vsv1(i)+v(i))
       x(i) = xsv(i) + u(i)*Dt
       y(i) = ysv(i) + v(i)*Dt
      End Do

c----------------------------
c time integration by the RK4
c----------------------------

      Else If(IRK.eq.4) then

      Dth = 0.50*Dt

      Do i=1,Npv
        xsv(i) = x(i)
        ysv(i) = y(i)
       usv1(i) = u(i)
       vsv1(i) = v(i)
       x(i) = xsv(i) + u(i)*Dt
       y(i) = ysv(i) + v(i)*Dt
      End Do

      call pvm_vel
     +            (Npv
     +            ,Iflow
     +            )

      Do i=1,Npv
       usv2(i) = u(i)
       vsv2(i) = v(i)
          x(i) = xsv(i) + u(i)*Dth
          y(i) = ysv(i) + v(i)*Dth
      End Do

      call pvm_vel
     +            (Npv
     +            ,Iflow
     +            )

      Do i=1,Npv
       usv3(i) = u(i)
       vsv3(i) = v(i)
          x(i) = xsv(i) + u(i)*Dt
          y(i) = ysv(i) + v(i)*Dt
      End Do

      call pvm_vel
     +            (Npv
     +            ,Iflow
     +            )

      Do i=1,Npv
       u(i) = (usv1(i)+2.0*usv2(i)+2.0*usv3(i)+u(i))/6.0
       v(i) = (vsv1(i)+2.0*vsv2(i)+2.0*vsv3(i)+v(i))/6.0
       x(i) = xsv(i) + u(i)*Dt
       y(i) = ysv(i) + v(i)*Dt
      End Do

      End If

c-------------------------
c  Reset counters and time
c-------------------------

      Kstep  = Kstep+1
      Istep  = Istep+1
      Iprint = Iprint+1

      time   = time+Dt

c--------------
c batch inquiry
c--------------

      If(Istep.eq.Nstep) then

       write (6,*) 
       write (6,*) " How many steps to pause ?"
       write (6,*) "           Enter 0 to quit"
       write (6,*) " -------------------------"

       read  (5,*) Nstep

       If(Nstep.eq.0) Go to 99

       Istep = 1
       Go to 98

      End If

c-------------
c another step
c-------------

       Go to 98

c-----
c Done
c-----

  99  Continue

      write (8,100) null
      close (8)

 100  Format (1x,i4,10(1x,f10.5))

      Stop
      End
