      subroutine arc_2d 
     +
     +  (Iarc
     +  ,X1,X2,X3
     +  ,Y1,Y2,Y3
     +  ,XC,YC,R
     +  ,TH1,TH2,TH3,ORNT
     +  ,AREA1,AREA2,AREA
     +  ,XCN1,XCN2,XCN
     +  ,YCN1,YCN2,YCN
     +  ,Istop
     +  )

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement.
c----------------------------------------

c------------------------------------------------------------
c  Properties of the circular arc passing through
c  three points numbered 1,2,3
c
c  Legend
c
c  (xc,yc):	coordinates of arc center
c  th1,th2,th3:	angles subtended by the arc center and
c		each one of the points
c
c  ornt:	arc orientation: 1 for counterclockwise
c			        -1 for clockwise
c
c  area1	area between the points 12 and the x-axis
c  area2	area between the points 23 and the x-axis
c  area         area1+area2
c
c  (xcn1,ycn1)	areal centroid integral of the first part of the arc
c  (xcn2,ycn2)	areal centroid integral of the second part of the arc
c  (xcn,ycn)	areal centroid integral of the whole arc
c
c------------------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Parameter (eps=0.0000000001,tol=0.001)

c---
c constants
c---

      pi  = 3.1415 92653 58979 32384 
      pi2 = 2.0*pi

      epsm = -eps

c---
c arc orientation
c---
 
      ORNT = (X2-X1)*(Y3-Y1)-(X3-X1)*(Y2-Y1)
      D1   = sqrt((X2-X1)**2+(Y2-Y1)**2)
      D2   = sqrt((X3-X1)**2+(Y3-Y1)**2)
      ORNT = ORNT/(D1*D2)
      
      If(ORNT.gt.eps) then
        ORNT = 1.0
      Else If(ORNT.lt.epsm) then
        ORNT =-1.0
      Else
        write (6,*) 
        write (6,*) ' Arc is not defined'
        write (6,*) 
        Istop = 1
        Go to 99
      End If

c---
c arc center
c---

      A1=(Y2-Y1)/(X2-X1)
      A2=(Y3-Y2)/(X3-X2)

      If(abs(A1).LE.0.0000000001) A1 = 0.0000000001
      If(abs(A2).LE.0.0000000001) A2 = 0.0000000001

      AA1 = -1.0/A1
      AA2 = -1.0/A2

      XX1 = 0.5*(X1+X2)
      YY1 = 0.5*(Y1+Y2)
      XX2 = 0.5*(X2+X3)
      YY2 = 0.5*(Y2+Y3)

      GG1 = YY1-AA1*XX1
      GG2 = YY2-AA2*XX2

      XC = (GG1-GG2)/(AA2-AA1)
      YC = AA1*XC+GG1

c---
c arc radius
c---

      R2 = (X1-XC)*(X1-XC)+(Y1-YC)*(Y1-YC)
      R  = SQRT(R2)

c---
c angles subtended by the three points
c from the arc center
c---

      ARG = (X1-XC)/R

      If(ARG.GT.1.0) then
        write (6,*)
        write (6,300) ARG
        write (6,*)
        ARG = 1.0
      Else If(ARG.LT.-1.0) then
        write (6,*)
        write (6,*) ARG
        write (6,*)
        ARG = -1.0
      End If

      TH1  =  ACOS(ARG)
      SINA =  Y1-YC

      If(SINA.LT.-0.000000001) TH1 = pi2-TH1

      PROD2 = (X1-XC)*(X2-XC)+(Y1-YC)*(Y2-YC)
      PROD3 = (X1-XC)*(X3-XC)+(Y1-YC)*(Y3-YC)

      DTH2 = ORNT*ACOS(PROD2/R2)
      DTH3 = ORNT*ACOS(PROD3/R2)

      TH2  = TH1+DTH2
      TH3  = TH1+DTH3

c----------------------
c Computes integrals:
c
c Int(y n_y dl) = Int( y dx) = Int(dA)
c Int(xy dx) = Int(x dA)
c Int(yy dx) = Int(y dA)
c
c----------------------

      COS1  = COS(TH1)
      COS2  = COS(TH2)
      COS3  = COS(TH3)
      SIN1  = SIN(TH1)
      SIN2  = SIN(TH2)
      SIN3  = SIN(TH3)

c---
c first part of the arc
c---

      DTH   = TH2           -TH1
      DSIN2 = SIN(2.0*TH2)  -SIN(2.0*TH1)
      DCS   = COS2          -COS1
      DCS3  = COS2**3       -COS1**3
      DSN   = SIN2          -SIN1
      DSN3  = SIN2**3       -SIN1**3

      AREA1 = -R*( YC*DCS - 0.5*R*(DTH-0.50*DSIN2))

      XCN1 = 0.5*R*( XC**2*DSN
     +              +R*XC*(DTH+0.5*DSIN2)
     +              +R**2*(DSN-DSN3/3.0) )
      YCN1 = 0.5*R*(-YC**2*DCS
     +              +R*YC*(DTH-0.5*DSIN2)
     +              -R**2*(DCS-DCS3/3.0) )

c---
c second part of the arc
c---

      DTH   = TH3           -TH2
      DSIN2 = SIN(2.0*TH3) -SIN(2.0*TH2)
      DCS   = COS3         -COS2
      DCS3  = COS3**3      -COS2**3
      DSN   = SIN3         -SIN2
      DSN3  = SIN3**3      -SIN2**3

      AREA2 = -R*( YC*DCS - 0.5*R*(DTH-0.50*DSIN2))

      XCN2 = 0.5*R*(XC**2 * DSN
     +             +2.0*R*XC*(0.50*DTH+0.25*DSIN2)
     +             +R**2    *(DSN-DSN3/3.0) )

      YCN2 = 0.5*R*(-YC**2 * DCS
     +             +R*YC*(DTH-0.5*DSIN2)
     +             -R**2*(DCS-DCS3/3.0) )

c---
c whole of the arc
c---

      AREA  = AREA1 + AREA2
      XCN   = XCN1  + XCN2
      YCN   = YCN1  + YCN2

c----------------------
c check for consistency
c----------------------

      XX1 = XC+R*COS(TH1)
      YY1 = YC+R*SIN(TH1)

      XX2 = XC+R*COS(TH2)
      YY2 = YC+R*SIN(TH2)

      XX3 = XC+R*COS(TH3)
      YY3 = YC+R*SIN(TH3)

      D1 = XX1-X1       ! error
      D2 = XX2-X2
      D3 = XX3-X3

      F1 = YY1-Y1       ! error
      F2 = YY2-Y2
      F3 = YY3-Y3

      If(abs(D1).GE.tol.or.
     +   abs(F1).GE.tol) then
         write (6,*)
         write (6,101) Iarc,X1,XX1,Y1,YY1
         write (6,*)
         Istop = 1
         Go to 99
      End If

      If(abs(D2).GE.tol.or.
     +   abs(F2).GE.tol) then
         write (6,*)
         write (6,102) Iarc,X2,XX2,Y2,YY2
         write (6,*)
         Istop = 1
         Go to 99
      End If

      If(abs(D3).GE.tol.or.
     +   abs(F3).GE.tol) then
         write (6,*)
         write (6,103) Iarc,X3,XX3,Y3,YY3
         write (6,*)
         Istop = 1
      End If

c---

  99  Continue

 100  Format (10(1X,F10.5))
 101  Format (' arc_2d: Problem with point 1 of arc ',I3,5(1x,F13.9))
 102  Format (' arc_2d: Problem with point 2 of arc ',I3,5(1x,F13.9))
 103  Format (' arc_2d: Problem with point 3 of arc ',I3,5(1x,F13.9))

 300  Format (' WARNING IN ARC',/,
     +        ' POINT 1 IS ALIGNED WITH THE X AXIS',/,
     +        ' DIRECTION COSINE WAS FOUND TO BE ',F15.12,/,
     +        ' WILL BE SET EQUAL TO +1 OR -1')

      Return
      End
