      subroutine vp_ax_vel
     +                    (Istop
     +                    )

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement.
c----------------------------------------

c----------------------------------------------
c Computes the velocity at the marker points of
c axisymmetric patch Ncntr contours
c----------------------------------------------

      Implicit Double Precision (a-h,o-z)

c---
c global variables
c---

      Dimension xg(0:166,25),yg(0:166,25)
      Dimension ug(0:166,25),vg(0:166,25)

      Dimension XCg(166,25), YCg(166,25),  Rg(166,25),   Sg(166,25)
      Dimension TH1g(166,25),TH2g(166,25),TH3g(166,25),orntg(166,25)

c---
c properties of patches
c---

      Dimension NSG  (25),NSG1(25),NSG2(25)
      Dimension omega(25)

c---
c common blocks for global variables
c---

      common/XXYYg/Xg,Yg
      common/ARCCg/XCg,YCg,Rg,Sg,TH1g,TH2g,TH3g,ORNTg
      common/UUVVg/Ug,Vg

c---
c various common blocks
c---

      common/ancI1/Ncntr,NSG,NSG1,NSG2
      common/ancR1/omega

      common/piii/pi,pih,piq,pi2,pi4,pi6,pi8,srpi

c---------------------------
c Contour integral evaluation
c---------------------------

      Do l = 1,Ncntr       ! run over contours

      Do j = 1,NSG1(l)          ! run over points

c      write (6,200) j

       ja  =  j-1
       j1  =  j+1

        X0 =   xg(j,l)
        Y0 =   yg(j,l)
       TH0 = th2g(j,l)

       Utot = 0.
       Vtot = 0.

c--------------------------------------
c Non-singular integrals
c will be computed over the blended arc
c--------------------------------------

       Do 20 k = 1,Ncntr     ! run over interfaces

       accx = 0.0     ! accumulator
       accy = 0.0

       Do 21 i = 1,NSG(k)      ! run over elements
 
c     write (6,201) i
 
       i1 = i+1

       If(l.eq.k) then
        If(j.eq.i.or.j.eq.i1) Go to 21
       End If

       XC1 =   XCg(i,k)
       YC1 =   YCg(i,k)
       RD1 =    Rg(i,k)
       T11 =  TH2g(i,k)
       T21 =  TH3g(i,k)
       OR1 = ORNTg(i,k)

       XC2 =   XCg(i1,k)
       YC2 =   YCg(i1,k)
       RD2 =    Rg(i1,k)
       T12 =  TH1g(i1,k)
       T22 =  TH2g(i1,k)
       OR2 = ORNTg(i1,k)

       call arc_bl
     +
     +     (XC1,YC1,RD1,T11,T21,OR1
     +     ,XC2,YC2,RD2,T12,T22,OR2
     +     ,X0,Y0
     +     ,Uarc,Varc
     +     ,Istop
     +     )

c---
c      write (6,100) k,i,Uarc,Varc
c---

       accx = accx + Uarc
       accy = accy + Varc

  21  Continue

      Utot = Utot + omega(k)*accx
      Vtot = Vtot + omega(k)*accy

  20  Continue

c--------------------------------------------------
c will integrate over the two sides of the host arc
c numbered j located at the lth interface
c-------------------------------------------------

      Ising = 1

      XCS =   XCg(j,l)
      YCS =   YCg(j,l)
      RDS =    Rg(j,l)
      T1S =  TH1g(j,l)
      T2S =  TH2g(j,l)
      T3S =  TH3g(j,l)
      ORS = ORNTg(j,l)

c---
      If(j.eq.1) Go to 88   ! skip the first part
c---

      call arc_host
     +
     +     (XCS,YCS,RDS,T1S,T2S,ORS
     +     ,X0,Y0,TH0
     +     ,Uarc,Varc
     +     ,Istop
     +     )

      Utot = Utot + omega(l)*Uarc
      Vtot = Vtot + omega(l)*Varc

 88   Continue

c---
      If(j.eq.NSG1(l)) Go to 89    ! skip the second part
c---

      call arc_host
     +
     +     (XCS,YCS,RDS,T2S,T3S,ORS
     +     ,X0,Y0,TH0
     +     ,Uarc,Varc
     +     ,Istop
     +     )

      Utot = Utot + omega(l)*Uarc
      Vtot = Vtot + omega(l)*Varc

  89  Continue


      Ug(j,l) = Utot/pi4
      Vg(j,l) = Vtot/pi4

      End Do       ! end of loop over points

      End Do           ! end of loop over contours

c-----------------------

 100  Format (1x,i3,1x,i3,10(1x,f15.10))
 200  Format (' EVALUATING THE CONTOUR INTEGRAL AT POINT:',I2)
 201  Format (' Arc:',I2)

      Return
      End

c==============================================

      subroutine arc_host
     +
     +     (XC,YC,RD,TH1,TH2,ORNT
     +     ,X0,Y0,TH0
     +     ,Uarc,Varc
     +     ,Istop
     +     )

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement.
c----------------------------------------

c---------------------------------------
c Computes the singular contour integral
c over a circular arc
c----------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension ZZ(20),WW(20)

      common/ancI3/NGL
      common/ZZWW/ZZ,WW
      common/piii/pi,pih,piq,pi2,pi4,pi6,pi8,srpi

c---
c constants
c---

      Null = 0
      Zero = 0.0

c---
c initialize
c---

      Istop  = 0

      Uarc = 0.0
      Varc = 0.0

      vnx0 = cos(TH0)*ORNT
      vny0 = sin(TH0)*ORNT

c---------------------------
c Prepare for Gauss-Legendre
c---------------------------

      STH = 0.5*(TH1+TH2)
      DTH = 0.5*(TH2-TH1)

c----

      Do 1 I=1,NGL

        AN  = STH + DTH*ZZ(I)
        CS  = COS(AN)
        SN  = SIN(AN)
        XX  = XC + RD * CS
        YY  = YC + RD * SN
        vnx = cs*ORNT
        vny = sn*ORNT 

        call kernel (x0,y0,xx,yy,RI10,RI11)

        U = - ( (x0-xx)*RI10*vnx + yy*RI11*vny)*yy
        V = RI11*vnx*yy**2

c---  ! desingularization disabled
c
c       rks = log((AN-TH0)**2)      ! for desingularization
c       U = U - rks*tnx0    ! subtract off singularity
c       V = U - rks*tny0
c-------------------------------------

        Uarc = Uarc + U*WW(I)
        Varc = Varc + V*WW(I)

   1  Continue

c----

      cf = DTH*ORNT*RD

      Uarc = Uarc * cf
      Varc = Varc * cf

c---
c add back singular contribution
c---

c---  ! desingularization disabled
c
c       ARG2 = TH2-TH0
c       ARG1 = TH1-TH0
c       RES  = 0.
c       If(ABS(ARG2).GT.0.0000001)
c    +  RES = RES + ARG2*(LOG(ABS(ARG2))-1.0)
c       If(ABS(ARG1).GT.0.000001)
c    +  RES = RES - ARG1*(LOG(ABS(ARG1))-1.0)
c       RES  = 2.0*RD*ORNT*RES
c       Uarc = Uarc + RES
c       Varc = Varc + RES
c---------------------------


 100  Format (5(1X,F10.5))

      Return
      End

c===================================================

      subroutine arc_bl
     +
     +     (XC1,YC1,RD1,T11,T21,OR1
     +     ,XC2,YC2,RD2,T12,T22,OR2
     +     ,X0,Y0
     +     ,Uarc,Varc
     +     ,Istop
     +     )

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement.
c----------------------------------------

c---------------------------------------
c Computes the nonsingular contour
c integral over a circular blended arc
c----------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension ZZ(20),WW(20)

      common/ancI3/NGL
      common/ZZWW/ZZ,WW
      common/piii/pi,pih,piq,pi2,pi4,pi6,pi8,srpi

c---
c prepare
c---

      Null  = 0
      Zero  = 0.0

      Istop  = 0

      Uarc = 0.0
      Varc = 0.0

c---------------------------
c Prepare for Gauss-Legendre
c---------------------------

      ST1 = 0.5*(T21+T11)
      DT1 = 0.5*(T21-T11)

      ST2 = 0.5*(T22+T12)
      DT2 = 0.5*(T22-T12)

c---
      Do 1 i = 1,NGL

        AN1  = ST1 + DT1*ZZ(i)
        CS1  = COS(AN1)
        SN1  = SIN(AN1)
        XX1  = XC1 + RD1 * CS1
        YY1  = YC1 + RD1 * SN1
        vnx1 = cs1*OR1
        vny1 = sn1*OR1

        AN2  = ST2 + DT2*ZZ(i)
        CS2  = COS(AN2)
        SN2  = SIN(AN2)
        XX2  = XC2 + RD2 * CS2
        YY2  = YC2 + RD2 * SN2
        vnx2 = cs2*OR2
        vny2 = sn2*OR2

        XX = 0.5*(XX1+XX2)
        YY = 0.5*(YY1+YY2)

        vnx = 0.5*(vnx1+vnx2)
        vny = 0.5*(vny1+vny2)
        vnm = sqrt(vnx**2+vny**2)
        vnx = vnx/vnm
        vny = vny/vnm

        call kernel (x0,y0,xx,yy,RI10,RI11)

        U = - ( (x0-xx)*RI10*vnx + yy*RI11*vny)*yy
        V = RI11*vnx*yy**2

        Uarc = Uarc + U*WW(i)
        Varc = Varc + V*WW(i)

  1   Continue

c---------------------------

      cf = 0.5*( DT1*OR1*RD1+ DT2*OR2*RD2 )

      Uarc = Uarc * cf
      Varc = Varc * cf

 100  Format (5(1X,F10.5))
 110  Format (' CURVATURE =', F13.7)

      Return
      End

c======================================

      subroutine kernel (x0,s0,x,s,RI10,RI11)

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement.
c----------------------------------------

c-----------------------------------------
c Computation of the integrals I10 and I11
c-----------------------------------------

      Implicit Double Precision (a-h,o-z)

      Parameter (tol=0.001)

      Iflag = 0

      If(s0.lt.tol) then  ! shift a point on the axis off the axis
        s0 = tol
        Iflag = 1
      End If

      Dx   = x-x0
      Dxs  = dx**2
      ss0s = (s+s0)**2

      rks = 4.0*s*s0/(Dxs+ss0s)

      call ell_int (rks,F,E)

      RJ10 = F
      RJ11 = ((2.0-rks)*F-2.0*E)/rks

      tmp1 = sqrt(Dxs+ss0s)

      RI10 = 4.0*RJ10/tmp1
      RI11 = 4.0*RJ11/tmp1

      If(Iflag.eq.1) then
c       write (6,100) x0,s0,x,s,RI10,RI11
        s0 = 0.0
      End If

 100  Format (10(1x,f10.5))

      Return
      End
