      subroutine sgrad3_3d
     +
     +   (npts,nelm
     +   ,f
     +   ,dfxxa,dfyxa,dfzxa
     +   ,dfxya,dfyya,dfzya
     +   ,dfxza,dfyza,dfzza
     +   )

c=========================================
c FDLIB, BEMLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c=========================================

c----------------------------------
c Computes the tangential Frechet derivative
c of the vector function f
c at the nodes of a triangular grid
c
c SYMBOLS:
c --------
c
c dfpq:    df_q/dx_p
c dfpqa:   df_q/dx_p averaged over host triangles
c----------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension      p(1026,3)
      Dimension     ne(1026,7)
      Dimension    vna(1026,3)
      Dimension      f(1026,3)
      Dimension itally(1026)

      Dimension dfxxa(1026),dfyxa(1026),dfzxa(1026)
      Dimension dfxya(1026),dfyya(1026),dfzya(1026)
      Dimension dfxza(1026),dfyza(1026),dfzza(1026)

      Dimension n    (512,6),nbe (512,3)
      Dimension alpha(512),  beta(512), gamma(512)

      Dimension dfxx(6),dfyx(6),dfzx(6)
      Dimension dfxy(6),dfyy(6),dfzy(6)
      Dimension dfxz(6),dfyz(6),dfzz(6)

      Dimension xxi(6),eet(6)

c---
c common blocks
c---

      common/points/p,ne
      common/elmnts/n,nbe
      common/albega/alpha,beta,gamma
      common/geo2/vna

c---
c initialize
c---

      Do i=1,npts

        dfxxa(i) = 0.0
        dfyxa(i) = 0.0
        dfzxa(i) = 0.0

        dfxya(i) = 0.0
        dfyya(i) = 0.0
        dfzya(i) = 0.0

        dfxza(i) = 0.0
        dfyza(i) = 0.0
        dfzza(i) = 0.0

        itally(i) = 0

      End Do

c-------------------------------
c compute the Frechet derivative
c of f at the nodes
c of each triangle
c-------------------------------

      Do k=1,nelm     ! run over elements

      i1 = n(k,1)
      i2 = n(k,2)
      i3 = n(k,3)
      i4 = n(k,4)
      i5 = n(k,5)
      i6 = n(k,6)

      al = alpha(k)
      be =  beta(k)
      ga = gamma(k)

c---
c triangle coordinates
c of the nodes
c---

      xxi(1) = 0.0
      eet(1) = 0.0

      xxi(2) = 1.0
      eet(2) = 0.0

      xxi(3) = 0.0
      eet(3) = 1.0

      xxi(4) = al
      eet(4) = 0.0

      xxi(5) = ga
      eet(5) = 1.0-ga

      xxi(6) = 0.0
      eet(6) = be

       Do i=1,6

        m = n(k,i)   ! global index of local node i on element k

        xi  = xxi(i)
        eta = eet(i)

        call sgrad3_3d_interp
     +
     +     (p(i1,1),p(i1,2),p(i1,3)
     +     ,p(i2,1),p(i2,2),p(i2,3)
     +     ,p(i3,1),p(i3,2),p(i3,3)
     +     ,p(i4,1),p(i4,2),p(i4,3)
     +     ,p(i5,1),p(i5,2),p(i5,3)
     +     ,p(i6,1),p(i6,2),p(i6,3)
     +     ,f(i1,1),f(i1,2),f(i1,3)
     +     ,f(i2,1),f(i2,2),f(i2,3)
     +     ,f(i3,1),f(i3,2),f(i3,3)
     +     ,f(i4,1),f(i4,2),f(i4,3)
     +     ,f(i5,1),f(i5,2),f(i5,3)
     +     ,f(i6,1),f(i6,2),f(i6,3)
     +     ,al,be,ga
     +     ,xi,eta
     +     ,vna(m,1),vna(m,2),vna(m,3)
     +     ,dfxx(i),dfyx(i),dfzx(i)
     +     ,dfxy(i),dfyy(i),dfzy(i)
     +     ,dfxz(i),dfyz(i),dfzz(i)
     +     )

        dfxxa(m) = dfxxa(m) + dfxx(i)
        dfyxa(m) = dfyxa(m) + dfyx(i)
        dfzxa(m) = dfzxa(m) + dfzx(i)

        dfxya(m) = dfxya(m) + dfxy(i)
        dfyya(m) = dfyya(m) + dfyy(i)
        dfzya(m) = dfzya(m) + dfzy(i)

        dfxza(m) = dfxza(m) + dfxz(i)
        dfyza(m) = dfyza(m) + dfyz(i)
        dfzza(m) = dfzza(m) + dfzz(i)

        itally(m) = itally(m)+1

       End Do

      End Do                ! loop over nodes

c------------------------------------------
c average the Frechet derivative
c over all elements
c------------------------------------------

      Do i=1,npts

        par = float(itally(i))

        dfxxa(i) = dfxxa(i)/par
        dfyxa(i) = dfyxa(i)/par
        dfzxa(i) = dfzxa(i)/par

        dfxya(i) = dfxya(i)/par
        dfyya(i) = dfyya(i)/par
        dfzya(i) = dfzya(i)/par

        dfxza(i) = dfxza(i)/par
        dfyza(i) = dfyza(i)/par
        dfzza(i) = dfzza(i)/par

      End Do

c---
c done
c---

 100  Format (1x,i3,10(f12.8))
 101  Format (f12.8)

      return
      end
