      subroutine vs_3d_bsi
     +
     +    (npts,nelm
     +    ,mint,NGL
     +    ,Iflow
     +    ,bsi
     +    )

c============================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c============================================

c----------------------------------
c Compute the Biot-Savart integral
c at the nodes of a triangular grid
c using the method developed by
c Pozrikidis (2000)
c----------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension    p(1026,3)
      Dimension   ne(1026,7)
      Dimension  vna(1026,3)
      Dimension zeta(1026,3)

      Dimension bsi(1026,3)

      Dimension n    (512,6),nbe (512,3)
      Dimension alpha(512),  beta(512), gamma(512)
      Dimension arel(512)
      Dimension crvm(512)

      Dimension zz(20),ww(20)
      Dimension xiq(20),etq(20),wq(20)

c---
c common blocks
c---

      common/points/p,ne
      common/elmnts/n,nbe
      common/albega/alpha,beta,gamma
      common/geo1/arel,crvm
      common/geo2/vna
      common/zzetaa/zeta

      common/zwl/zz,ww
      common/trq/xiq,etq,wq

c--------
c  launch
c--------

      Do 1 i=1,npts

c      write (6,*) " Computing the Biot-Savart integral at point",i

       x0 = p(i,1)
       y0 = p(i,2)
       z0 = p(i,3)

c---
c vortex strength and normal vector
c at the evaluation point
c---

       zetx0 = zeta(i,1)
       zety0 = zeta(i,2)
       zetz0 = zeta(i,3)

       vnx0 = vna(i,1)
       vny0 = vna(i,2)
       vnz0 = vna(i,3)

c-------------------------
c compile the integrals
c over the triangles
c 
c will accumulate the velocity
c at uxel,uyel,uzel
c
c will compute the surface area
c for verification
c-------------------------

       srf_area = 0.0

       uxel = 0.0
       uyel = 0.0
       uzel = 0.0

       Do 2 k=1,nelm     ! run over elements

c      write (6,*) " Integrating over element",k

c----------------------------------
c non-singular integration:
c use a regular triangle quadrature
c to compute integrals A and B
c-----------------------------------

        if(    i.ne.n(k,1).and.i.ne.n(k,2)
     +    .and.i.ne.n(k,3).and.i.ne.n(k,4)
     +    .and.i.ne.n(k,5).and.i.ne.n(k,6)
     +    ) then

        call vs_3d_bsi_integrate
     +
     +     (x0,y0,z0
     +     ,vnx0,vny0,vnz0
     +     ,zetx0,zety0,zetz0
     +     ,k
     +     ,mint,Iflow
     +     ,uxel,uyel,uzel
     +     ,arelm
     +     )

        srf_area = srf_area+arelm

        Go to 2

        end if

c-------------------------------------------
c Singular integration:
c
c
c Compute integral B using a quadrature
c
c Computation of integral A as follows:
c
c If the point i is a vertex node,
c integrate over the flat triangle
c defined by the node
c using the polar integration rule
c
c If the point i is a mid node,
c breakup the curved triangle into four
c flat triangles
c and integrate over the flat triangles
c using the polar integration rule
c  
c   Iopt_int = 1 only the position vector
c              2 position vector and rest of variables
c--------------------------------------------

      i1 = n(k,1)
      i2 = n(k,2)
      i3 = n(k,3)
      i4 = n(k,4)
      i5 = n(k,5)
      i6 = n(k,6)

c--------------------------------------------
c singular element with singularity at node 1
c this is a vertex node
c--------------------------------------------

       if(i.eq.n(k,1)) then

c---
c compute integral I2
c---

        call vs_3d_bsi_integrate_flat
     +
     +    (x0,y0,z0
     +    ,vnx0,vny0,vnz0
     +    ,zetx0,zety0,zetz0
     +    ,k
     +    ,mint,Iflow
     +    ,uxel,uyel,uzel
     +    ,crvm_cntr      ! mean curvature
     +    )               ! at element centroid

c---
c compute integrals I1 and I3
c---

          x1 =    p(i1,1)
          y1 =    p(i1,2)
          z1 =    p(i1,3)
        vnx1 =  vna(i1,1)
        vny1 =  vna(i1,2)
        vnz1 =  vna(i1,3)
        ztx1 = zeta(i1,1)
        zty1 = zeta(i1,2)
        ztz1 = zeta(i1,3)

          x2 =    p(i2,1)
          y2 =    p(i2,2)
          z2 =    p(i2,3)
        vnx2 =  vna(i2,1)
        vny2 =  vna(i2,2)
        vnz2 =  vna(i2,3)
        ztx2 = zeta(i2,1)
        zty2 = zeta(i2,2)
        ztz2 = zeta(i2,3)

          x3 =    p(i3,1)
          y3 =    p(i3,2)
          z3 =    p(i3,3)
        vnx3 =  vna(i3,1)
        vny3 =  vna(i3,2)
        vnz3 =  vna(i3,3)
        ztx3 = zeta(i3,1)
        zty3 = zeta(i3,2)
        ztz3 = zeta(i3,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +     (NGL,Iflow
     +     ,zetx0,zety0,zetz0
     +     ,vnx0,vny0,vnz0
     +     ,x1,y1,z1
     +     ,x2,y2,z2
     +     ,x3,y3,z3
     +     ,vnx1,vny1,vnz1
     +     ,vnx2,vny2,vnz2
     +     ,vnx3,vny3,vnz3
     +     ,ztx1,zty1,ztz1
     +     ,ztx2,zty2,ztz2
     +     ,ztx3,zty3,ztz3
     +     ,crvm_cntr
     +     ,uuux,uuuy,uuuz
     +     ,arelm
     +     )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

c--------------------------------------------
c singular element with singularity at node 2
c vertex node
c--------------------------------------------

        else if(i.eq.n(k,2)) then

c---
c compute integral I2
c---

        call vs_3d_bsi_integrate_flat
     +
     +      (x0,y0,z0
     +      ,vnx0,vny0,vnz0
     +      ,zetx0,zety0,zetz0
     +      ,k
     +      ,mint,Iflow
     +      ,uxel,uyel,uzel
     +      ,crvm_cntr
     +      )

c---
c compute integrals I1 and I3
c---

          x1 =    p(i2,1)
          y1 =    p(i2,2)
          z1 =    p(i2,3)
        vnx1 =  vna(i2,1)
        vny1 =  vna(i2,2)
        vnz1 =  vna(i2,3)
        ztx1 = zeta(i2,1)
        zty1 = zeta(i2,2)
        ztz1 = zeta(i2,3)

          x2 =    p(i3,1)
          y2 =    p(i3,2)
          z2 =    p(i3,3)
        vnx2 =  vna(i3,1)
        vny2 =  vna(i3,2)
        vnz2 =  vna(i3,3)
        ztx2 = zeta(i3,1)
        zty2 = zeta(i3,2)
        ztz2 = zeta(i3,3)

          x3 =    p(i1,1)
          y3 =    p(i1,2)
          z3 =    p(i1,3)
        vnx3 =  vna(i1,1)
        vny3 =  vna(i1,2)
        vnz3 =  vna(i1,3)
        ztx3 = zeta(i1,1)
        zty3 = zeta(i1,2)
        ztz3 = zeta(i1,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +        (NGL,Iflow
     +        ,zetx0,zety0,zetz0
     +        ,vnx0,vny0,vnz0
     +        ,x1,y1,z1
     +        ,x2,y2,z2
     +        ,x3,y3,z3
     +        ,vnx1,vny1,vnz1
     +        ,vnx2,vny2,vnz2
     +        ,vnx3,vny3,vnz3
     +        ,ztx1,zty1,ztz1
     +        ,ztx2,zty2,ztz2
     +        ,ztx3,zty3,ztz3
     +        ,crvm_cntr
     +        ,uuux,uuuy,uuuz
     +        ,arelm
     +        )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

c--------------------------------------------
c singular element with singularity at node 3
c vertex node
c--------------------------------------------

        else if(i.eq.n(k,3)) then

c---
c compute integral I2
c---

        call vs_3d_bsi_integrate_flat
     +
     +         (x0,y0,z0
     +         ,vnx0,vny0,vnz0
     +         ,zetx0,zety0,zetz0
     +         ,k
     +         ,mint,Iflow
     +         ,uxel,uyel,uzel
     +         ,crvm_cntr
     +         )

c---
c compute integrals I1 and I3
c---

          x1 =    p(i3,1)
          y1 =    p(i3,2)
          z1 =    p(i3,3)
        vnx1 =  vna(i3,1)
        vny1 =  vna(i3,2)
        vnz1 =  vna(i3,3)
        ztx1 = zeta(i3,1)
        zty1 = zeta(i3,2)
        ztz1 = zeta(i3,3)

          x2 =    p(i1,1)
          y2 =    p(i1,2)
          z2 =    p(i1,3)
        vnx2 =  vna(i1,1)
        vny2 =  vna(i1,2)
        vnz2 =  vna(i1,3)
        ztx2 = zeta(i1,1)
        zty2 = zeta(i1,2)
        ztz2 = zeta(i1,3)

          x3 =    p(i2,1)
          y3 =    p(i2,2)
          z3 =    p(i2,3)
        vnx3 =  vna(i2,1)
        vny3 =  vna(i2,2)
        vnz3 =  vna(i2,3)
        ztx3 = zeta(i2,1)
        zty3 = zeta(i2,2)
        ztz3 = zeta(i2,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +      (NGL,Iflow
     +      ,zetx0,zety0,zetz0
     +      ,vnx0,vny0,vnz0
     +      ,x1,y1,z1
     +      ,x2,y2,z2
     +      ,x3,y3,z3
     +      ,vnx1,vny1,vnz1
     +      ,vnx2,vny2,vnz2
     +      ,vnx3,vny3,vnz3
     +      ,ztx1,zty1,ztz1
     +      ,ztx2,zty2,ztz2
     +      ,ztx3,zty3,ztz3
     +      ,crvm_cntr
     +      ,uuux,uuuy,uuuz
     +      ,arelm
     +      )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

c--------------------------------------------
c singular element with singularity at node 4
c mid-node
c Will integrate over 4 flat triangles
c--------------------------------------------

        else if(i.eq.n(k,4)) then

c---
c compute integral I2
c---

        call vs_3d_bsi_integrate_flat
     +
     +       (x0,y0,z0
     +       ,vnx0,vny0,vnz0
     +       ,zetx0,zety0,zetz0
     +       ,k
     +       ,mint,Iflow
     +       ,uxel,uyel,uzel
     +       ,crvm_cntr
     +       )

c---
c compute integrals I1 and I3
c over four descendant triangles
c---

          x1 =    p(i4,1)
          y1 =    p(i4,2)
          z1 =    p(i4,3)
        vnx1 =  vna(i4,1)
        vny1 =  vna(i4,2)
        vnz1 =  vna(i4,3)
        ztx1 = zeta(i4,1)
        zty1 = zeta(i4,2)
        ztz1 = zeta(i4,3)

          x2 =    p(i6,1)
          y2 =    p(i6,2)
          z2 =    p(i6,3)
        vnx2 =  vna(i6,1)
        vny2 =  vna(i6,2)
        vnz2 =  vna(i6,3)
        ztx2 = zeta(i6,1)
        zty2 = zeta(i6,2)
        ztz2 = zeta(i6,3)

          x3 =    p(i1,1)
          y3 =    p(i1,2)
          z3 =    p(i1,3)
        vnx3 =  vna(i1,1)
        vny3 =  vna(i1,2)
        vnz3 =  vna(i1,3)
        ztx3 = zeta(i1,1)
        zty3 = zeta(i1,2)
        ztz3 = zeta(i1,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +         (NGL,Iflow
     +         ,zetx0,zety0,zetz0
     +         ,vnx0,vny0,vnz0
     +         ,x1,y1,z1
     +         ,x2,y2,z2
     +         ,x3,y3,z3
     +         ,vnx1,vny1,vnz1
     +         ,vnx2,vny2,vnz2
     +         ,vnx3,vny3,vnz3
     +         ,ztx1,zty1,ztz1
     +         ,ztx2,zty2,ztz2
     +         ,ztx3,zty3,ztz3
     +         ,crvm_cntr
     +         ,uuux,uuuy,uuuz
     +         ,arelm
     +         )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

          x1 =    p(i4,1)
          y1 =    p(i4,2)
          z1 =    p(i4,3)
        vnx1 =  vna(i4,1)
        vny1 =  vna(i4,2)
        vnz1 =  vna(i4,3)
        ztx1 = zeta(i4,1)
        zty1 = zeta(i4,2)
        ztz1 = zeta(i4,3)

          x2 =    p(i3,1)
          y2 =    p(i3,2)
          z2 =    p(i3,3)
        vnx2 =  vna(i3,1)
        vny2 =  vna(i3,2)
        vnz2 =  vna(i3,3)
        ztx2 = zeta(i3,1)
        zty2 = zeta(i3,2)
        ztz2 = zeta(i3,3)

          x3 =    p(i6,1)
          y3 =    p(i6,2)
          z3 =    p(i6,3)
        vnx3 =  vna(i6,1)
        vny3 =  vna(i6,2)
        vnz3 =  vna(i6,3)
        ztx3 = zeta(i6,1)
        zty3 = zeta(i6,2)
        ztz3 = zeta(i6,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +      (NGL,Iflow
     +      ,zetx0,zety0,zetz0
     +      ,vnx0,vny0,vnz0
     +      ,x1,y1,z1
     +      ,x2,y2,z2
     +      ,x3,y3,z3
     +      ,vnx1,vny1,vnz1
     +      ,vnx2,vny2,vnz2
     +      ,vnx3,vny3,vnz3
     +      ,ztx1,zty1,ztz1
     +      ,ztx2,zty2,ztz2
     +      ,ztx3,zty3,ztz3
     +      ,crvm_cntr
     +      ,uuux,uuuy,uuuz
     +      ,arelm
     +      )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

          x1 =    p(i4,1)
          y1 =    p(i4,2)
          z1 =    p(i4,3)
        vnx1 =  vna(i4,1)
        vny1 =  vna(i4,2)
        vnz1 =  vna(i4,3)
        ztx1 = zeta(i4,1)
        zty1 = zeta(i4,2)
        ztz1 = zeta(i4,3)

          x2 =    p(i5,1)
          y2 =    p(i5,2)
          z2 =    p(i5,3)
        vnx2 =  vna(i5,1)
        vny2 =  vna(i5,2)
        vnz2 =  vna(i5,3)
        ztx2 = zeta(i5,1)
        zty2 = zeta(i5,2)
        ztz2 = zeta(i5,3)

          x3 =    p(i3,1)
          y3 =    p(i3,2)
          z3 =    p(i3,3)
        vnx3 =  vna(i3,1)
        vny3 =  vna(i3,2)
        vnz3 =  vna(i3,3)
        ztx3 = zeta(i3,1)
        zty3 = zeta(i3,2)
        ztz3 = zeta(i3,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +      (NGL,Iflow
     +      ,zetx0,zety0,zetz0
     +      ,vnx0,vny0,vnz0
     +      ,x1,y1,z1
     +      ,x2,y2,z2
     +      ,x3,y3,z3
     +      ,vnx1,vny1,vnz1
     +      ,vnx2,vny2,vnz2
     +      ,vnx3,vny3,vnz3
     +      ,ztx1,zty1,ztz1
     +      ,ztx2,zty2,ztz2
     +      ,ztx3,zty3,ztz3
     +      ,crvm_cntr
     +      ,uuux,uuuy,uuuz
     +      ,arelm
     +      )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

          x1 =    p(i4,1)
          y1 =    p(i4,2)
          z1 =    p(i4,3)
        vnx1 =  vna(i4,1)
        vny1 =  vna(i4,2)
        vnz1 =  vna(i4,3)
        ztx1 = zeta(i4,1)
        zty1 = zeta(i4,2)
        ztz1 = zeta(i4,3)

          x2 =    p(i2,1)
          y2 =    p(i2,2)
          z2 =    p(i2,3)
        vnx2 =  vna(i2,1)
        vny2 =  vna(i2,2)
        vnz2 =  vna(i2,3)
        ztx2 = zeta(i2,1)
        zty2 = zeta(i2,2)
        ztz2 = zeta(i2,3)

          x3 =    p(i5,1)
          y3 =    p(i5,2)
          z3 =    p(i5,3)
        vnx3 =  vna(i5,1)
        vny3 =  vna(i5,2)
        vnz3 =  vna(i5,3)
        ztx3 = zeta(i5,1)
        zty3 = zeta(i5,2)
        ztz3 = zeta(i5,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +        (NGL,Iflow
     +        ,zetx0,zety0,zetz0
     +        ,vnx0,vny0,vnz0
     +        ,x1,y1,z1
     +        ,x2,y2,z2
     +        ,x3,y3,z3
     +        ,vnx1,vny1,vnz1
     +        ,vnx2,vny2,vnz2
     +        ,vnx3,vny3,vnz3
     +        ,ztx1,zty1,ztz1
     +        ,ztx2,zty2,ztz2
     +        ,ztx3,zty3,ztz3
     +        ,crvm_cntr
     +        ,uuux,uuuy,uuuz
     +        ,arelm
     +        )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

c--------------------------------------------
c singular element with singularity at node 5
c mid-node
c
c Will integrate over 4 flat triangles
c--------------------------------------------

        else if(i.eq.n(k,5)) then

c---
c compute integral I2
c---

        call vs_3d_bsi_integrate_flat
     +
     +       (x0,y0,z0
     +       ,vnx0,vny0,vnz0
     +       ,zetx0,zety0,zetz0
     +       ,k
     +       ,mint,Iflow
     +       ,uxel,uyel,uzel
     +       ,crvm_cntr
     +       )

c---
c compute integrals I1 and I3
c over four descendant triangles
c---

          x1 =    p(i5,1)
          y1 =    p(i5,2)
          z1 =    p(i5,3)
        vnx1 =  vna(i5,1)
        vny1 =  vna(i5,2)
        vnz1 =  vna(i5,3)
        ztx1 = zeta(i5,1)
        zty1 = zeta(i5,2)
        ztz1 = zeta(i5,3)

          x2 =    p(i4,1)
          y2 =    p(i4,2)
          z2 =    p(i4,3)
        vnx2 =  vna(i4,1)
        vny2 =  vna(i4,2)
        vnz2 =  vna(i4,3)
        ztx2 = zeta(i4,1)
        zty2 = zeta(i4,2)
        ztz2 = zeta(i4,3)

          x3 =    p(i2,1)
          y3 =    p(i2,2)
          z3 =    p(i2,3)
        vnx3 =  vna(i2,1)
        vny3 =  vna(i2,2)
        vnz3 =  vna(i2,3)
        ztx3 = zeta(i2,1)
        zty3 = zeta(i2,2)
        ztz3 = zeta(i2,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +       (NGL,Iflow
     +       ,zetx0,zety0,zetz0
     +       ,vnx0,vny0,vnz0
     +       ,x1,y1,z1
     +       ,x2,y2,z2
     +       ,x3,y3,z3
     +       ,vnx1,vny1,vnz1
     +       ,vnx2,vny2,vnz2
     +       ,vnx3,vny3,vnz3
     +       ,ztx1,zty1,ztz1
     +       ,ztx2,zty2,ztz2
     +       ,ztx3,zty3,ztz3
     +       ,crvm_cntr
     +       ,uuux,uuuy,uuuz
     +       ,arelm
     +       )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

          x1 =    p(i5,1)
          y1 =    p(i5,2)
          z1 =    p(i5,3)
        vnx1 =  vna(i5,1)
        vny1 =  vna(i5,2)
        vnz1 =  vna(i5,3)
        ztx1 = zeta(i5,1)
        zty1 = zeta(i5,2)
        ztz1 = zeta(i5,3)

          x2 =    p(i1,1)
          y2 =    p(i1,2)
          z2 =    p(i1,3)
        vnx2 =  vna(i1,1)
        vny2 =  vna(i1,2)
        vnz2 =  vna(i1,3)
        ztx2 = zeta(i1,1)
        zty2 = zeta(i1,2)
        ztz2 = zeta(i1,3)

          x3 =    p(i4,1)
          y3 =    p(i4,2)
          z3 =    p(i4,3)
        vnx3 =  vna(i4,1)
        vny3 =  vna(i4,2)
        vnz3 =  vna(i4,3)
        ztx3 = zeta(i4,1)
        zty3 = zeta(i4,2)
        ztz3 = zeta(i4,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +       (NGL,Iflow
     +       ,zetx0,zety0,zetz0
     +       ,vnx0,vny0,vnz0
     +       ,x1,y1,z1
     +       ,x2,y2,z2
     +       ,x3,y3,z3
     +       ,vnx1,vny1,vnz1
     +       ,vnx2,vny2,vnz2
     +       ,vnx3,vny3,vnz3
     +       ,ztx1,zty1,ztz1
     +       ,ztx2,zty2,ztz2
     +       ,ztx3,zty3,ztz3
     +       ,crvm_cntr
     +       ,uuux,uuuy,uuuz
     +       ,arelm
     +       )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

          x1 =    p(i5,1)
          y1 =    p(i5,2)
          z1 =    p(i5,3)
        vnx1 =  vna(i5,1)
        vny1 =  vna(i5,2)
        vnz1 =  vna(i5,3)
        ztx1 = zeta(i5,1)
        zty1 = zeta(i5,2)
        ztz1 = zeta(i5,3)

          x2 =    p(i6,1)
          y2 =    p(i6,2)
          z2 =    p(i6,3)
        vnx2 =  vna(i6,1)
        vny2 =  vna(i6,2)
        vnz2 =  vna(i6,3)
        ztx2 = zeta(i6,1)
        zty2 = zeta(i6,2)
        ztz2 = zeta(i6,3)

          x3 =    p(i1,1)
          y3 =    p(i1,2)
          z3 =    p(i1,3)
        vnx3 =  vna(i1,1)
        vny3 =  vna(i1,2)
        vnz3 =  vna(i1,3)
        ztx3 = zeta(i1,1)
        zty3 = zeta(i1,2)
        ztz3 = zeta(i1,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +      (NGL,Iflow
     +      ,zetx0,zety0,zetz0
     +      ,vnx0,vny0,vnz0
     +      ,x1,y1,z1
     +      ,x2,y2,z2
     +      ,x3,y3,z3
     +      ,vnx1,vny1,vnz1
     +      ,vnx2,vny2,vnz2
     +      ,vnx3,vny3,vnz3
     +      ,ztx1,zty1,ztz1
     +      ,ztx2,zty2,ztz2
     +      ,ztx3,zty3,ztz3
     +      ,crvm_cntr
     +      ,uuux,uuuy,uuuz
     +      ,arelm
     +      )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

          x1 =    p(i5,1)
          y1 =    p(i5,2)
          z1 =    p(i5,3)
        vnx1 =  vna(i5,1)
        vny1 =  vna(i5,2)
        vnz1 =  vna(i5,3)
        ztx1 = zeta(i5,1)
        zty1 = zeta(i5,2)
        ztz1 = zeta(i5,3)

          x2 =    p(i3,1)
          y2 =    p(i3,2)
          z2 =    p(i3,3)
        vnx2 =  vna(i3,1)
        vny2 =  vna(i3,2)
        vnz2 =  vna(i3,3)
        ztx2 = zeta(i3,1)
        zty2 = zeta(i3,2)
        ztz2 = zeta(i3,3)

          x3 =    p(i6,1)
          y3 =    p(i6,2)
          z3 =    p(i6,3)
        vnx3 =  vna(i6,1)
        vny3 =  vna(i6,2)
        vnz3 =  vna(i6,3)
        ztx3 = zeta(i6,1)
        zty3 = zeta(i6,2)
        ztz3 = zeta(i6,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +     (NGL,Iflow
     +     ,zetx0,zety0,zetz0
     +     ,vnx0,vny0,vnz0
     +     ,x1,y1,z1
     +     ,x2,y2,z2
     +     ,x3,y3,z3
     +     ,vnx1,vny1,vnz1
     +     ,vnx2,vny2,vnz2
     +     ,vnx3,vny3,vnz3
     +     ,ztx1,zty1,ztz1
     +     ,ztx2,zty2,ztz2
     +     ,ztx3,zty3,ztz3
     +     ,crvm_cntr
     +     ,uuux,uuuy,uuuz
     +     ,arelm
     +     )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

c--------------------------------------------
c singular element with singularity at node 6
c edge-node
c
c Will integrate over 4 flat triangles
c--------------------------------------------

        else if(i.eq.n(k,6)) then

c---
c compute integral I2
c---

        call vs_3d_bsi_integrate_flat
     +
     +       (x0,y0,z0
     +       ,vnx0,vny0,vnz0
     +       ,zetx0,zety0,zetz0
     +       ,k
     +       ,mint,Iflow
     +       ,uxel,uyel,uzel
     +       ,crvm_cntr
     +       )

c---
c compute integrals I1 and I3
c over four descendant triangles
c---

          x1 =    p(i6,1)
          y1 =    p(i6,2)
          z1 =    p(i6,3)
        vnx1 =  vna(i6,1)
        vny1 =  vna(i6,2)
        vnz1 =  vna(i6,3)
        ztx1 = zeta(i6,1)
        zty1 = zeta(i6,2)
        ztz1 = zeta(i6,3)

          x2 =    p(i1,1)
          y2 =    p(i1,2)
          z2 =    p(i1,3)
        vnx2 =  vna(i1,1)
        vny2 =  vna(i1,2)
        vnz2 =  vna(i1,3)
        ztx2 = zeta(i1,1)
        zty2 = zeta(i1,2)
        ztz2 = zeta(i1,3)

          x3 =    p(i4,1)
          y3 =    p(i4,2)
          z3 =    p(i4,3)
        vnx3 =  vna(i4,1)
        vny3 =  vna(i4,2)
        vnz3 =  vna(i4,3)
        ztx3 = zeta(i4,1)
        zty3 = zeta(i4,2)
        ztz3 = zeta(i4,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +      (NGL,Iflow
     +      ,zetx0,zety0,zetz0
     +      ,vnx0,vny0,vnz0
     +      ,x1,y1,z1
     +      ,x2,y2,z2
     +      ,x3,y3,z3
     +      ,vnx1,vny1,vnz1
     +      ,vnx2,vny2,vnz2
     +      ,vnx3,vny3,vnz3
     +      ,ztx1,zty1,ztz1
     +      ,ztx2,zty2,ztz2
     +      ,ztx3,zty3,ztz3
     +      ,crvm_cntr
     +      ,uuux,uuuy,uuuz
     +      ,arelm
     +      )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

          x1 =    p(i6,1)
          y1 =    p(i6,2)
          z1 =    p(i6,3)
        vnx1 =  vna(i6,1)
        vny1 =  vna(i6,2)
        vnz1 =  vna(i6,3)
        ztx1 = zeta(i6,1)
        zty1 = zeta(i6,2)
        ztz1 = zeta(i6,3)

          x2 =    p(i4,1)
          y2 =    p(i4,2)
          z2 =    p(i4,3)
        vnx2 =  vna(i4,1)
        vny2 =  vna(i4,2)
        vnz2 =  vna(i4,3)
        ztx2 = zeta(i4,1)
        zty2 = zeta(i4,2)
        ztz2 = zeta(i4,3)

          x3 =    p(i2,1)
          y3 =    p(i2,2)
          z3 =    p(i2,3)
        vnx3 =  vna(i2,1)
        vny3 =  vna(i2,2)
        vnz3 =  vna(i2,3)
        ztx3 = zeta(i2,1)
        zty3 = zeta(i2,2)
        ztz3 = zeta(i2,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +       (NGL,Iflow
     +       ,zetx0,zety0,zetz0
     +       ,vnx0,vny0,vnz0
     +       ,x1,y1,z1
     +       ,x2,y2,z2
     +       ,x3,y3,z3
     +       ,vnx1,vny1,vnz1
     +       ,vnx2,vny2,vnz2
     +       ,vnx3,vny3,vnz3
     +       ,ztx1,zty1,ztz1
     +       ,ztx2,zty2,ztz2
     +       ,ztx3,zty3,ztz3
     +       ,crvm_cntr
     +       ,uuux,uuuy,uuuz
     +       ,arelm
     +       )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

          x1 =    p(i6,1)
          y1 =    p(i6,2)
          z1 =    p(i6,3)
        vnx1 =  vna(i6,1)
        vny1 =  vna(i6,2)
        vnz1 =  vna(i6,3)
        ztx1 = zeta(i6,1)
        zty1 = zeta(i6,2)
        ztz1 = zeta(i6,3)

          x2 =    p(i2,1)
          y2 =    p(i2,2)
          z2 =    p(i2,3)
        vnx2 =  vna(i2,1)
        vny2 =  vna(i2,2)
        vnz2 =  vna(i2,3)
        ztx2 = zeta(i2,1)
        zty2 = zeta(i2,2)
        ztz2 = zeta(i2,3)

          x3 =    p(i5,1)
          y3 =    p(i5,2)
          z3 =    p(i5,3)
        vnx3 =  vna(i5,1)
        vny3 =  vna(i5,2)
        vnz3 =  vna(i5,3)
        ztx3 = zeta(i5,1)
        zty3 = zeta(i5,2)
        ztz3 = zeta(i5,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +       (NGL,Iflow
     +       ,zetx0,zety0,zetz0
     +       ,vnx0,vny0,vnz0
     +       ,x1,y1,z1
     +       ,x2,y2,z2
     +       ,x3,y3,z3
     +       ,vnx1,vny1,vnz1
     +       ,vnx2,vny2,vnz2
     +       ,vnx3,vny3,vnz3
     +       ,ztx1,zty1,ztz1
     +       ,ztx2,zty2,ztz2
     +       ,ztx3,zty3,ztz3
     +       ,crvm_cntr
     +       ,uuux,uuuy,uuuz
     +       ,arelm
     +       )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

          x1 =    p(i6,1)
          y1 =    p(i6,2)
          z1 =    p(i6,3)
        vnx1 =  vna(i6,1)
        vny1 =  vna(i6,2)
        vnz1 =  vna(i6,3)
        ztx1 = zeta(i6,1)
        zty1 = zeta(i6,2)
        ztz1 = zeta(i6,3)

          x2 =    p(i5,1)
          y2 =    p(i5,2)
          z2 =    p(i5,3)
        vnx2 =  vna(i5,1)
        vny2 =  vna(i5,2)
        vnz2 =  vna(i5,3)
        ztx2 = zeta(i5,1)
        zty2 = zeta(i5,2)
        ztz2 = zeta(i5,3)

          x3 =    p(i3,1)
          y3 =    p(i3,2)
          z3 =    p(i3,3)
        vnx3 =  vna(i3,1)
        vny3 =  vna(i3,2)
        vnz3 =  vna(i3,3)
        ztx3 = zeta(i3,1)
        zty3 = zeta(i3,2)
        ztz3 = zeta(i3,3)

         call vs_3d_bsi_integrate_flat_sing
     +
     +      (NGL,Iflow
     +      ,zetx0,zety0,zetz0
     +      ,vnx0,vny0,vnz0
     +      ,x1,y1,z1
     +      ,x2,y2,z2
     +      ,x3,y3,z3
     +      ,vnx1,vny1,vnz1
     +      ,vnx2,vny2,vnz2
     +      ,vnx3,vny3,vnz3
     +      ,ztx1,zty1,ztz1
     +      ,ztx2,zty2,ztz2
     +      ,ztx3,zty3,ztz3
     +      ,crvm_cntr
     +      ,uuux,uuuy,uuuz
     +      ,arelm
     +      )

        uxel = uxel + uuux
        uyel = uyel + uuuy
        uzel = uzel + uuuz

        srf_area = srf_area+arelm

c------------
       end if    ! done integrating over a singular element
c------------

  2   Continue

       bsi(i,1) = uxel
       bsi(i,2) = uyel
       bsi(i,3) = uzel

       bsi(i,1) = bsi(i,1) - 0.5D0*(zety0*vnz0 - zetz0*vny0)
       bsi(i,2) = bsi(i,2) - 0.5D0*(zetz0*vnx0 - zetx0*vnz0)
       bsi(i,3) = bsi(i,3) - 0.5D0*(zetx0*vny0 - zety0*vnx0)

c----------------------------------------------
c       write (6,*) " total surface area: ",srf_area
c       write (6,*)
c
c       write (6,*) zeta(i,1),zeta(i,2),zeta(i,3)
c       write (6,*) zetx0,zety0,zetz0
c       write (6,*)
c       write (6,*) p(i,1),p(i,2),p(i,3)
c       write (6,*)
c       write (6,*) bsi(i,1),bsi(i,2),bsi(i,3)
c       write (6,*) testx,testy,testz
c       stop
c----------------------------------------------

  1   Continue               ! loop over evaluation nodes

c-----
c done
c-----


 100  Format (1x,i3,10(f12.8))
 101  Format (f12.8)

      return
      end
