      subroutine vs_3d_bsi_integrate_flat
     +
     +    (x0,y0,z0
     +    ,vnx0,vny0,vnz0
     +    ,zetx0,zety0,zetz0
     +    ,k
     +    ,mint,Iflow
     +    ,uxel,uyel,uzel
     +    ,crvm
     +    )

c======================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c======================================

c--------------------------------------
c Compute the modified Biot-Savart integral I2
c over a non-singular triangle labeled k
c--------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension    p(1026,3)
      Dimension   ne(1026,7)
      Dimension  vna(1026,3)
      Dimension zeta(1026,3)

      Dimension     n(512,6),nbe(512,3)
      Dimension alpha(512),beta(512),gamma(512)

      Dimension xiq(20),etq(20),wq(20)

c---
c common blocks
c---

      common/points/p,ne
      common/elmnts/n,nbe
      common/albega/alpha,beta,gamma
      common/geo2/vna
      common/zzetaa/zeta

      common/trq/xiq,etq,wq

      common/pii/pi,piq,pih,pi2,pi4,pi6,pi8

c---
c initialize
c---

      Iopt_int = 2   ! need the surface metric
      Iopt_lgf = 2   ! need G and gradient

      area = 0.0

c---
c triangle nodes
c---

      i1 = n(k,1)
      i2 = n(k,2)
      i3 = n(k,3)
      i4 = n(k,4)
      i5 = n(k,5)
      i6 = n(k,6)

      al = alpha(k)
      be = beta (k)
      ga = gamma(k)

c---
c loop over integration points
c---

      Do i=1,mint

        xi  = xiq(i)
        eta = etq(i)

        call vs_3d_bsi_interp
     +
     +     (p(i1,1),p(i1,2),p(i1,3)
     +     ,p(i2,1),p(i2,2),p(i2,3)
     +     ,p(i3,1),p(i3,2),p(i3,3)
     +     ,p(i4,1),p(i4,2),p(i4,3)
     +     ,p(i5,1),p(i5,2),p(i5,3)
     +     ,p(i6,1),p(i6,2),p(i6,3)
     +
     +     ,vna(i1,1),vna(i1,2),vna(i1,3)
     +     ,vna(i2,1),vna(i2,2),vna(i2,3)
     +     ,vna(i3,1),vna(i3,2),vna(i3,3)
     +     ,vna(i4,1),vna(i4,2),vna(i4,3)
     +     ,vna(i5,1),vna(i5,2),vna(i5,3)
     +     ,vna(i6,1),vna(i6,2),vna(i6,3)
     +
     +     ,zeta(i1,1),zeta(i1,2),zeta(i1,3)
     +     ,zeta(i2,1),zeta(i2,2),zeta(i2,3)
     +     ,zeta(i3,1),zeta(i3,2),zeta(i3,3)
     +     ,zeta(i4,1),zeta(i4,2),zeta(i4,3)
     +     ,zeta(i5,1),zeta(i5,2),zeta(i5,3)
     +     ,zeta(i6,1),zeta(i6,2),zeta(i6,3)
     +
     +     ,al,be,ga
     +     ,xi,eta
     +     ,x,y,z
     +     ,vnax,vnay,vnaz
     +     ,vnx,vny,vnz
     +     ,crvm,hs
     +     ,zetx,zety,zetz
     +     ,Iopt_int
     +     )

c---
c compute the Green's function
c---

       call lgf_3d_fs 
     +
     +    (Iopt_lgf
     +    ,x,y,z
     +    ,x0,y0,z0
     +    ,G
     +    ,Gx,Gy,Gz
     +    )

c-----------
c integral I2
c-----------

      prj = vnx*Gx + vny*Gy + vnz*Gz

      vvx = vnax-vnx0
      vvy = vnay-vny0
      vvz = vnaz-vnz0

      rintx = (zety0*vvz - zetz0*vvy)*prj
      rinty = (zetz0*vvx - zetx0*vvz)*prj
      rintz = (zetx0*vvy - zety0*vvx)*prj

c---
c finish up the quadrature
c---

      cf = 0.5D0*hs*wq(i)

      area = area+cf

      uxel = uxel + rintx*cf 
      uyel = uyel + rinty*cf 
      uzel = uzel + rintz*cf 

      End Do

c---
c End of loop over integration points
c---

c---
c compute the mean curvature at the element centroid
c---

        xi  = 1.0D0/3.0D0
        eta = 1.0D0/3.0D0

        call vs_3d_bsi_interp
     +
     +     (p(i1,1),p(i1,2),p(i1,3)
     +     ,p(i2,1),p(i2,2),p(i2,3)
     +     ,p(i3,1),p(i3,2),p(i3,3)
     +     ,p(i4,1),p(i4,2),p(i4,3)
     +     ,p(i5,1),p(i5,2),p(i5,3)
     +     ,p(i6,1),p(i6,2),p(i6,3)
     +
     +     ,vna(i1,1),vna(i1,2),vna(i1,3)
     +     ,vna(i2,1),vna(i2,2),vna(i2,3)
     +     ,vna(i3,1),vna(i3,2),vna(i3,3)
     +     ,vna(i4,1),vna(i4,2),vna(i4,3)
     +     ,vna(i5,1),vna(i5,2),vna(i5,3)
     +     ,vna(i6,1),vna(i6,2),vna(i6,3)
     +
     +     ,zeta(i1,1),zeta(i1,2),zeta(i1,3)
     +     ,zeta(i2,1),zeta(i2,2),zeta(i2,3)
     +     ,zeta(i3,1),zeta(i3,2),zeta(i3,3)
     +     ,zeta(i4,1),zeta(i4,2),zeta(i4,3)
     +     ,zeta(i5,1),zeta(i5,2),zeta(i5,3)
     +     ,zeta(i6,1),zeta(i6,2),zeta(i6,3)
     +
     +     ,al,be,ga
     +     ,xi,eta
     +     ,x,y,z
     +     ,vnax,vnay,vnaz
     +     ,vnx,vny,vnz
     +     ,crvm,hs
     +     ,zetx,zety,zetz
     +     ,Iopt_int
     +     )

c     write (6,*) "mean curvature at element centroid",crvm

c---
c end
c---

      return
      end
