clear all
close all

%========
% Solve the Blasius boundary-layer equation
% for flow over a semi-infinite flat plate
%=======

Nstep = 64;
etamax = 10.0;

deta = etamax/Nstep;

eta = 0.0;

%---
% initial conditions
%---

x(1)= 0.0; x(2) = 0.0; x(3) = 0.332;

xplot(1) = eta;

yplot1(1) = x(1);
yplot2(1) = x(2);
yplot3(1) = x(3);

%---
% integrate
%---

for i=2:Nstep+1

   xp = blasius_ode(x);

   for j=1:3
    xsave(j) = x(j);
    xpsave(j) = xp(j);
    x(j) = x(j) + xp(j)*deta;
   end

   eta = eta + deta;
   xp = blasius_ode(x);

   for j=1:3
     x(j) = xsave(j) + 0.5*(xpsave(j)+xp(j))*deta;
   end

   xplot(i)  = eta;
   yplot1(i) = x(1);
   yplot2(i) = x(2);
   yplot3(i) = x(3);
end

%---
% plot the solution
%---

figure(1)
hold on
plot(yplot1,xplot,'k--')
plot(yplot2,xplot,'k')
plot(yplot3,xplot,'k:')
ylabel('\eta','fontsize',15)
set(gca,'fontsize',15)
axis([0 1.5 0 6])
box
