clear all
close all

%===
% Solve the boundary-value problem
% for the sakiadis equation
%
% will solve the boundary-value problem
% with newton iterations
%
% will solve an extended system of six ODEs 
%===

tspan = [0, 40];

options = odeset('RelTol',1e-8,'AbsTol',1e-08);

%---
% initial guess
%----

 y0 = [0.0 1.0 -0.44375 0.0 0.0 1.0];

%---
% iterate
%---

 for iter=1:10

  [t, y]= ode23(@blas_fnc,tspan,y0,options);

  NN = size(y);
  N  = NN(1);

  residual = y(N,2)-0.0;

  y0 = [0.0; ...
        1.0; ...
        y0(3)-residual/y(N,5);...
        0.0; ...
        0.0; ...
        1.0];

  if(abs(residual)<0.00000001) break; end

 end

%---
% converged value
%---

format long
y0(3)
format short

%---
% plot the solution
%---

figure(1)
plot(t,y(:,2))
xlabel('\eta','fontsize',15)
ylabel('f','fontsize',15)
set(gca,'fontsize',15)
box
