close all
clear all

%===
% Solution of the boundary-value problem
% for the Falkner-Skan equation
%
% will solve the boundary-value problem
% with newton iterations
%
% will solve an extended system of six ODEs 
%===

global m

figure(1)
hold on


%---
% initial guess
%----

 options = odeset('RelTol',1e-8,'AbsTol',1e-08);

 y0 = [0.0 0.0 0.332 0.0 0.0 1.0];

%---
for repeat=-2:12
%---

if(repeat==0)
 m = -0.05; tspan = [0, 10.0];
 y0 = [0.0 0.0 0.332 0.0 0.0 1.0];
elseif(repeat==-1)
 m = -0.08; tspan = [0, 10.0];
elseif(repeat==-2)
 m = -0.0904; tspan = [0, 10.0];
elseif(repeat==1)
 m = 0.0; tspan = [0, 10.0];
 y0 = [0.0 0.0 0.332 0.0 0.0 1.0];
elseif(repeat==2)
 m = 0.1; tspan = [0, 9.0];
elseif(repeat==3)
 m = 0.2; tspan = [0, 9.0];
elseif(repeat==4)
 m = 0.3; tspan = [0, 9.0];
elseif(repeat==5)
 m = 0.4; tspan = [0, 8.0];
elseif(repeat==6)
 m = 0.6; tspan = [0, 7.0];
elseif(repeat==7)
 m = 0.8; tspan = [0, 6.0];
elseif(repeat==8)
 m = 1.0; tspan = [0, 5.0];
elseif(repeat==9)
 m = 1.2; tspan = [0, 4.0];
elseif(repeat==10)
 m = 1.4; tspan = [0, 3.5];
elseif(repeat==11)
 m = 1.6; tspan = [0, 3.0];
elseif(repeat==12)
 m = 1.8; tspan = [0, 2.5];
end

%---
% iterate
%---

 for iter=1:10

  [t, y]= ode23(@fskan_fnc,tspan,y0,options);
  NN = size(y);
  N  = NN(1);
  residual = y(N,2)-1.0;
  y0 = [0.0; ...
        0.0; ...
        y0(3)-residual/y(N,5);...
        0.0; ...
        0.0; ...
        1.0];

  if(abs(residual)<0.00000001) break; end

 end

%---
% converged solution
%---

vel = y(:,2);
nsize = size(vel);
n = nsize(1);

%---
% compute the coefficient alpha
% for the displacement thickness
%---

alpha1 = t(n)-y(n,1);
alpha2 = t(n-30)-y(n-30,1);

%---
% compute the coefficient beta
% for the momentum thickness
% by the trapezoidal rule
%---

beta = 0.0;

for i=1:n-1
 v1 = vel(i)  *(1.0-vel(i));
 v2 = vel(i+1)*(1.0-vel(i+1));
 beta = beta + (t(i+1)-t(i))*0.5*(v1+v2);
end

%---
% now record
%---

format long
y0(3)
mplot(repeat+3) = m;
yplot(repeat+3) = y0(3);
a1plt(repeat+3) = alpha1;
a2plt(repeat+3) = alpha2;
b1plt(repeat+3) = beta;
lambd(repeat+3) = m*beta^2;
format short


figure(1)

if(abs(m-1.8)<0.000001)
 plot(vel,t,'k:')
elseif(abs(m-0.0)<0.000001)
 plot(vel,t,'k--')
else
 plot(vel,t,'k')
end

%================
end % of repeat
%================

%---
% plot the solution
%---

figure(1)
xlabel('u/U','fontsize',13)
ylabel('\eta','fontsize',13)
set(gca,'fontsize',13)
axis([0, 1.1, 0, 5])
box on

figure(2)
hold on
plot(mplot,yplot,'k-')
plot(mplot,a1plt,'k--')
plot(mplot,b1plt,'k:')
%plot(mplot,a2plt,'k--')
xlabel('m','fontsize',13)
ylabel('x_3(0),  \alpha,  \beta','fontsize',13)
set(gca,'fontsize',13)
axis([-0.2 2 -0.1 2])
box on

%----------------
% compute and plot
% the Holstein-Bohlen parameter lambda
%----------------

nsize = size(mplot);
n = nsize(2);

lpohl =-0.1;

for i=1:n
 m = mplot(i);
 [Lpohl,lpohl] = fskan_pohl(m,lpohl);
 holbo1(i) = lpohl;
 holbo2(i) = Lpohl;
end

figure(3)
hold on
plot(mplot,lambd,'k-')
plot(mplot,holbo1,'k--')
%plot(mplot,holbo2,'k--')
plot([1],[0.0770356],'or')
plot([-0.2 2.0],[0, 0],'b:')
plot([0 0],[-0.1 0.15],'b:')
xlabel('m','fontsize',13)
ylabel('\lambda','fontsize',13)
set(gca,'fontsize',13)
axis([-0.2 2 -0.1 0.15])
box on
