      program kp_cc

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-------------------------------------------------
c Boundary-layer around a circular cylinder 
c computed by the Karman-Pohlhausen method
c
c See Pozrikidis (1997)
c
c SYMBOLS:
c
c rll:	lower-case lamda
c rlc:	capital lamda
c theta: capital theta (momentum thickness)
c
c Up:  U' (derivative)
c t:  angle theta
c z: rll/Up
c-------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      common/flow1/a,U0

c----------
c constants
c----------

      pi = 3.14159 265358

      Null = 0

c---
c input
c---

      write (6,*)
      write (6,*) ' Enter 0 to quit'
      write (6,*) '       1 to read them from file: kp_cc.dat'
      write (6,*) '       2 to type the data into the keyboard'
      write (6,*) ' ------------------------------------------'
      read  (5,*)   Ienrd

c------------------------
      If(Ienrd.eq.2) then
c------------------------

      write (6,*) 
      write (6,*) " Please enter the cylinder radius"
      write (6,*) " --------------------------------"
      read  (5,*) a

      write (6,*) 
      write (6,*) " Please enter the velocity of the streaming flow"
      write (6,*) " -----------------------------------------------"
      read  (5,*) U0

      write (6,*)
      write (6,*) " Please enter the fluid viscosity"
      write (6,*) " --------------------------------"
      read  (5,*) vs

      write (6,*) 
      write (6,*) " Please enter the kinematic viscosity"
      write (6,*) " ------------------------------------"
      read  (5,*) rnu

      write (6,*) 
      write (6,*) " Enter max integration angle around the cylinder"
      write (6,*) " where the computation will stop"
      write (6,*) 
      write (6,*) " in multiples of pi"
      write (6,*) " ------------------------------------------"

      read  (5,*) angle

      write (6,*) 
      write (6,*) " How many divisions around the cylinder"
      write (6,*) "     up to the last integration point ?"
      write (6,*) " --------------------------------------"
      read  (5,*) Ndiv

c---------
      Else
c---------

      open (2,file="kp_cc.dat")

       read (2,*) a
       read (2,*) U0
       read (2,*) vs
       read (2,*) rnu
       read (2,*) angle
       read (2,*) Ndiv

      close (2)

c-----------
      End If
c-----------


c---
c prepare
c---

      Ndiv1 = Ndiv+1

      angle = angle*pi
      Dt    = angle/Ndiv
      Dx    = a*Dt              ! differential arc length

c---
c output files
c---

      open (1,file='kp_cc.out1')
      open (2,file='kp_cc.out2')
      open (3,file='kp_cc.out3')

      write (1,100) Ndiv1
      write (2,100) Ndiv1
      write (3,100) Ndiv1

c---
c initial values
c---

      rll = 0.0770
      rlc = 7.052
      t   = 0.0
      Up  = 2.0*U0*cos(t)/a
      z   = rll/Up

c---
c Advance rll around the cylinder
c by the second-order Runge-Kutta method
c---

      Do 10 i=1,Ndiv1

c---
c computation of:
c
c  H and S; equations (8.4.21)
c  capital delta; equation (8.4.10)
c  delta star; equation (8.4.13a)
c  sigma_wall; equation (8.4.13c)
c  theta: equation (8.4.20)
c---

      U   = 2.0*U0*sin(t)
      Up  = 2.0*U0*cos(t)/a

      tmp = 37.0-rlc/3.0-5.0*rlc**2/144.0

      H     = 31.5*(3.0-rlc/12.0)/tmp
      S     = (2.0+rlc/6.0)*tmp/315.0
      delc  = sqrt(abs(rnu*rlc/Up))
      dels  = 0.1*delc*(3.0-rlc/12.0)

      sw    = vs*U/delc*(2.0+rlc/6.0)

      If(abs(sw).gt.0.000001) then
       theta = vs*U*S/sw
      Else
       theta = 0.0
      End If

c---
c printing session
c---

      write (6,100)
      write (6,100) i,t/pi,rll,rlc
      write (6,100) i,t/pi,delc,dels,theta,sw
      write (6,100) i,t/pi,S,H

      write (1,100) i,t/pi,rll,rlc
      write (2,100) i,t/pi,delc,dels,theta,sw
      write (3,100) i,t/pi,S,H

c---
c second-order RK
c---
    
      If(i.eq.1) then  
       fv = 0.0
      Else 
       call root (rll,rlc)            ! solve (8.4.18) for rlc
       call phase_vel (t,rlc,rll,fv)  ! compute rhs of (8.4.19) 
      End If

       z_save = z
      fv_save = fv

      t = t+Dt
      z = z + fv*Dx

      Up  = 2.0*U0*cos(t)/a
      rll = z*Up

      call root (rll,rlc)           ! solve (8.4.18) for rlc
      call phase_vel (t,rlc,rll,fv)  ! compute rhs of (8.4.19)

      z = z_save + 0.5*(fv+fv_save)*Dx

      rll = z*Up

  10  Continue

c---
c done
c---

      write (1,100) Null
      write (2,100) Null
      write (3,100) Null
      close (1)
      close (2)
      close (3)

 100  Format (1x,i3,10(1x,f10.5))

      Stop
      End
