      subroutine pois_gs2
     +
     +  (N,M
     +  ,Dx,Dy
     +  ,source
     +  ,IGhia
     +  ,Itermax
     +  ,tol
     +  ,T
     +  )

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c------------------------------------------
c Gauss-Siedel solution of Poisson's equation
c in a rectangular domain
c with Neumann boundary conditions
c
c Equation is applied ONLY at the interior
c nodes
c
c Boundary conditions
c involving inward differences:
c ----------------------------
c
c  df/dx =  pnbcl   at x = ax
c  df/dx = -pnbcr   at x = bx
c  df/dy =  pnbcb   at y = ay
c  df/dy = -pnbct   at y = by
c
c SYMBOLS:
c -------
c
c N=Nx:   Grid size in the x direction
c M=Ny:   Grid size in the y direction
c
c------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension source(16666)

      Dimension     T(0:130,0:130)
      Dimension Tsave(0:130,0:130)

      Dimension pnbcl(129),pnbcr(129)
      Dimension pnbcb(129),pnbct(129)

      Parameter (relax = 1.0D0)

c--------------
c common blocks
c--------------

      common/pnbc/pnbcl,pnbcr,pnbcb,pnbct

c--------
c prepare
c--------

      Na = N-1
      N1 = N+1

      Ma = M-1
      M1 = M+1

      Nodes = Na*Ma          ! number of internal nodes

      Dx2 = 2.0D0*Dx
      Dy2 = 2.0D0*Dy

      Dxs   = Dx**2
      Dys   = Dy**2
      beta  = Dxs/Dys
      beta1 = 2.0D0*(beta+1.0D0)

      open (9,file="error.out")

c------------------------
c Gauss-Siedel iterations
c------------------------

      Do 97 iter=1,itermax

c----------------------------------------------
c Implement the homogeneous Neumann condition
c
c the inhomegeneity is incorporated
c into the RHS
c----------------------------------------------

      If(IGhia.eq.0) then

c---------
c three-point (second-order) biased differences
c---------

       Do j=2,M
         T( 1,j) = (4.0D0*T(2,j)-T(3,j) )/3.0D0
         T(N1,j) = (4.0D0*T(N,j)-T(Na,j))/3.0D0
       End Do

       Do i=2,N
         T(i, 1) = (4.0D0*T(i,2)-T(i, 3))/3.0D0
         T(i,M1) = (4.0D0*T(i,M)-T(i,Ma))/3.0D0
       End Do

c--------------
c Ghia's method:
c
c   the BC is applied midway
c   between the first and second layer
c--------------

      Else If(IGhia.eq.1) then

       Do j=2,M
         T( 1,j) = T(2,j)
         T(N1,j) = T(N,j)
       End Do

       Do i=2,N
         T(i, 1) = T(i,2)
         T(i,M1) = T(i,M)
       End Do

c---------
      Else 
c---------

        write (6,*) " pois_gs2: method ",IGhia," not implemented"
        stop

c----
      End If
c----

c-------------------------------------
c update the internal nodes row-by-row
c-------------------------------------

c-----
c save
c-----

       Do JL=1,M1
        Do IL=1,N1
          Tsave(IL,JL) = T(IL,JL)
        End Do
       End Do

c---
c iterations
c---

      Ic = 0

       Do JL=2,M
        Do IL=2,N
         Ic = Ic+1
         corr = (        T(IL+1,JL)+T(IL-1,JL)
     +           + beta*(T(IL,JL+1)+T(IL,JL-1))
     +            - source(Ic) ) /beta1 - T(IL,JL)
         T(IL,JL) = T(IL,JL) + relax*corr
        End Do
       End Do

c-----------------------
c shift to prevent drift
c-----------------------

       Tref = T(N/2+1,M/2+1)

       Do JL=2,M
        Do IL=2,N
         T(IL,JL) = T(IL,JL)-Tref
        End Do
       End Do

c-------------------------------
c compute the maximum correction
c-------------------------------

       corrmax = 0.0D0

       Do JL=2,M
        Do IL=2,N
         testo = T(IL,JL)-Tsave(IL,JL)
         If(abs(testo).gt.corrmax) corrmax = abs(testo)
        End Do
       End Do

c      write (6,*) iter,T(3,3)

c----
c      write (6,*)
c      write (6,117) iter
c      Do i=2,N
c        write (6,110) (T(i,j),j=2,M)
c      End Do
c----

c      write (6,201) iter,corrmax
       write (9,201) iter,corrmax

       If(corrmax.lt.tol) Go to 98

  97  Continue

c-------
c failed
c-------

      write (6,*) " pois_gs2: Gauss-Siedel iterations did not converge"
      stop

c------------------
c solution computed
c------------------

  98  Continue

c     write (6,*) " pois_gs2: Gauss-Siedel required "
c    +            ,iter," iterations"

      close (9)

c-------------------------------------
c Set boundary values
c using the Neumann boundary condition
c-------------------------------------

c------------------------
      If(IGhia.eq.0) then
c------------------------

       Do j=2,M
        T( 1,j) = ( 4.0D0*T(2,j)-T(3,j) -Dx2*pnbcl(j))/3.0D0
        T(N1,j) = ( 4.0D0*T(N,j)-T(Na,j)-Dx2*pnbcr(j))/3.0D0
       End Do

       Do i=2,N
        T(i, 1) = ( 4.0D0*T(i,2)-T(i,3) -Dy2*pnbcb(i))/3.0D0
        T(i,M1) = ( 4.0D0*T(i,M)-T(i,Ma)-Dy2*pnbct(i))/3.0D0
       End Do

c--------------------------
      Else If(IGhia.eq.1) then
c---------------------------

       Do j=2,M
        T( 1,j) = T(2,j) - Dx * pnbcl(j)
        T(N1,j) = T(N,j) - Dx * pnbcr(j)
       End Do

       Do i=2,N
        T(i, 1) = T(i,2) - Dy * pnbcb(i)
        T(i,M1) = T(i,M) - Dy * pnbct(i)
       End Do

c-----------
      End If
c-----------

c-----
c Done
c-----

 100  Format (50(1x,f8.4))
 101  Format (2(1x,i3),2(1x,f5.3))
 102  Format (" Correction factor           = ",f10.5)
 103  Format (" Residuals = ",2(1x,f15.4))

 110  Format (30(1x,f6.3))
 117  Format ("Iteration: ",i5,1x,f15.8)

 201  Format (1x,i5,3(1x,f15.10))

      Return
      End
