clear all
close all

%==========
% steady Stokes flow in a rectangular cavity
% occupying 0<x<Lx, 0<y<Ly
% computed on a staggered Cartesian grid
%======

Lx = 1.0;
Ly = 0.25;
Ly = 0.50;
Ly = 1.0;

Nx = 24;  % x divisions
Ny = 16;  % y divisions

Nx = 16;  % x divisions
Ny = 16;  % y divisions

Nx = 32;  % x divisions
Ny = 8;  % y divisions

Nx = 32;  % x divisions
Ny = 16;  % y divisions

Nx = 16;  % x divisions
Ny = 32;  % y divisions

visc = 1.0; % viscosity
Vlid = 1.0; % lid velocity

%---
% prepare
%---

Dx = Lx/Nx;
Dy = Ly/Ny;

Dxs = Dx*Dx;
Dys = Dy*Dy;

mats = Nx*Ny+Nx*(Ny-1)+Ny*(Nx-1);  % matrix size

%---
% initialize
%---

u = zeros(Nx+1,Ny+1); % impulse matrices
v = zeros(Nx+1,Ny+1);
p = zeros(Nx+1,Ny+1);

%---
% body force
%---

for i=1:Nx+1
 for j=1:Ny+1
  gx(i,j)=0.0;
  gy(i,j)=0.0;
 end
end

%=======
% compile the coefficient matrix
%=======

Jc = 0;  % counter of impulses

%---
for ipass=1:3  % impulse for u,v,p
%---

 if(ipass==1)
  klim=Nx; llim=Ny+1;   % u vel
 elseif(ipass==2)
  klim = Nx+1; llim = Ny;   % v vel
 elseif(ipass==3)
  klim = Nx+1; llim = Ny+1; % pressure
 end

%---
 for l=2:llim   % scan
  for k=2:klim
  Jc = Jc+1;
%---

  if(ipass==1);  % impulse
   u(k,l) = 1.0;
  elseif(ipass==2);
   v(k,l) = 1.0;
  elseif(ipass==3);
   p(k,l) = 1.0;
  end

%---
% boundary conditions
%---

  for ii=2:Nx
   u(ii,1)   = (-6*u(ii,2)+u(ii,3))/3.0;
   u(ii,Ny+2) = (-6*u(ii,Ny+1)+u(ii,Ny))/3.0;
  end

  for jj=2:Ny
   v(1,jj)   = (-6*v(2,jj)+v(3,jj))/3.0;
   v(Nx+2,jj) = (-6*v(Nx+1,jj)+v(Nx,jj))/3.0;
  end
%---

  Ic=0;  % counter of equations

  for j=2:Ny+1 
   for i=2:Nx   % x Stokes
   Ic = Ic+1;
    MAT(Ic,Jc) = -(p(i+1,j)-p(i,j))/Dx ...
              +visc*(u(i-1,j)-2*u(i,j)+u(i+1,j))/Dxs ...
              +visc*(u(i,j-1)-2*u(i,j)+u(i,j+1))/Dys;
   end
  end
  for j=2:Ny
   for i=2:Nx+1   % y Stokes
    Ic = Ic+1;
    MAT(Ic,Jc) = -(p(i,j+1)-p(i,j))/Dy ...
              +visc*(v(i-1,j)-2*v(i,j)+v(i+1,j))/Dxs ...
              +visc*(v(i,j-1)-2*v(i,j)+v(i,j+1))/Dys;
   end
  end
  for j=2:Ny+1 
   for i=2:Nx+1   % continuity
    Ic = Ic+1;
    MAT(Ic,Jc) = (u(i,j)-u(i-1,j))/Dx+(v(i,j)-v(i,j-1))/Dy;
   end
  end

  u = zeros(Nx+1,Ny+1);  % reset
  v = zeros(Nx+1,Ny+1);
  p = zeros(Nx+1,Ny+1);

  end
 end
end

%====
% right-hand side
%===

for i=1:mats
 rhs(i) = 0.0;
end

 Ic=0;
 for j=2:Ny+1
  for i=2:Nx    % x Stokes
   Ic = Ic+1;
   rhs(Ic) = -gx(i,j);
  end
 end

 for j=2:Ny
  for i=2:Nx+1   % y Stokes
   Ic = Ic+1;
   rhs(Ic) = -gy(i,j);
  end
 end

  Ic = (Nx-1)*(Ny-1);

  for i=2:Nx    % x Stokes
   Ic = Ic+1;
   rhs(Ic) = rhs(Ic)-8.0/3.0*visc*Vlid/Dys;
  end

%====
% set p=0 at the last node
% and find the solution
%====

SOL = rhs(1:mats-1)/MAT(1:mats-1,1:mats-1)';
SOL(mats) = 0.0;

%====
% distribute the solution
%====

Ic = 0;
for j=2:Ny+1
 for i=2:Nx
  Ic = Ic+1;
  uvel(i,j) = SOL(Ic);
 end
end

for j=2:Ny
 for i=2:Nx+1
  Ic = Ic+1;
  vvel(i,j) = SOL(Ic);
 end
end

for j=2:Ny+1
 for i=2:Nx+1
  Ic = Ic+1;
  pressure(i,j) = SOL(Ic);
 end
end

%=====
% interpolate the velocity field at the pressure nodes
%=====

for j=2:Ny+1
 uvel(1,j) = 0;
 uvel(Nx+1,j) = 0;
 for i=2:Nx+1
  uint(i,j) = 0.5*(uvel(i,j)+uvel(i-1,j));
 end
end

for i=2:Nx+1
 vvel(i,1) = 0;
 vvel(i,Ny+1) = 0;
 for j=2:Ny+1
  vint(i,j) = 0.5*(vvel(i,j)+vvel(i,j-1));
 end
end

%====
% prepare a velocity vector plot
%====

figure(1)
hold on
plot([0 Lx Lx 0 0],[0 0 Ly Ly 0],'k')
axis([-0.1*Lx 1.1*Lx, -0.1*Ly 1.1*Ly])
axis equal
xlabel('x','fontsize',15)
ylabel('y','fontsize',15)
set(gca,'fontsize',15)
box on

for j=2:Ny+1
 ylevel=0.5*Dy+(j-2)*Dy;
 for i=2:Nx+1
  xlevel=0.5*Dx+(i-2)*Dx;
  xarrow = 0.75*Dx*uint(i,j)/Vlid;
  yarrow = 0.75*Dx*vint(i,j)/Vlid;
  arrow=arrow_cp(xlevel,ylevel,xarrow,yarrow);
  plot(arrow(:,1),arrow(:,2),'k');
 end
end
